#include "modulo1.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

////////////////////////////////////////////////////////////////////////
// funzione che copia da un file i primi 'dimensione' caratteri nel blocco di memoria 'testo'
// e mette nella variabile 'dimensione' il numero di caratteri letti
////////////////////////////////////////////////////////////////////////
int leggiFile(char * nomefile, int dimensione, char * testo)
{
    // controllo nomefile
	if (!nomefile)		return NULL_POINTER;
	if (!nomefile[0])	return EMPTY_STRING;
	FILE * f = fopen(nomefile, "r");

    // controllo il file
    if (!f) {
        switch (errno) {
            case ENOENT:
                return FILE_MISSING;
            case EACCES:
                return FILE_UNREADABLE;
            default:
                return UNKNOWN_ERROR;
       }
    }
    // controllo la dimensione
    if (dimensione<=0) {
        close(f);
        return BUFFER_TOO_SMALL;
    }
	// controllo il buffer
	if (!testo) {
        close(f);
		return NULL_POINTER;
    }

    // leggo il contenuto (meno un carattere)
	int l = fread(testo, 1, dimensione - 1, f);
	// chiudo il file
	close(f);
	// metto il terminatore di stringa
	testo[l] = '\0';
	// se non ho letto nulla il file  vuoto
    if (!l)     return FILE_EMPTY;
	
	return RESULT_OK;
}

////////////////////////////////////////////////////////////////////////
// funzione che individua le intestazioni della email e le copia nel blocco di memoria 'headers' di lunghezza 'dimh'
////////////////////////////////////////////////////////////////////////
int estraiHeaders(char * email, char * headers, int dimh)
{
	if (!email)		        return NULL_POINTER;
	if (!email[0])		    return EMPTY_STRING;
	if (email[0]=='\n')	    return HEADER_NOT_FOUND;
	if (!headers)		    return NULL_POINTER;
	if (dimh<=0)		    return BUFFER_TOO_SMALL;

    int pos;
	char * nlnl = strstr(email, "\n\n");
	if (!nlnl)
    	pos = strlen(email);
    else
    	pos = nlnl - email + 1; // per tenere uno dei due '\n'
    
	if (strlen(email)-pos < 3) return BODY_NOT_FOUND;
	if (pos >= dimh)	      return BUFFER_TOO_SMALL;

	memcpy(headers, email, pos);
	headers[pos] = '\0';

	return RESULT_OK;
}

////////////////////////////////////////////////////////////////////////
// funzione che estrae il primo header dagli headers e lo copia in primo_header
////////////////////////////////////////////////////////////////////////
int primoHeader(char * headers, char * primo_header, int dim)
{
	if (!headers)		    return NULL_POINTER;
	if (!headers[0])	    return EMPTY_STRING;
	if (headers[0]=='\t' || headers[0]==' ')
                            return BAD_HEADER;
	if (!primo_header)	    return NULL_POINTER;

	// cerco il primo accapo
	// se  seguito da spazio vado avanti a cercare accapi
	char * nl = headers;
	do {
		nl = strstr( nl, "\n" );
		if (!nl) {		// si tratta dell'ultimo header
			nl = headers + strlen(headers);
			break;
		} else
			nl++;
	} while ( nl[0] == ' ' || nl[0] == '\t' );

	int len = nl - headers;
	if (len >= dim)		return BUFFER_TOO_SMALL;
	if (headers[len-1] == '\n') len += -1;
	memcpy(primo_header, headers, len);
	primo_header[len] = '\0';
	
	return RESULT_OK;
}

////////////////////////////////////////////////////////////////////////
// funzione che estrae il tipo ed il valore di uno header
////////////////////////////////////////////////////////////////////////
int parseHeader(char * header, char * tipo, int dimt, char * valore, int dimv)
{
	if (!header)		return NULL_POINTER;
	if (!header[0])		return EMPTY_STRING;
	if (header[0]==' ' || header[0] == '\t' || header[0] == ':')
                        return BAD_HEADER;
	if (!tipo)		    return NULL_POINTER;
	if (dimt <=0)	    return BUFFER_TOO_SMALL;
	if (!valore)		return NULL_POINTER;
	if (dimv <=0)	    return BUFFER_TOO_SMALL;

	int l = strlen(header);
	int i ;
	for (i=0 ; i<l ; i++)
		if (header[i] == ' ' || header[i] == '\t')
			break;

	if (i == 0 || i >= l-1)return BAD_HEADER;
	if (i >= dimt)		return BUFFER_TOO_SMALL;

	memcpy(tipo, header, i);
	tipo[i] = '\0';

	if (l-i-2 >= dimv)	return BUFFER_TOO_SMALL;
	strcpy(valore, header + i + 1);

	return RESULT_OK;
}

////////////////////////////////////////////////////////////////////////
// funzione che estrae la prima parola da una stringa
////////////////////////////////////////////////////////////////////////
int primaParola(char * testo, char * parola, int dim)
{
	if (!testo)	return NULL_POINTER;
	if (!testo[0])	return EMPTY_STRING;
	if (!parola)	return NULL_POINTER;
	if (dim <=0)	return BUFFER_TOO_SMALL;

	int i, j=0, delimlen, wordlen;
	char * lettere = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
	do {
        // salto i delimitatori iniziali
    	testo += strcspn(testo,lettere);
    	// calcolo la lunghezza della parola
        wordlen = strspn(testo,lettere);
        // ignoro le parole di una sola lettera
        if (wordlen==1) testo++;
    } while ( testo[0] && wordlen<2 );
    if (wordlen >= dim) return BUFFER_TOO_SMALL;
	memcpy(parola,testo,wordlen);
	parola[wordlen] = '\0';
	return RESULT_OK;
}

////////////////////////////////////////////////////////////////////////
// funzione che tokenizza un testo assegnandoli un contesto
////////////////////////////////////////////////////////////////////////
int tokenizzaTesto(char * testo, char * contesto, Token tokenArray[], int * dimT)
{
    int i = 0,
        res,
        l = strlen(testo)+1;
    // alloco lo spazio per la parola
    char * parola = calloc(1,l);
    if (!parola)
        return MEM_ALLOC_ERROR;
	char * lettere = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
	do {
        // estraggo la prima parola
		if (res = primaParola(testo, parola, l))  {
            free(parola);
            printf( "leggendo primaParola\n");
            return res;
        }
		int dimp = strlen(parola) ;
		// se  diversa da stringa vuota
		if (dimp) {
            // se siamo negli headers setto il contesto
            if (contesto) {
                tokenArray[i].contesto	=calloc(1,strlen(contesto)+1);
                if (!tokenArray[i].contesto) {
                    free(parola);
                    return MEM_ALLOC_ERROR;
                }
    		    strcpy(tokenArray[i].contesto, contesto);
            } else 
                tokenArray[i].contesto	= NULL;
            // setto la parola
            tokenArray[i].parola	= calloc(1,dimp+1);
            if (!tokenArray[i].parola) { 
                free(parola);
                return MEM_ALLOC_ERROR;
            }
			strcpy(tokenArray[i].parola, parola);
			// incremento il numero di elementi inseriti
		    i++;
		    if (i>= *dimT)  {
                free(parola);
                return BUFFER_TOO_SMALL;
            }
        } else 
            break;
        // salto i delimitatori iniziali e i caratteri singoli
        while (strcspn(testo,lettere) || strspn(testo,lettere) == 1) {
	       testo += strcspn(testo,lettere) + strspn(testo,lettere);
        }
		// avanzo il puntatore al testo per cercare la prossima parola
	    testo += dimp;
	} while ( testo[0] );
	
	// comunico quante parole ho inserito nel tokenArray
	*dimT = i;
    free(parola);
	return RESULT_OK;
}

////////////////////////////////////////////////////////////////////////
// funzione che estrae i primi dimT token dalla email
////////////////////////////////////////////////////////////////////////
int tokenizzaEmail(char * email, Token tokenArray[], int dimT)
{
    int res = 0;
	if (!email)	    return NULL_POINTER;
	if (!email[0])	return EMPTY_STRING;
	if (!tokenArray)return NULL_POINTER;
	if (dimT <=0)	return BUFFER_TOO_SMALL;

	int l = strlen(email) + 1;
	char * headers = calloc(1, l);
	if (!headers)
        return MEM_ALLOC_ERROR;
	// ricordo il puntatore per poi deallocarlo
	char * all_headers = headers;
	
	// estraggo gli headers
	if (res = estraiHeaders(email, headers, l)) {
        free(all_headers);
        return res;
    }

	// estraggo il body della email
	char * body = email + strlen(all_headers);
	if (!strlen(body)) {
        free(all_headers);
        return BODY_NOT_FOUND;
    }

    // alloco spazio per header tipo e parola
	int i = 0;
	char * header  = calloc(1, l);
	char * tipo	   = calloc(1, l);
	char * valore  = calloc(1, l);
    if (! header) {
        free(all_headers);
        return MEM_ALLOC_ERROR;
    }
    if (! tipo ) {
        free(all_headers);
        free(header);
        return MEM_ALLOC_ERROR;
    }
    if (! valore ) {
        free(all_headers);
        free(header);
        free(tipo);
        return MEM_ALLOC_ERROR;
    }

	do {
        // estraggo uno header
		if (res = primoHeader( headers, header, l)) {
            free(all_headers);
            free(header);
            free(tipo);
            free(valore);
//            printf( "in primoHeader\n" );
            return res;
        }
        // ne estraggo il valore e tipo
		int dimh = strlen(header);
		if (res = parseHeader(header, tipo, l, valore, l)) {
            free(all_headers);
            free(header);
            free(tipo);
            free(valore);
//            printf( "in parseHeader\n" );
            return res;
        }
        // ne tokenizzo il valore
		int dimt = strlen(tipo);
		int restano = dimT - i;
        if (res = tokenizzaTesto(valore, tipo, tokenArray + i, &restano)) {
            free(all_headers);
            free(header);
            free(tipo);
            free(valore);
//            printf( "in tokenizzaTesto\n" );
            return res;
        }
        // aggiorno il numero di elementi inseriti nel vettore di tokens
        i += restano;
        // salto i delimitatori iniziali
        headers += strspn(headers,"\n"); // FIXME?
		// avanzo al next header
    	headers += dimh;
    	// se avanza un '\n' lo salto
    	if (headers[0] == '\n')
    	   headers ++;
    }  while ( headers[0] );

	// tokenizzo il body
	int restano = dimT - i;
    if (res = tokenizzaTesto(body, NULL, tokenArray + i, &restano)) {
        free(all_headers);
        free(header);
        free(tipo);
        free(valore);
//        printf( "in tokenizzaTesto\n" );
        return res;
    }
    // aggiorno il numero di elementi nel vettore di tokens
    i += restano;
    // aggiungo un record {NULL, NULL} se c'e' spazio
	if (i<dimT-1) {
		tokenArray[i].contesto = NULL;
		tokenArray[i].parola = NULL;
	}
    free(all_headers);
    free(header);
    free(tipo);
    free(valore);
	return RESULT_OK;
}

////////////////////////////////////////////////////////////////////////
// funzione che stampa i token uno per riga nel formato 'contesto*parola' oppure 'parola'
////////////////////////////////////////////////////////////////////////
int printTokens(Token tokenArray[], int dimT)
{
	int i;
	if (!tokenArray) return NULL_POINTER;
	for (i=0 ; i<dimT && tokenArray[i].parola ; i++)
		if (tokenArray[i].contesto)
			printf("%s*%s\n",tokenArray[i].contesto,tokenArray[i].parola);
		else
			printf("%s\n",tokenArray[i].parola);
	return RESULT_OK;
}


