#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <stdarg.h>
#include <ansidecl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <mcheck.h>
#include "modulo1.h"
#include "testModulo1.h"

#undef printf
#undef freopen
#undef fflush
#undef fclose


char * test_str[] = {
        "TEST_PASSED",
        "TEST_FAILED",
        "ARG_MISSING",
        "NOT_VALID",
        "NOT_IMPLEMENTED",
	"SEG_FAULT",
	"TIMEOUT"
};

char * result_str[] = {
        "RESULT_OK",
        "UNKNOWN_ERROR",
        "NULL_POINTER",
        "EMPTY_STRING",
        "FILE_MISSING",
        "FILE_UNREADABLE",
        "FILE_UNWRITABLE",
        "FILE_EMPTY",
        "BUFFER_TOO_SMALL",
        "HEADER_NOT_FOUND",
        "BODY_NOT_FOUND",
        "BAD_HEADER",
        "MEM_ALLOC_ERROR"
};

#define SEG_FAULT 5
#define TIMEOUT 6
#undef LAST_TEST
#define LAST_TEST TIMEOUT+1

int conteggi[LAST_TEST] = { 0 };

void write_conteggi() {
    int i;
    FILE * f = fopen("stats", "w");
    for (i=0 ; i<LAST_TEST ; i++)
	fprintf(f,"%d ",conteggi[i]);
    fclose(f);
}

void read_conteggi() {
    int i;
    FILE * f = fopen("stats", "r");
    for (i=0 ; i<LAST_TEST ; i++)
	fscanf(f,"%d ",&conteggi[i]);
    fclose(f);
}


void stampa_test(char * funzione, int arg, int tipo, int ris)
{
    read_conteggi();
    conteggi[ris]++;
    write_conteggi();
    switch (ris) {
        case NOT_IMPLEMENTED: 
            fprintf(stderr, arg>0 && tipo == RESULT_OK ? "<TD> </TD>" : "<TD align=\"center\" bgcolor=\"yellow\">NI</TD>");
            break;
        case TEST_FAILED: 
            fprintf(stderr, "<TD align=\"center\" bgcolor=\"red\"><A href=\"log.html#%s_%d_%d\">TF</A></TD>",funzione,tipo,arg);
	        break;
        case TEST_PASSED: 
            fprintf(stderr, "<TD align=\"center\" bgcolor=\"green\">OK</TD>");
	        break;
        case ARG_MISSING: 
            fprintf(stderr, "<TD> </TD>");
	        break;
        case NOT_VALID: 
            fprintf(stderr, "<TD align=\"center\" >NV</TD>");
	        break;
    }
}

char * funzione = NULL;
int argno = 0;
int codice = 0;

#define STAMPA_TEST(f, arg, tipo) \
    {\
    int res ; \
    funzione = "test_" #f ; \
    argno = arg; \
    codice = tipo; \
    int pid = fork(); \
    if(pid == 0){ \
        alarm(5); \
	printf("\n<A NAME=\"%s_%d_%d\"></A><pre>\n", funzione, argno, codice);\
	fprintf(stderr,"<td>"); \
        res = test_ ## f (tipo, arg); \
	fprintf(stderr,"</td>"); \
	stampa_test( #f , arg, tipo, res); \
	printf("</pre>");\
	fflush(stdout); \
        /* alarm(0); */ \
        exit(0); \
    } else{ \
	alarm(10); \
        waitpid(pid, NULL, 0); \
	alarm(0); \
    } \
    }

// funzione che gestisce il segnale prodotto dal SEGMENTATION FAULT
void sighand(int signo){
	if (signo != SIGSEGV)	return;
    read_conteggi();
    conteggi[SEG_FAULT]++;
    write_conteggi();
	fprintf(stderr, "\n<TD><FONT color=\"red\"><B><A href=\"log.html#%s_%d_%d\">Segm. Fault</A></B></FONT></TD>\n", funzione, argno, codice);
	printf("\nTest fallito: Segmentation Fault durante la chiamata a %s(%d,%d)\n</pre>\n", funzione, argno, codice);
	fflush(stdout);
	abort();  // FIXME: non credo di avere altra scelta
}

// funzione che gestisce il segnale prodotto dal SEGMENTATION FAULT
void sigalarm(int signo){
	if (signo != SIGALRM)	return;
    read_conteggi();
    conteggi[TIMEOUT]++;
    write_conteggi();
	fprintf(stderr, "\n<TD><FONT color=\"red\"><B><A href=\"log.html#%s_%d_%d\">Timeout</A></B></FONT></TD>\n", funzione, argno, codice);
	printf("\nTest fallito: Timeout durante la chiamata a %s(%d,%d)\n</pre>\n", funzione, argno, codice);
	fflush(stdout);
	abort();  // FIXME: non credo di avere altra scelta
}

char * funzioni[] = {
    "leggiFile",
    "estraiHeaders",
    "primoHeader",
    "parseHeader",
    "primaParola",
    "tokenizzaEmail",
    "printTokens"
};

#define timeout 3

#define STAMPA_TESTS(funzione) \
    fprintf(stderr, "\n<H2 align=\"center\"> %s </H2>", #funzione); \
    fprintf(stderr, "\n<TABLE border=\"1\" align=\"center\">");\
    fprintf(stderr, "\n<TR><TH>Args:</TH>");\
    for (j=0 ; j<5 ; j++) \
    	fprintf(stderr, "<TH width=\"50\" colspan=\"2\">%d</TH>",j);\
    fprintf(stderr, "</TR>");\
    for (i=RESULT_OK ; i<LAST_CODE ; i++) {\
    	fprintf(stderr, "\n<TR><TH> %s </TH>", result_str[i]);\
    	for (k=0 ; k<5 ; k++) \
            STAMPA_TEST( funzione,  k, i ) \
    	fprintf(stderr, "</TR>");\
    }\
    fprintf(stderr, "\n</TABLE>");


// TODO: gestire i timeout

int main(int argc, char *argv[])
{
    write_conteggi();
    int i, j, k;
    if (argc < 1) 
	return 1;
    // installo un interrupt-handler per catturare i Segmentation Fault
    signal(SIGSEGV, sighand);
    signal(SIGALRM, sigalarm);
    fprintf(stderr, "<HTML><HEAD><TITLE> %s - Modulo 1</TITLE></HEAD><BODY>\n", argv[1]);
    fprintf(stderr, "<H1 align=\"center\"> %s - Modulo 1</H1>\n", argv[1]);
	mtrace();
    STAMPA_TESTS(leggiFile)
    STAMPA_TESTS(estraiHeaders)
    STAMPA_TESTS(primoHeader)
    STAMPA_TESTS(parseHeader)
    STAMPA_TESTS(primaParola)
    STAMPA_TESTS(tokenizzaEmail)
    STAMPA_TESTS(printTokens)
	muntrace();
    fprintf(stderr, "\n</BODY></HTML>", result_str[i]);
    read_conteggi();
    FILE * f = fopen("stats", "w");
    fprintf(f,"(%d/%d)",conteggi[TEST_PASSED],conteggi[TEST_PASSED]+conteggi[TEST_FAILED]+conteggi[SEG_FAULT]+conteggi[TIMEOUT]);
    fclose(f);
//  system("PAUSE");	
    return 0;
}


FILE * my_stdout = NULL;
FILE * old_stdout = NULL;

FILE * my_freopen(char * filename, char * mode, FILE * file) {
    if (my_stdout) {
        fflush(my_stdout);
        fclose(my_stdout);
        my_stdout = NULL;
        old_stdout = NULL;
	return stdout;
    } else {
        my_stdout = fopen(filename, mode);
        old_stdout = stdout;
	return my_stdout;
    }
}

int my_fflush(FILE * file) {
    if (file == old_stdout)
        return fflush(my_stdout);
    else
        return fflush(file);
}

int my_printf VPARAMS ((const char *format, ...))
{
    int result;
    VA_OPEN (ap, format);
    VA_FIXEDARG (ap, const char *, format);
    if (my_stdout) {
        result = vfprintf(my_stdout,format,ap);
    } else {
        result = vfprintf(stdout,format,ap);
    }
    VA_CLOSE (ap);
    return result;
}

int my_fclose(FILE * file) {
    if (file == old_stdout) {
        return 0;
    } else {
        return fclose(file);
    }
}
