#include "testModulo1.h"
#include <stdlib.h>
#include <stdio.h>

#ifdef WIN32
#warning "compiling for Windows"
#   define CONSOLE "CON"
#else
#warning "compiling for Unix"
#   define CONSOLE "/dev/tty"
#endif

#ifndef TRUE
#   define TRUE 1
#endif

#ifndef FALSE
#   define FALSE 0
#endif

#ifndef NULL
#   define NULL 0
#endif

extern char * result_str[];

////////////////////////////////////////////////////////////////////////////////////////////////////////////
void explain_int(char * funzione, char * stringa, int argno, int tipo, int res, char * explanation,
                int expected, int obtained)
{
    printf("\n\
Test failed\n\
    Funzione chiamata  : %s\n\
    Argomento esaminato: %d\n\
    Argomento passato  : '%s'\n\
    Risultato atteso   : %s\n\
    Risultato ottenuto : %s\n\
    Il valore ottenuto doveva essere '%d' invece che '%d'\n\
    Possibile spiegazione: %s\n",
    funzione, argno, stringa, result_str[tipo], result_str[res], expected, obtained, explanation);
}

void explain_str(char * funzione, char * stringa, int argno, int tipo, int res, char * explanation,
                char * expected, char * obtained)
{
    printf("\n\
Test failed\n\
    Funzione chiamata  : %s\n\
    Argomento esaminato: %d\n\
    Argomento passato  : '%s'\n\
    Risultato atteso   : %s\n\
    Risultato ottenuto : %s\n\
    Il testo ottenuto doveva essere '%s'\n\
                         invece che '%s'\n\
    Possibile spiegazione: %s\n",
    funzione, argno, stringa, result_str[tipo], result_str[res], expected, obtained, explanation);
}

////////////////////////////// macro per definire i dispatcher dei singoli test ////////////////////////

#define TEST_CASE(funz,i,n) \
	case i: return (i<n ? test_ ## funz ## i (tipo_di_test) : ARG_MISSING);

#define TEST_ARGS(funz,n) \
int test_ ## funz (int tipo_di_test, int argomento) {\
	switch (argomento) { \
		TEST_CASE(funz,0,n) \
		TEST_CASE(funz,1,n) \
		TEST_CASE(funz,2,n) \
		TEST_CASE(funz,3,n) \
		TEST_CASE(funz,4,n) \
		default:\
			return ARG_MISSING;\
	}\
}

//////////////// definizione dei dispatcher per i singoli test ////////////////////////////////
TEST_ARGS(leggiFile,3)
TEST_ARGS(estraiHeaders,3)
TEST_ARGS(primoHeader,3)
TEST_ARGS(parseHeader,5)
TEST_ARGS(primaParola,3)
TEST_ARGS(tokenizzaEmail,3)
TEST_ARGS(printTokens,2)

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
int test_leggiFile0(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
        {
            // provo a leggere un file conosciuto
            char * email = calloc(1,10);
            char * expected = "From: and";
	    int tested=0;
	    int failed=0;
            int res = leggiFile("email1.txt",10,email);
	    tested++;
            if (res != RESULT_OK) {
                explain_int("leggiFile(\"email1.txt\",10,email)",
                        "email1.txt",
                        0,RESULT_OK,res,
                        "La funzione ha dato un errore imprevisto.",
                        RESULT_OK,res);
		failed++;
            }
            // controllo che vada bene
	    tested++;
            if (strcmp(email,expected)) {
                explain_str("leggiFile(\"email1.txt\",10,email)",
                        "email1.txt",
                        0,RESULT_OK,res,
                        "Il testo letto e' diverso da quanto atteso.",
                        expected, email);
		failed++;
            }
            free(email);
	    fprintf(stderr, "(%d/%d)", tested-failed, tested);
	    if (failed) return TEST_FAILED;
            return TEST_PASSED;
        }
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            // provo a passare NULL
            char * email = calloc(1,10);
            int res = leggiFile(NULL,10,email);
            if (res != NULL_POINTER) {
                explain_int("leggiFile(NULL,10,email)",
                        NULL,
                        0,NULL_POINTER,res,
                        "La funzione non controlla se il primo argomento e' NULL.",
                        NULL_POINTER,res);
                free(email);
                return TEST_FAILED;
            }
            free(email);
            return TEST_PASSED;
        }
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        {
            // provo a passare ""
            char * email = calloc(1,10);
            int res = leggiFile("",10,email);
            if (res != EMPTY_STRING) {
                explain_int("leggiFile(\"\",10,email)",
                        "",
                        0,EMPTY_STRING,res,
                        "La funzione non controlla se il primo argomento e' ''.",
                        EMPTY_STRING,res);
                free(email);
                return TEST_FAILED;
            }
            free(email);
            return TEST_PASSED;
        }
        case FILE_MISSING:	    // il file non esiste
        {
            // provo a passare un nome di file che non esiste
            char * email = calloc(1,10);
            int res = leggiFile("file-missing",10,email);
            if (res != FILE_MISSING) {
                explain_int("leggiFile(\"file-missing\",10,email)",
                        "file-missing",
                        0,FILE_MISSING,res,
                        "La funzione non controlla se il file non esiste.",
                        FILE_MISSING,res);
                free(email);
                return TEST_FAILED;
            }
            free(email);
            return TEST_PASSED;
        }
        case FILE_UNREADABLE:	// il file non e' leggibile
        {
            // provo a passare un nome di file che non esiste
            char * email = calloc(1,10);
            //TODO: creare un file non leggibile
            int res = leggiFile("file-unreadable",10,email);
            if (res != FILE_UNREADABLE) {
                explain_int("leggiFile(\"file-unreadable\",10,email)",
                        "file-unreadable",
                        0,FILE_UNREADABLE,res,
                        "La funzione non controlla se il file non e' leggibile.",
                        FILE_UNREADABLE,res);
                free(email);
                return TEST_FAILED;
            }
            free(email);
            return TEST_PASSED;
        }
        case FILE_EMPTY:	    // il file e' vuoto
        {
            // provo a passare un nome di file che non esiste
            char * email = calloc(1,10);
            //TODO: creare un file vuoto
            int res = leggiFile("file-empty",10,email);
            if (res != FILE_EMPTY) {
                explain_int("leggiFile(\"file-empty\",10,email)",
                        "file-empty",
                        0,FILE_EMPTY,res,
                        "La funzione non controlla se il file e' vuoto.",
                        FILE_EMPTY,res);
                free(email);
                return TEST_FAILED;
            }
            free(email);
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_leggiFile1(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        {
            // provo a passare -1
            char * email = calloc(1,10);
            int res = leggiFile("email1.txt",-1,email);
            if (res != BUFFER_TOO_SMALL) {
                explain_int("leggiFile(\"email1.txt\",-1,email)",
                        "-1",
                        0,BUFFER_TOO_SMALL,res,
                        "La funzione non controlla se il secondo argomento e' negativo o nullo.",
                        BUFFER_TOO_SMALL,res);
                free(email);
                return TEST_FAILED;
            }
            free(email);
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_leggiFile2(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            // provo a passare NULL
            int res = leggiFile("email1.txt",10,NULL);
            if (res != NULL_POINTER) {
                explain_int("leggiFile(\"email1.txt\",10,NULL)",
                        NULL,
                        2,NULL_POINTER,res,
                        "La funzione non controlla se il terzo argomento e' NULL.",
                        NULL_POINTER,res);
                return TEST_FAILED;
            }
            return TEST_PASSED;
        }
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
int test_estraiHeaders0(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
        {
            // provo a passare una email buona
            char * email =
"From\t10.0.0.1\n\
    10.0.0.2\n\
To:\tbabbo_natale@northpole.org\n\
Subject: Happy Easter!\n\
\n\
Buona Pasqua da Andrea\n\
\n";
            char * expected =
"From\t10.0.0.1\n\
    10.0.0.2\n\
To:\tbabbo_natale@northpole.org\n\
Subject: Happy Easter!\n";
	    int tested=0;
	    int failed=0;
            char * headers = calloc(1,10000);
            int res = estraiHeaders(email,headers,10000);
	    tested++;
            if (res != RESULT_OK) {
                explain_int("estraiHeaders(\"...\",\"...\",10000)",
                        email,
                        0,RESULT_OK,res,
                        "La funzione ha dato un errore imprevisto.",
                        RESULT_OK,res);
		failed++;
            }
	    tested++;
            if (strcmp(expected,headers)) {
                explain_str("estraiHeaders(\"...\",\"...\",10000)",
                        email,
                        0,RESULT_OK,res,
                        "Gli header estratti non sono quelli attesi (con un solo '\\n' in fondo).",
                        expected,headers);
		failed++;
            }
            free(headers);
	    fprintf(stderr, "(%d/%d)", tested-failed, tested);
	    if (failed) return TEST_FAILED;
            return TEST_PASSED;
        }
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            // provo a passare NULL
            char * headers = calloc(1,10000);
            int res = estraiHeaders(NULL,headers,10000);
            if (res != NULL_POINTER) {
                explain_int("estraiHeaders(NULL,\"...\",10000)",
                        NULL,
                        0,NULL_POINTER,res,
                        "La funzione non controlla se il primo argomento e' NULL.",
                        NULL_POINTER,res);
                free(headers);
                return TEST_FAILED;
            }
            free(headers);
            return TEST_PASSED;
        }
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        {
            // provo a passare ''
            char * headers = calloc(1,10000);
            int res = estraiHeaders("",headers,10000);
            if (res != EMPTY_STRING) {
                explain_int("estraiHeaders(\"\",\"...\",10000)",
                        "",
                        0,EMPTY_STRING,res,
                        "La funzione non controlla se il primo argomento e' ''.",
                        EMPTY_STRING,res);
                free(headers);
                return TEST_FAILED;
            }
            free(headers);
            return TEST_PASSED;
        }
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        {
            // provo a passare '\npippo\n\n'
            char * headers = calloc(1,10000);
            int res = estraiHeaders("\npippo\n\n",headers,10000);
            if (res != HEADER_NOT_FOUND) {
                explain_int("estraiHeaders(\"\\npippo\\n\\n\",\"...\",10000)",
                        "\npippo\n\n",
                        0,HEADER_NOT_FOUND,res,
                        "La funzione non si accorge che l'email non ha header.",
                        HEADER_NOT_FOUND,res);
                free(headers);
                return TEST_FAILED;
            }
            free(headers);
            return TEST_PASSED;
        }
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
/*
        {
            // provo a passare '\npippo\n\n'
            char * headers = calloc(1,10000);
            int res = estraiHeaders("pippo\n\n",headers,10000);
            if (res != BODY_NOT_FOUND) {
                explain_int("estraiHeaders(\"pippo\\n\\n\",\"...\",10000)",
                        "pippo\n\n",
                        0,BODY_NOT_FOUND,res,
                        "La funzione non si accorge che l'email non ha body.",
                        BODY_NOT_FOUND,res);
                free(headers);
                return TEST_FAILED;
            }
            free(headers);
            return TEST_PASSED;
        }
*/
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_estraiHeaders1(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            // provo a passare una email buona
            char * email =
"From\t10.0.0.1\n\
    10.0.0.2\n\
To:\tbabbo_natale@northpole.org\n\
Subject: Happy Easter!\n\
\n\
Buona Pasqua da Andrea\n";
            // provo a passare NULL
            int res = estraiHeaders(email,NULL,10000);
            if (res != NULL_POINTER) {
                explain_int("estraiHeaders(\"...\",NULL,10000)",
                        NULL,
                        1,NULL_POINTER,res,
                        "La funzione non controlla se il secondo argomento e' NULL.",
                        NULL_POINTER,res);
                return TEST_FAILED;
            }
            return TEST_PASSED;
        }
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_estraiHeaders2(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        {
            // provo a passare una email buona
            char * email =
"From\t10.0.0.1\n\
    10.0.0.2\n\
To:\tbabbo_natale@northpole.org\n\
Subject: Happy Easter!\n\
\n\
Buona Pasqua da Andrea\n";
            char * headers = calloc(1,10000);
            int res = estraiHeaders(email,headers,-1);
            if (res != BUFFER_TOO_SMALL) {
                explain_int("estraiHeaders(\"...\",\"...\",-1)",
                        "-1",
                        2,BUFFER_TOO_SMALL,res,
                        "La funzione non controlla se il terzo argomento e' negativo o nullo.",
                        BUFFER_TOO_SMALL,res);
                free(headers);
                return TEST_FAILED;
            }
            res = estraiHeaders(email,headers,10);
            if (res != BUFFER_TOO_SMALL) {
                explain_int("estraiHeaders(\"...\",\"...\",10)",
                        "10",
                        2,BUFFER_TOO_SMALL,res,
                        "La funzione non controlla se il terzo argomento e' troppo piccolo.",
                        BUFFER_TOO_SMALL,res);
                free(headers);
                return TEST_FAILED;
            }
            free(headers);
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
int test_primoHeader0(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
        {
            // provo a passare una email buona
            char * headers =
"From\t10.0.0.1\n\
    10.0.0.2\n\
To:\tbabbo_natale@northpole.org\n\
Subject: Happy Easter!\n";
            char * expected =
"From\t10.0.0.1\n\
    10.0.0.2";
            char * header = calloc(1,10000);
	    int tested=0;
	    int failed=0;
            int res = primoHeader(headers,header,10000);
	    tested++;
            if (res != RESULT_OK) {
                explain_int("primoHeader(\"...\",\"...\",10000)",
                        headers,
                        2,RESULT_OK,res,
                        "La funzione ha dato un errore imprevisto.",
                        RESULT_OK,res);
		failed++;
            }
	    tested++;
            if (strcmp(expected,header)) {
                explain_str("primoHeader(\"...\",\"...\",10000)",
                        headers,
                        2,RESULT_OK,res,
                        "Lo header estratto non e' quello atteso (ad. es. senza '\\n' in fondo).",
                        expected,header);
		failed++;
            }
            free(header);
	    fprintf(stderr, "(%d/%d)", tested-failed, tested);
	    if (failed) return TEST_FAILED;
            return TEST_PASSED;
        }
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            // provo a passare NULL
            char * header = calloc(1,10000);
            int res = primoHeader(NULL,header,10000);
            if (res != NULL_POINTER) {
                explain_int("primoHeader(NULL,\"...\",10000)",
                        NULL,
                        0,NULL_POINTER,res,
                        "La funzione non controlla se il primo argomento e' NULL.",
                        NULL_POINTER,res);
                free(header);
                return TEST_FAILED;
            }
            free(header);
            return TEST_PASSED;
        }
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        {
            // provo a passare ''
            char * header = calloc(1,10000);
            int res = primoHeader("",header,10000);
            if (res != EMPTY_STRING) {
                explain_int("primoHeader(\"\",\"...\",10000)",
                        "",
                        0,EMPTY_STRING,res,
                        "La funzione non controlla se il primo argomento e' ''.",
                        EMPTY_STRING,res);
                free(header);
                return TEST_FAILED;
            }
            free(header);
            return TEST_PASSED;
        }
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
/*
        {
            // provo a passare ''
            char * header = calloc(1,10000);
            int res = primoHeader(" From\t10.0.0.1\n",header,10000);
            if (res != BAD_HEADER) {
                explain_int("primoHeader(\" From\\t10.0.0.1\\n\",\"...\",10000)",
                        " From\t10.0.0.1\n",
                        0,BAD_HEADER,res,
                        "La funzione non si accorge che lo header inizia con spazio.",
                        BAD_HEADER,res);
                free(header);
                return TEST_FAILED;
            }
            free(header);
            return TEST_PASSED;
        }
*/
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_primoHeader1(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            char * headers =
"From\t10.0.0.1\n\
    10.0.0.2\n\
To:\tbabbo_natale@northpole.org\n\
Subject: Happy Easter!\n";
            // provo a passare NULL
            int res = primoHeader(headers,NULL,10000);
            if (res != NULL_POINTER) {
                explain_int("primoHeader(\"...\",NULL,10000)",
                        NULL,
                        1,NULL_POINTER,res,
                        "La funzione non controlla se il secondo argomento e' NULL.",
                        NULL_POINTER,res);
                return TEST_FAILED;
            }
            return TEST_PASSED;
        }
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_primoHeader2(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        {
            char * headers =
"From\t10.0.0.1\n\
    10.0.0.2\n\
To:\tbabbo_natale@northpole.org\n\
Subject: Happy Easter!\n";
            // provo a passare -1
            char * header = calloc(1,10000);
            int res = primoHeader(headers,header,-1);
            if (res != BUFFER_TOO_SMALL) {
                explain_int("primoHeader(\"...\",\"...\",-1)",
                        "-1",
                        2,BUFFER_TOO_SMALL,res,
                        "La funzione non controlla se il terzo argomento e' negativo o nullo.",
                        BUFFER_TOO_SMALL,res);
                free(header);
                return TEST_FAILED;
            }
            res = primoHeader(headers,header,5);
            if (res != BUFFER_TOO_SMALL) {
                explain_int("primoHeader(\"...\",\"...\",5)",
                        "5",
                        2,BUFFER_TOO_SMALL,res,
                        "La funzione non controlla se il terzo argomento e' troppo piccolo.",
                        BUFFER_TOO_SMALL,res);
                free(header);
                return TEST_FAILED;
            }
            free(header);
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
int test_parseHeader0(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
        {
            char * header[]  = { 
                "From\t10.0.0.1\n\t10.0.0.2\n",
                "From 10.0.0.1\n\t10.0.0.2\n",
                "From        10.0.0.1\n\t10.0.0.2\n",
                "From:        10.0.0.1\n\t10.0.0.2\n",
                };
            char * expectedt[] = { 
                "From",
                "From",
                "From",
                "From:",
                };
            char * expectedv[] = { 
                "10.0.0.1\n\t10.0.0.2\n",
                "10.0.0.1\n\t10.0.0.2\n",
                "       10.0.0.1\n\t10.0.0.2\n",
                "       10.0.0.1\n\t10.0.0.2\n",
                };
	    int tested=0;
	    int failed=0;
            char * tipo = calloc(1,10000);
            char * valore = calloc(1,10000);
            int res,i;
            for (i=0 ; i<4 ; i++) {
                res = parseHeader(header[i],tipo,10000,valore,10000);
	    	tested++;
                if (res != RESULT_OK) {
                    explain_int("parseHeader(\"...\",\"...\",10000,\"...\",10000)",
                            header[i],
                            0,RESULT_OK,res,
                            "La funzione ha tornato un errore imprevisto.",
                            RESULT_OK,res);
		    failed++;
                }
	    	tested++;
                if (strcmp(tipo,expectedt[i])) {
                    explain_str("parseHeader(\"...\",\"...\",10000,\"...\",10000)",
                            header[i],
                            0,RESULT_OK,res,
                            "La funzione ha sbagliato nell'individuare il tipo.",
                            expectedt[i],tipo);
		    failed++;
                }
	    	tested++;
                if (strcmp(valore,expectedv[i])) {
                    explain_str("parseHeader(\"...\",\"...\",10000,\"...\",10000)",
                            header[i],
                            0,RESULT_OK,res,
                            "La funzione ha sbagliato nell'individuare il valore.",
                            expectedv[i],valore);
		    failed++;
                }
            }
            free(tipo);
            free(valore);
	    fprintf(stderr, "(%d/%d)", tested-failed, tested);
	    if (failed) return TEST_FAILED;
            return TEST_PASSED;
        }
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            char * tipo = calloc(1,10000);
            char * valore = calloc(1,10000);
            // provo a passare NULL
            int res = parseHeader(NULL,tipo,10000,valore,10000);
            if (res != NULL_POINTER) {
                explain_int("parseHeader(NULL,\"...\",10000,\"...\",10000)",
                        NULL,
                        0,NULL_POINTER,res,
                        "La funzione non controlla se il primo argomento e' NULL.",
                        NULL_POINTER,res);
                free(tipo);
                free(valore);
                return TEST_FAILED;
            }
            free(tipo);
            free(valore);
            return TEST_PASSED;
        }
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        {
            char * tipo = calloc(1,10000);
            char * valore = calloc(1,10000);
            // provo a passare ''
            int res = parseHeader("",tipo,10000,valore,10000);
            if (res != EMPTY_STRING) {
                explain_int("parseHeader(\"\",\"...\",10000,\"...\",10000)",
                        "",
                        0,EMPTY_STRING,res,
                        "La funzione non controlla se il primo argomento e' ''.",
                        EMPTY_STRING,res);
                free(tipo);
                free(valore);
                return TEST_FAILED;
            }
            free(tipo);
            free(valore);
            return TEST_PASSED;
        }
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        {
            char * header[]  = { 
                " From\t10.0.0.1\n\t10.0.0.2\n",
                "From10.0.0.1\n10.0.0.2\n",
                ": pippo\n",
                "From!\"$%/$()//()&/=&%%GAE%&W$%pipppo",
                };
            char * tipo = calloc(1,10000);
            char * valore = calloc(1,10000);
            int res,i;
            for (i=0 ; i<4 ; i++) {
                res = parseHeader(header[i],tipo,10000,valore,10000);
                if (res != BAD_HEADER) {
                    explain_int("parseHeader(\"...\",\"...\",10000,\"...\",10000)",
                            header[i],
                            0,BAD_HEADER,res,
                            "La funzione non ha capito che l'header e' errato.",
                            BAD_HEADER,res);
                    free(tipo);
                    free(valore);
                    return TEST_FAILED;
                }
            }
            free(tipo);
            free(valore);
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_parseHeader1(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            char * header =
"From\t10.0.0.1\n\
    10.0.0.2";
            char * valore = calloc(1,10000);
            // provo a passare NULL
            int res = parseHeader(header,NULL,10000,valore,10000);
            if (res != NULL_POINTER) {
                explain_int("parseHeader(\"...\",NULL,10000,\"...\",10000",
                        NULL,
                        1,NULL_POINTER,res,
                        "La funzione non controlla se il secondo argomento e' NULL.",
                        NULL_POINTER,res);
                free(valore);
                return TEST_FAILED;
            }
            free(valore);
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_parseHeader2(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        {
            char * header =
"From\t10.0.0.1\n\
    10.0.0.2";
            char * tipo = calloc(1,10000);
            char * valore = calloc(1,10000);
            // provo a passare -1
            int res = parseHeader(header,tipo,-1,valore,10000);
            if (res != BUFFER_TOO_SMALL) {
                explain_int("parseHeader(\"...\",\"...\",-1,\"...\",10000",
                        "-1",
                        2,BUFFER_TOO_SMALL,res,
                        "La funzione non controlla se il terzo argomento e' negativo o nullo.",
                        BUFFER_TOO_SMALL,res);
                free(tipo);
                free(valore);
                return TEST_FAILED;
            }
            // provo a passare 3
            res = parseHeader(header,tipo,3,valore,10000);
            if (res != BUFFER_TOO_SMALL) {
                explain_int("parseHeader(\"...\",\"...\",3,\"...\",10000",
                        "3",
                        2,BUFFER_TOO_SMALL,res,
                        "La funzione non controlla se il terzo argomento e' troppo piccolo.",
                        BUFFER_TOO_SMALL,res);
                free(tipo);
                free(valore);
                return TEST_FAILED;
            }
            free(tipo);
            free(valore);
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_parseHeader3(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            char * header =
"From\t10.0.0.1\n\
    10.0.0.2";
            char * tipo = calloc(1,10000);
            // provo a passare NULL
            int res = parseHeader(header,tipo,10000,NULL,10000);
            if (res != NULL_POINTER) {
                explain_int("parseHeader(\"...\",\"...\",10000,NULL,10000)",
                        NULL,
                        3,NULL_POINTER,res,
                        "La funzione non controlla se il quarto argomento e' NULL.",
                        NULL_POINTER,res);
                free(tipo);
                return TEST_FAILED;
            }
            free(tipo);
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_parseHeader4(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        {
            char * header =
"From\t10.0.0.1\n\
    10.0.0.2";
            char * tipo = calloc(1,10000);
            char * valore = calloc(1,10000);
            // provo a passare -1
            int res = parseHeader(header,tipo,10000,valore,-1);
            if (res != BUFFER_TOO_SMALL) {
                explain_int("parseHeader(\"...\",\"...\",10000,\"...\",-1",
                        "-1",
                        4,BUFFER_TOO_SMALL,res,
                        "La funzione non controlla se il quinto argomento e' negativo o nullo.",
                        BUFFER_TOO_SMALL,res);
                free(tipo);
                free(valore);
                return TEST_FAILED;
            }
            // provo a passare 3
            res = parseHeader(header,tipo,10000,valore,3);
            if (res != BUFFER_TOO_SMALL) {
                explain_int("parseHeader(\"...\",\"...\",10000,\"...\",3",
                        "3",
                        4,BUFFER_TOO_SMALL,res,
                        "La funzione non controlla se il quinto argomento e' troppo piccolo.",
                        BUFFER_TOO_SMALL,res);
                free(tipo);
                free(valore);
                return TEST_FAILED;
            }
            free(tipo);
            free(valore);
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
int test_primaParola0(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
        {
            int i;
            char * testo[] = {
                "la vecchia con la borsa",
                "\"&%%&&$%%&salta90$il fosso",
                "senzrincors",
                };
            char * expected[] = {
                "la",
                "salta90",
                "senz",
                };
            char * parola = calloc(1,10000);
	    int tested=0;
	    int failed=0;
            for (i=0 ; i<3 ; i++) {
                // provo a passare NULL
                int res = primaParola(testo[i],parola,10000);
	    	tested++;
                if (res != RESULT_OK) {
                    explain_int("primaParola(\"...\",\"...\",10000)",
                            testo[i],
                            0,RESULT_OK,res,
                            "La funzione ha riportato un codice di errore imprevisto.",
                            RESULT_OK,res);
		    failed++;
                }
	    	tested++;
                if (strcmp(expected[i],parola)) {
                    explain_str("primaParola(\"...\",\"...\",10000)",
                            testo[i],
                            0,RESULT_OK,res,
                            "La funzione non ha riconosciuto la prima parola.",
                            expected[i],parola);
		    failed++;
                }
            }
            free(parola);
	    fprintf(stderr, "(%d/%d)", tested-failed, tested);
	    if (failed) return TEST_FAILED;
            return TEST_PASSED;
        }
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            char * parola = calloc(1,10000);
            // provo a passare NULL
            int res = primaParola(NULL,parola,10000);
            if (res != NULL_POINTER) {
                explain_int("primaParola(NULL,\"...\",10000)",
                        NULL,
                        0,NULL_POINTER,res,
                        "La funzione non controlla se il primo argomento e' NULL.",
                        NULL_POINTER,res);
                free(parola);
                return TEST_FAILED;
            }
            free(parola);
            return TEST_PASSED;
        }
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        {
            char * parola = calloc(1,10000);
            // provo a passare NULL
            int res = primaParola("",parola,10000);
            if (res != EMPTY_STRING) {
                explain_int("parseHeader(\"\",\"...\",10000)",
                        "",
                        0,EMPTY_STRING,res,
                        "La funzione non controlla se il primo argomento e' ''.",
                        EMPTY_STRING,res);
                free(parola);
                return TEST_FAILED;
            }
            free(parola);
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_primaParola1(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            char * testo = "la vecchia con la borsa salta il fosso senza rincorsa";
            // provo a passare NULL
            int res = primaParola(testo,NULL,10000);
            if (res != NULL_POINTER) {
                explain_int("primaParola(\"...\",NULL,10000)",
                        NULL,
                        1,NULL_POINTER,res,
                        "La funzione non controlla se il secondo argomento e' NULL.",
                        NULL_POINTER,res);
                return TEST_FAILED;
            }
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_primaParola2(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        {
            char * testo = " !%\"$%!$%1254387687From\t10.0.0.1\n    10.0.0.2";
            char * parola = calloc(1,10000);
            // provo a passare -1
            int res = primaParola(testo,parola,-1);
            if (res != BUFFER_TOO_SMALL) {
                explain_int("primaParola(\"...\",\"...\",-1)",
                        "-1",
                        2,BUFFER_TOO_SMALL,res,
                        "La funzione non controlla se il terzo argomento e' negativo o nullo.",
                        BUFFER_TOO_SMALL,res);
                free(parola);
                return TEST_FAILED;
            }
            // provo a passare 3
            res = primaParola(testo,parola,3);
            if (res != BUFFER_TOO_SMALL) {
                explain_int("primaParola(\"...\",\"...\",3)",
                        "3",
                        2,BUFFER_TOO_SMALL,res,
                        "La funzione non controlla se il terzo argomento e' troppo piccolo.",
                        BUFFER_TOO_SMALL,res);
                free(parola);
                return TEST_FAILED;
            }
            free(parola);
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}


void tokens_free(Token tokens[], int dim) {
    int i;
    for (i=0 ; i<dim ; i++) {
	if (tokens[i].contesto) {
	    free(tokens[i].contesto);
	    tokens[i].contesto=NULL;
	}
	if (tokens[i].parola) {
	    free(tokens[i].parola);
	    tokens[i].parola=NULL;
	}
    }	
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
int test_tokenizzaEmail0(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
        {
            char * email =
"From\t10.0.0.1\n\
    10.0.0.2\n\
To:\tbabbo_natale@northpole.org\n\
Subject: Happy Easter!\n\
\n\
Buona Pasqua da Andrea\n\
\n";
            Token expected[] = { 
                {"10",          "From"},
                {"10",          "From"},
                {"babbo",       "To:"},
                {"natale",      "To:"},
                {"northpole",   "To:"},
                {"org",         "To:"},
                {"Happy",       "Subject:"},
                {"Easter",      "Subject:"},
                {"Buona",       NULL},
                {"Pasqua",      NULL},
                {"da",          NULL},
                {"Andrea",      NULL},
                {NULL,          NULL},
                };
            Token tokArray[10000] = { 0 };
	    memset(tokArray, 0, sizeof(tokArray));
	    int tested=0;
	    int failed=0;
            int res = tokenizzaEmail(email,tokArray,10000);
	    tested++;
            if (res != RESULT_OK) {
                explain_int("tokenizzaEmail(\"...\",tokArray,10000)",
                        email,
                        0,RESULT_OK,res,
                        "La funzione ha avuto un errore imprevisto.",
                        RESULT_OK,res);
		failed++;
            }
            int i;
            for (i=0 ; i<10000 && expected[i].parola; i++)
            {
	    	tested++;
                if (expected[i].contesto && (!tokArray[i].contesto || strcmp(expected[i].contesto,tokArray[i].contesto))) {
                    explain_str("tokenizzaEmail(\"...\",tokArray,10000)",
                            email,
                            0,RESULT_OK,res,
                            "La funzione ha tornato una serie di token errati (uno differisce per il contesto).",
                            expected[i].contesto,tokArray[i].contesto);
		    failed++;
                }
	    	tested++;
                if (!tokArray[i].parola || strcmp(expected[i].parola,tokArray[i].parola)) {
                    explain_str("tokenizzaEmail(\"...\",tokArray,10000)",
                            email,
                            0,RESULT_OK,res,
                            "La funzione ha tornato una serie di token errati (uno differisce per la parola).",
                            expected[i].parola,tokArray[i].parola);
		    failed++;
                }
            }
	    tested++;
            if (tokArray[i].parola || tokArray[i].contesto) {
                explain_str("tokenizzaEmail(\"...\",tokArray,10000)",
                        email,
                        0,RESULT_OK,res,
                        "La funzione ha tornato una serie di token piu' lunga di quanto atteso.",
                        expected[i].parola,tokArray[i].parola);
		    failed++;
            }
	    tokens_free(tokArray,10000);
	    fprintf(stderr, "(%d/%d)", tested-failed, tested);
	    if (failed) return TEST_FAILED;
            return TEST_PASSED;
        }
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            // provo a passare NULL
            Token tokArray[10000] = { 0 };
            int res = tokenizzaEmail(NULL,tokArray,10000);
            if (res != NULL_POINTER) {
                explain_int("tokenizzaEmail(NULL,tokArray,10000)",
                        NULL,
                        0,NULL_POINTER,res,
                        "La funzione non controlla se il primo argomento e' NULL.",
                        NULL_POINTER,res);
		tokens_free(tokArray,10000);
                return TEST_FAILED;
            }
	    tokens_free(tokArray,10000);
            return TEST_PASSED;
        }
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        {
            // provo a passare ""
            Token tokArray[10000] = { 0 };
            int res = tokenizzaEmail("",tokArray,10000);
            if (res != EMPTY_STRING) {
                explain_int("tokenizzaEmail(\"\",tokArray,10000)",
                        "",
                        0,EMPTY_STRING,res,
                        "La funzione non controlla se il primo argomento e' ''.",
                        EMPTY_STRING,res);
                return TEST_FAILED;
            }
	    tokens_free(tokArray,10000);
            return TEST_PASSED;
        }
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        {
            char * email =
"From\t10.0.0.1\n\
    10.0.0.2\n\
To:\tbabbo_natale@northpole.org\n\
Subject: Happy Easter!\n\
\n";
            Token tokArray[10000] = { 0 };
            int res = tokenizzaEmail(email,tokArray,10000);
            if (res != BODY_NOT_FOUND) {
                explain_int("tokenizzaEmail(\"...\",tokArray,10000)",
                        email,
                        0,BODY_NOT_FOUND,res,
                        "La funzione non si accorge che l'email non ha il corpo.",
                        BODY_NOT_FOUND,res);
		tokens_free(tokArray,10000);
                return TEST_FAILED;
            }
	    tokens_free(tokArray,10000);
            return TEST_PASSED;
        }
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
            //TODO: trovare un modo per far fallire l'allocazione della memoria
            return NOT_IMPLEMENTED;
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        default:
            return NOT_VALID;
    }
}
int test_tokenizzaEmail1(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            char * email =
"From\t10.0.0.1\n\
    10.0.0.2\n\
To:\tbabbo_natale@northpole.org\n\
Subject: Happy Easter!\n\
\n\
Buona Pasqua da Andrea\n";
            // provo a passare NULL
            int res = tokenizzaEmail(email,NULL,10000);
            if (res != NULL_POINTER) {
                explain_int("tokenizzaEmail(\"...\",NULL,10000)",
                        NULL,
                        1,NULL_POINTER,res,
                        "La funzione non controlla se il secondo argomento e' NULL.",
                        NULL_POINTER,res);
                return TEST_FAILED;
            }
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_tokenizzaEmail2(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
            return NOT_IMPLEMENTED;
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
int test_printTokens0(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
        {
	    int tested=0;
	    int failed=0;
            Token tokens[] = { 
                {"10",          "From"},
                {"10",          "From"},
                {"babbo",       "To:"},
                {"natale",      "To:"},
                {"northpole",   "To:"},
                {"org",         "To:"},
                {"Happy",       "Subject:"},
                {"Easter",      "Subject:"},
                {"Buona",       NULL},
                {"Pasqua",      NULL},
                {"da",          NULL},
                {"Andrea",      NULL},
                {NULL,          NULL},
                };
            char * expected = \
"From*10\n\
From*10\n\
To:*babbo\n\
To:*natale\n\
To:*northpole\n\
To:*org\n\
Subject:*Happy\n\
Subject:*Easter\n\
Buona\n\
Pasqua\n\
da\n\
Andrea\n";

            freopen("mio_stdout.txt","w",stdout);
            int res = printTokens(tokens,12);
            fflush(stdout);
            fclose(stdout);
            freopen(CONSOLE,"w",stdout);
	    tested++;
            if (res != RESULT_OK) {
                explain_int("printTokens(NULL,10000)",
                        "[From*10,From*10,To:*babbo,...",
                        0,RESULT_OK,res,
                        "La funzione ha avuto un errore imprevisto.",
                        RESULT_OK,res);
		failed++;
            }
            FILE * out = fopen("mio_stdout.txt","r");
            char * buffer = calloc(1,10000);
            int nchar = fread(buffer,1,10000,out);
            buffer[nchar] = '\0';
	    tested++;
            if (strcmp(buffer,expected)) {
                explain_str("printTokens(\"...\",10000)",
                        "[From*10,From*10,To:*babbo,...",
                        0,RESULT_OK,res,
                        "La funzione non ha stampato il testo corretto.",
                        expected,buffer);
		failed++;
            }
	    free(buffer);
	    fprintf(stderr, "(%d/%d)", tested-failed, tested);
	    if (failed) return TEST_FAILED;
            return TEST_PASSED;
        }
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        {
            // provo a passare NULL
            int res = printTokens(NULL,10000);
            if (res != NULL_POINTER) {
                explain_int("printTokens(NULL,10000)",
                        NULL,
                        0,NULL_POINTER,res,
                        "La funzione non controlla se il primo argomento e' NULL.",
                        NULL_POINTER,res);
                return TEST_FAILED;
            }
            return TEST_PASSED;
        }
        case UNKNOWN_ERROR:	    // errore sconosciuto
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}
int test_printTokens1(int tipo) {
    switch (tipo) {
        case RESULT_OK:	        // la funzione ha avuto successo
            return NOT_IMPLEMENTED;
        case UNKNOWN_ERROR:	    // errore sconosciuto
            return NOT_IMPLEMENTED;
        case NULL_POINTER:	    // uno degli argomenti della funzione era NULL
        case EMPTY_STRING:	    // uno degli argomenti era una stringa vuota
        case FILE_MISSING:	    // il file non esiste
        case FILE_UNREADABLE:	// il file non e' leggibile
        case FILE_UNWRITABLE:	// il file non e' scrivibile
        case FILE_EMPTY:	    // il file e' vuoto
        case BUFFER_TOO_SMALL:	// il blocco di memoria (o vettore) passato non e' sufficiente a contenere i dati
        case HEADER_NOT_FOUND:	// lo header del messaggio non esiste
        case BODY_NOT_FOUND:	// il corpo del messaggio non esiste
        case BAD_HEADER:	    // una riga degli headers non segue la sintassi giusta
        case MEM_ALLOC_ERROR:	// non si e' potuto allocare un blocco di memoria
        default:
            return NOT_VALID;
    }
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

