#include "modulo1.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#define ALR_MIN(a, b) (((a) <= (b)) ? (a) : (b))
#define ALR_MAX(a, b) (((a) >= (b)) ? (a) : (b))

int CheckBufParams(void*, int);
int IsAlphanum(char c);
int ReadWhile(char *pBuf, char *pSet);
int ReadUntil(char *pBuf, char *pSet);
int TokenizeBuffer(char*, Token[], int, int*, char*);

int leggiFile(char *nomefile, int dimensione, char *testo) {
    FILE *f;
    if (!nomefile) {
        return NULL_POINTER;
    }
    if (!*nomefile) {
        return EMPTY_STRING;
    }
    if (f = fopen(nomefile, "r")) {
        int nRetVal = RESULT_OK;
        char foo;
        int nRead = fread(&foo, 1, 1, f);
        if (!nRead) {
            nRetVal = FILE_EMPTY;
        }
        else {
            fseek(f, 0, SEEK_SET);
            if (dimensione - 1 >= 0) {
                if (testo) {
                    int nRead = fread(testo, 1, dimensione, f);
                    testo[ALR_MIN(nRead, dimensione - 1)] = 0;
                    if (dimensione == nRead) {
                        nRetVal = BUFFER_TOO_SMALL;
                    }
                    /*else {
                        if (!*testo) {
                            nRetVal = FILE_EMPTY;
                        }
                    }*/
                }
                else {
                    nRetVal = NULL_POINTER;
                }
            }
            else {
                nRetVal = BUFFER_TOO_SMALL;
            }
        }
        fclose(f);
        return nRetVal;
    }
    else {
        struct stat s;
        if (-1 == stat(nomefile, &s)) {
            return FILE_MISSING;
        }
        else {
            return FILE_UNREADABLE;
        }
    }
}

int estraiHeaders(char *email, char *headers, int dimh) {
    int nRetVal, i, j, done = 0;
    if (!email) {
        return NULL_POINTER;
    }
    if (!*email) {
        return EMPTY_STRING;
    }
    if ('\n' == email[0]) {
        return HEADER_NOT_FOUND;
    }
    for (i = 0; email[i]; i++) {
        if (('\n' == email[i]) &&
            ('\n' == email[i + 1]))
        {
            done = 1;
            i++;
            break;
        }
    }
    if (done) {
        if (RESULT_OK == (nRetVal = CheckBufParams(headers, --dimh))) {
            if (i > dimh) {
                nRetVal = BUFFER_TOO_SMALL;
            }
            j = ALR_MIN(i, dimh);
            memcpy(headers, email, j);
            headers[j] = 0;
        }
    }
    else {
        nRetVal = BODY_NOT_FOUND;
    }
    return nRetVal;
}

int primoHeader(char *headers, char *primo_header, int dim) {
    int i, nRetVal;
    if (!headers) {
        return NULL_POINTER;
    }
    if (!*headers) {
        return EMPTY_STRING;
    }
    i = -1;
    do {
        i++;
        i += ReadUntil(headers + i, "\n");
    }
    while ((' ' == headers[i + 1]) || ('\t' == headers[i + 1]));
    if (i) {
        if (RESULT_OK == (nRetVal = CheckBufParams(primo_header, --dim))) {
            int j = ALR_MIN(i, dim);
            if (i > j) {
                nRetVal = BUFFER_TOO_SMALL;
            }
            memcpy(primo_header, headers, j);
            primo_header[j] = 0;
        }
    }
    else {
        nRetVal = HEADER_NOT_FOUND;
    }
    return nRetVal;
}

int parseHeader(char *header, char *tipo, int dimt, char *valore, int dimv) {
    int i, nRetVal;
    if (!header) {
        return NULL_POINTER;
    }
    if (!*header) {
        return EMPTY_STRING;
    }
    i = ReadUntil(header, ": \t");
    if (header[i]) {
        if (RESULT_OK == (nRetVal = CheckBufParams(tipo, --dimt))) {
            if (RESULT_OK == (nRetVal = CheckBufParams(valore, --dimv))) {
                int j;
                if (':' == header[i]) {
                    i++;
                }
                j = ALR_MIN(i, dimt);
                if (i > j) {
                    nRetVal = BUFFER_TOO_SMALL;
                }
                memcpy(tipo, header, j);
                tipo[j] = 0;
                header += j;
                header += ReadWhile(header, " \t");
                for (i = strlen(header) - 1; (' ' == header[i]) ||
                    ('\t' == header[i]) || ('\n' == header[i]); i--);
                i++;
                j = ALR_MIN(i, dimv);
                if (i > j) {
                    nRetVal = BUFFER_TOO_SMALL;
                }
                memcpy(valore, header, j);
                valore[j] = 0;
            }
        }
    }
    else {
        nRetVal = BAD_HEADER;
    }
    return nRetVal;
}

int primaParola(char *testo, char *parola, int dim) {
    int nRetVal;
    if (!testo) {
        return NULL_POINTER;
    }
    if (!*testo) {
        return EMPTY_STRING;
    }
    if (RESULT_OK == (nRetVal = CheckBufParams(parola, --dim))) {
        int i, j;
        while (*testo) {
            if (IsAlphanum(testo[0]) && IsAlphanum(testo[1])) {
                break;
            }
            testo++;
        }
        for (i = 0; testo[i] && IsAlphanum(testo[i]); i++);
        j = ALR_MIN(i, dim);
        if (i > j) {
            nRetVal = BUFFER_TOO_SMALL;
        }
        memcpy(parola, testo, j);
        parola[j] = 0;
    }
    return nRetVal;
}

int tokenizzaEmail(char *email, Token tokenArray[], int dimT) {
    int nRetVal = RESULT_OK;
    if (email) {
        int nIndex = 0,
            nHdrsLen = strlen(email) + 1;
        char *pHdrs = (char*)malloc(nHdrsLen * sizeof(char)), *pTemp = pHdrs;
        if (RESULT_OK == (nRetVal = estraiHeaders(email, pHdrs, nHdrsLen))) {
            if (RESULT_OK == (nRetVal = CheckBufParams(tokenArray, dimT))) {
                char *pCurHdr;
                nHdrsLen = strlen(pHdrs) + 1;
                pCurHdr = (char*)malloc(nHdrsLen * sizeof(char));
                tokenArray[0].contesto = NULL;
                tokenArray[0].parola = NULL;
                while (*pHdrs && (RESULT_OK == nRetVal)) {
                    nRetVal = primoHeader(pHdrs, pCurHdr, nHdrsLen);
                    if (RESULT_OK == nRetVal) {
                        int len = strlen(pCurHdr) + 1;
                        char *pCxt = (char*)malloc(len * sizeof(char)),
                            *pVal = (char*)malloc(len * sizeof(char));
                        pHdrs += len;
                        if (RESULT_OK == (nRetVal = parseHeader(pCurHdr, pCxt, len, pVal, len))) {
                            nRetVal = TokenizeBuffer(pVal, tokenArray, dimT, &nIndex, pCxt);
                        }
                        free(pCxt);
                        free(pVal);
                    }
                }
                free(pCurHdr);
                if (RESULT_OK == nRetVal) {
                    email += nHdrsLen;
                    if (*email) {
                        TokenizeBuffer(email, tokenArray, dimT, &nIndex, NULL);
                    }
                    else {
                        nRetVal = BODY_NOT_FOUND;
                    }
                }
            }
        }
        free(pTemp);
    }
    else {
        nRetVal = NULL_POINTER;
    }
    return nRetVal;
}

int printTokens(Token tokenArray[], int dimT) {
    int nRetVal = CheckBufParams(tokenArray, dimT);
    if (RESULT_OK == nRetVal) {
        int i;
        for (i = 0; (i < dimT) && tokenArray[i].parola; i++) {
            if (tokenArray[i].contesto) {
                printf("%s*%s\n", tokenArray[i].contesto, tokenArray[i].parola);
            }
            else {
                printf(tokenArray[i].parola);
                printf("\n");
            }
        }
    }
    return nRetVal;
}

int CheckBufParams(void *pBuf, int nSize) {
    int nRes = RESULT_OK;
    if (!pBuf) {
        nRes = NULL_POINTER;
    }
    else if (nSize < 0) {
        nRes = BUFFER_TOO_SMALL;
    }
    return nRes;
}

int IsAlphanum(char c) {
    return ((c >= 'A') && (c <= 'Z')) ||
        ((c >= 'a') && (c <= 'z')) ||
        ((c >= '0') && (c <= '9'));
}

int ReadWhile(char *pBuf, char *pSet) {
    int nSetLen = strlen(pSet),
        nRead = 0, fQuit = 0;
    for (; pBuf[nRead] && !fQuit; nRead++) {
        int i;
        fQuit = 1;
        for (i = 0; i < nSetLen; i++) {
            if (pSet[i] == pBuf[nRead]) {
                fQuit = 0;
                break;
            }
        }
        if (fQuit) {
            nRead--;
        }
    }
    return nRead;
}

int ReadUntil(char *pBuf, char *pSet) {
    int nSetLen = strlen(pSet),
        nRead = 0, fQuit = 0;
    for (; pBuf[nRead] && !fQuit; nRead++) {
        int i;
        for (i = 0; i < nSetLen; i++) {
            if (pSet[i] == pBuf[nRead]) {
                fQuit = 1;
                nRead--;
                break;
            }
        }
    }
    return nRead;
}

int TokenizeBuffer(char *pBuf, Token tokenArray[], int dimT, int *pIndex, char *pContext) {
    if (pBuf && pIndex) {
        int nRetVal, nIndex = *pIndex;
        dimT = dimT - nIndex - 1;
        tokenArray += nIndex;
        nIndex = 0;
        if (RESULT_OK == (nRetVal = CheckBufParams(tokenArray, dimT))) {
            int nCxtLen = 0, nWordBufLen = strlen(pBuf) + 1;
            char *pWord = (char*)malloc(nWordBufLen * sizeof(char));
            if (pContext) {
                nCxtLen = strlen(pContext);
            }
            while (*pBuf && (RESULT_OK == nRetVal)) {
                while (*pBuf) {
                    if (IsAlphanum(pBuf[0]) && IsAlphanum(pBuf[1])) {
                        break;
                    }
                    pBuf++;
                }
                if (*pBuf) {
                    if (nIndex < dimT) {
                        int nWordLen;
                        if (nCxtLen) {
                            tokenArray[nIndex].contesto = (char*)malloc((nCxtLen + 1) * sizeof(char));
                            strcpy(tokenArray[nIndex].contesto, pContext);
                        }
                        else {
                            tokenArray[nIndex].contesto = NULL;
                        }
                        nRetVal = primaParola(pBuf, pWord, nWordBufLen);
                        nWordLen = strlen(pWord) + 1;
                        tokenArray[nIndex].parola = (char*)malloc(nWordLen * sizeof(char));
                        memcpy(tokenArray[nIndex].parola, pWord, nWordLen);
                        pBuf += nWordLen - 1;
                        nIndex++;
                    }
                    else {
                        nRetVal = BUFFER_TOO_SMALL;
                    }
                }
            }
            free(pWord);
            if (RESULT_OK == nRetVal) {
                tokenArray[nIndex].contesto = NULL;
                tokenArray[nIndex].parola = NULL;
            }
        }
        *pIndex += nIndex;
        return nRetVal;
    }
    else {
        return NULL_POINTER;
    }
}

