#include "testModulo1.h"
#include <stdlib.h>
#include <stdio.h>

#define ALR_DOTEST(call, descr) \
    if (tipo_di_test != (nRes = call)) { \
        nRetVal = TEST_FAILED; \
        PrintReport(#call, tipo_di_test, nRes, descr); \
    } \
    else { \
        nRetVal = TEST_PASSED; \
    } \


#define ALR_DOTEST_CHECKBUF(call, descr, expected) \
    if (tipo_di_test != (nRes = call)) { \
        nRetVal = TEST_FAILED; \
        PrintReport(#call, tipo_di_test, nRes, descr); \
    } \
    else { \
        if (CheckBuffer(expected, pBuf)) { \
            nRetVal = TEST_PASSED; \
        } \
        else { \
            nRetVal = TEST_FAILED; \
            PrintReport(#call, tipo_di_test, nRes, \
                "la funzione ha restituito il risultato atteso, ma il contenuto del buffer e' errato"); \
        } \
    } \


#define ALR_CASEARG(arg, call, descr) \
    case arg: \
        ALR_DOTEST(call, descr); \
        break; \


#define ALR_CASEARG2(arg, call, descr, expected) \
    case arg: \
        ALR_DOTEST_CHECKBUF(call, descr, expected); \
        break; \


#define ALR_CASEARGS(arg1, arg2, call, descr) \
    case arg1: \
    case arg2: \
        ALR_DOTEST(call, descr); \
        break; \


#define ALR_CASEARGS2(arg1, arg2, call, descr, expected) \
    case arg1: \
    case arg2: \
        ALR_DOTEST_CHECKBUF(call, descr, expected); \
        break; \


#define ALR_CHECKARG(arg, errcode, call, descr) \
    if (arg != argomento) { \
        nRetVal = NOT_VALID; \
    } \
    else { \
        ALR_DOTEST(call, descr); \
    } \


#define ALR_CHECKARG2(arg, errcode, call, descr, expected) \
    if (arg != argomento) { \
        nRetVal = NOT_VALID; \
    } \
    else { \
        ALR_DOTEST_CHECKBUF(call, descr, expected); \
    } \


#define ALR_SWITCHARG_B \
    switch (argomento) { \


#define ALR_SWITCHARG_E \
    default: \
        nRetVal = NOT_VALID; \
        break; \
    } \


char *GetCodeName(int);
int CheckBuffer(char*, char*);
void PrintReport(char*, int, int, char*);
void FreeTokens(Token tokenArray[]);

int test_leggiFile(int tipo_di_test, int argomento) {
    if ((argomento >= 0) && (argomento <= 2)) {
        int nRetVal = NOT_VALID, nRes;
        char pBuf[0x400];
        char *descr[7] = {
            "leggiFile non si comporta correttamente in casi normali",
            "leggiFile non verifica la validita' dei parametri nell'ordine corretto",
            "leggiFile non controlla la validita' del terzo parametro",
            "leggiFile reagisce in maniera errata se il file da aprire non esiste",
            "leggiFile non avvisa se il file e' bloccato in lettura",
            "leggiFile non avvisa quando il file e' vuoto",
            "leggiFile non controlla correttamente che il buffer da riempire abbia dimensioni sufficientemente grandi"
        };
        switch (tipo_di_test) {
        case RESULT_OK:
            ALR_CHECKARG(0, NOT_IMPLEMENTED, leggiFile("email1.txt", 0x400, pBuf), descr[0])
            break;
        case UNKNOWN_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        case NULL_POINTER:
            ALR_SWITCHARG_B
                ALR_CASEARG(0, leggiFile(NULL, -1, pBuf), descr[1])
                ALR_CASEARG(2, leggiFile("email1.txt", 2, NULL), descr[2])
            ALR_SWITCHARG_E
            break;
        case FILE_MISSING:
            ALR_SWITCHARG_B
                ALR_CASEARG(0, leggiFile("file-missing", -1, NULL), descr[3])
            ALR_SWITCHARG_E
            break;
        case FILE_UNREADABLE:
            ALR_CHECKARG(0, NOT_VALID, leggiFile("file-unreadable", -1, NULL), descr[4])
            break;
        case FILE_EMPTY:
            ALR_CHECKARG(0, NOT_VALID, leggiFile("file-empty", 0x400, pBuf), descr[5])
            break;
        case BUFFER_TOO_SMALL:
            ALR_SWITCHARG_B
                ALR_CASEARGS(1, 2, leggiFile("email1.txt", 1, pBuf), descr[6])
            ALR_SWITCHARG_E
            break;
        case MEM_ALLOC_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        }
        return nRetVal;
    }
    else {
        return ARG_MISSING;
    }
}

int test_estraiHeaders(int tipo_di_test, int argomento) {
    if ((argomento >= 0) && (argomento <= 2)) {
        int nRetVal = NOT_VALID, nRes;
        char pBuf[0x400];
        char *descr[4] = {
            "estraiHeaders non si comporta correttamente in casi normali",
            "estraiHeaders non verifica la validita' dei parametri nell'ordine corretto",
            "estraiHeaders non verifica che le dimensioni del buffer siano sufficienti per gli headers",
            "estraiHeaders non determina correttamente la presenza effettiva di un header"
        };
        switch (tipo_di_test) {
        case RESULT_OK:
            ALR_CHECKARG2(0, NOT_IMPLEMENTED, estraiHeaders("headers\n\nbody", pBuf, 13), descr[0], "headers\n\0")
            break;
        case UNKNOWN_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        case NULL_POINTER:
            ALR_SWITCHARG_B
                ALR_CASEARG(0, estraiHeaders(NULL, pBuf, -1), descr[1])
                ALR_CASEARG(1, estraiHeaders("headers\n\nbody", NULL, -1), descr[1])
            ALR_SWITCHARG_E
            break;
        case EMPTY_STRING:
            ALR_CHECKARG(0, NOT_VALID, estraiHeaders("", NULL, -1), descr[1])
            break;
        case BUFFER_TOO_SMALL:
            ALR_SWITCHARG_B
                ALR_CASEARGS(1, 2, estraiHeaders("headers\n\nbody", pBuf, 2), descr[2])
            ALR_SWITCHARG_E
            break;
        case HEADER_NOT_FOUND:
            ALR_CHECKARG(0, NOT_VALID, estraiHeaders("\nFrom: andrea@sterbini.org\n\nblah blah blah", pBuf, 0x400), descr[3])
            break;
        case MEM_ALLOC_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        }
        return nRetVal;
    }
    else {
        return ARG_MISSING;
    }
}

int test_primoHeader(int tipo_di_test, int argomento) {
    if ((argomento >= 0) && (argomento <= 2)) {
        int nRetVal = NOT_VALID, nRes;
        char pBuf[0x400];
        char *descr[5] = {
            "primoHeader non si comporta correttamente in casi normali",
            "primoHeader non controlla la validita' dei parametri nell'ordine corretto",
            "primoHeader non avvisa quando la stringa degli headers e' vuota",
            "primoHeader non avverte quando il buffer  troppo piccolo per contenere il primo header",
            "primoHeader non avvisa quando l'header e' assente"
        };
        switch (tipo_di_test) {
        case RESULT_OK:
            ALR_CHECKARG2(0, NOT_IMPLEMENTED, primoHeader("01234:56789\n\n", pBuf, 13), descr[0], "01234:56789\0")
            break;
        case UNKNOWN_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        case NULL_POINTER:
            ALR_SWITCHARG_B
                ALR_CASEARG(0, primoHeader(NULL, pBuf, -1), descr[1])
                ALR_CASEARG(1, primoHeader("header\n\n", NULL, -1), descr[1])
            ALR_SWITCHARG_E
            break;
        case EMPTY_STRING:
            ALR_CHECKARG(0, NOT_VALID, primoHeader("", pBuf, 1), descr[2])
            break;
        case BUFFER_TOO_SMALL:
            ALR_SWITCHARG_B
                ALR_CASEARGS(1, 2, primoHeader("01234:56789\n", pBuf, -1), descr[3])
            ALR_SWITCHARG_E
            break;
        case HEADER_NOT_FOUND:
            ALR_CHECKARG(0, NOT_VALID, primoHeader("\nFrom: andrea@sterbini.org\n\nblah blah blah", pBuf, 0x400), descr[4])
            break;
        case MEM_ALLOC_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        }
        return nRetVal;
    }
    else {
        return ARG_MISSING;
    }
}

int test_parseHeader(int tipo_di_test, int argomento) {
    if ((argomento >= 0) && (argomento <= 4)) {
        int nRetVal = NOT_VALID, nRes;
        char pBuf[0x400];
        int nCxtMaxLen = 0x200;
        char *descr[5] = {
            "parseHeader non si comporta correttamente in casi normali",
            "parseHeader non verifica la validita' dei parametri nell'ordine corretto",
            "parseHeader non verifica correttamente che le dimensioni del buffer \"tipo\" siano sufficienti",
            "parseHeader non verifica correttamente che le dimensioni del buffer \"valore\" siano sufficienti",
            "parseHeader controlla male la sintassi dell'header"
        };
        switch (tipo_di_test) {
        case RESULT_OK:
            ALR_CHECKARG2(0, NOT_IMPLEMENTED, parseHeader("From\tandrea@\n\tsterbini\n\t.org \n", pBuf, 5, pBuf + 5, 0x400 - 5),
                descr[0], "From\0andrea@\n\tsterbini\n\t.org\0")
            break;
        case UNKNOWN_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        case NULL_POINTER:
            ALR_SWITCHARG_B
                ALR_CASEARG(0, parseHeader(NULL, pBuf, -1, pBuf + nCxtMaxLen, -1), descr[1])
                ALR_CASEARG(1, parseHeader("From\tandrea@\n\tsterbini\n\t.org", NULL, -1, pBuf + nCxtMaxLen, -1), descr[1])
                ALR_CASEARG(3, parseHeader("From\tandrea@\n\tsterbini\n\t.org", pBuf, nCxtMaxLen, NULL, -1), descr[1])
            ALR_SWITCHARG_E
            break;
        case EMPTY_STRING:
            ALR_CHECKARG(0, NOT_VALID, parseHeader("", NULL, -1, NULL, -1), descr[1])
            break;
        case BUFFER_TOO_SMALL:
            ALR_SWITCHARG_B
                ALR_CASEARGS(1, 2, parseHeader("From\tandrea@\n\tsterbini\n\t.org", pBuf, -1, NULL, 0x400 - nCxtMaxLen), descr[2])
                ALR_CASEARGS(3, 4, parseHeader("From\tandrea@\n\tsterbini\n\t.org", pBuf, nCxtMaxLen, pBuf + nCxtMaxLen, -1), descr[3])
            ALR_SWITCHARG_E
            break;
        case BAD_HEADER:
            ALR_CHECKARG(0, NOT_VALID, parseHeader("tiposenzavalore", NULL, -1, NULL, -1), descr[4])
            break;
        case MEM_ALLOC_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        }
        return nRetVal;
    }
    else {
        return ARG_MISSING;
    }
}

int test_primaParola(int tipo_di_test, int argomento) {
    if ((argomento >= 0) && (argomento <= 2)) {
        int nRetVal = NOT_VALID, nRes;
        char pBuf[0x400];
        char *descr[3] = {
            "primaParola non si comporta correttamente in casi normali",
            "primaParola non testa la validita' dei parametri nell'ordine corretto",
            "primaParola non avvisa quando le dimensioni del buffer sono insufficienti"
        };
        switch (tipo_di_test) {
        case RESULT_OK:
            ALR_CHECKARG2(0, NOT_IMPLEMENTED, primaParola("a  1 2 blah1 blah", pBuf, 5), descr[0], "blah\0")
            break;
        case UNKNOWN_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        case NULL_POINTER:
            ALR_SWITCHARG_B
                ALR_CASEARG(0, primaParola(NULL, pBuf, -1), descr[1])
                ALR_CASEARG(1, primaParola("a  1 2 blah1 blah", NULL, -1), descr[1])
            ALR_SWITCHARG_E
            break;
        case EMPTY_STRING:
            ALR_CHECKARG(0, NOT_VALID, primaParola("", NULL, -1), descr[1])
            break;
        case BUFFER_TOO_SMALL:
            ALR_SWITCHARG_B
                ALR_CASEARGS(1, 2, primaParola("a  1 2 blah1 blah", pBuf, -1), descr[2])
            ALR_SWITCHARG_E
            break;
        case MEM_ALLOC_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        }
        return nRetVal;
    }
    else {
        return ARG_MISSING;
    }
}

int test_tokenizzaEmail(int tipo_di_test, int argomento) {
    if ((argomento >= 0) && (argomento <= 2)) {
        int nRetVal = NOT_VALID, nRes;
        char pBuf[0x400];
        char *descr[6] = {
            "tokenizzaEmail non si comporta correttamente in casi normali",
            "tokenizzaEmail non verifica la validita' dei parametri nell'ordine corretto",
            "tokenizzaEmail non verifica che le dimensioni del buffer siano sufficienti",
            "tokenizzaEmail non propaga l'errore HEADER_NOT_FOUND",
            "tokenizzaEmail non propaga l'errore BODY_NOT_FOUND",
            "tokenizzaEmail non propaga l'errore BAD_HEADER"
        };
        ((Token*)pBuf)[0].contesto = NULL;
        ((Token*)pBuf)[0].parola = NULL;
        switch (tipo_di_test) {
        case RESULT_OK:
            {
                Token *pActual = (Token*)pBuf, pExpected[6] = {
                    {"me\0", "from:\0"},
                    {"myself\0", "from:\0"},
                    {"blah\0", NULL},
                    {"blah\0", NULL},
                    {"blah\0", NULL},
                    {NULL, NULL}
                };
                ALR_CHECKARG(0, NOT_VALID, tokenizzaEmail("from:me@myself.i\n\nblah blah\n\nblah", (Token*)pBuf, 0x400 / sizeof(Token)), descr[0])
                if (TEST_PASSED == nRetVal) {
                    int i;
                    for (i = 0; pExpected[i].parola; i++) {
                        if (pExpected[i].contesto) {
                            if (pActual[i].contesto) {
                                if (!CheckBuffer(pExpected[i].contesto, pActual[i].contesto)) {
                                    nRetVal = TEST_FAILED;
                                }
                            }
                            else {
                                nRetVal = TEST_FAILED;
                            }
                        }
                        if (TEST_PASSED == nRetVal) {
                            if (!CheckBuffer(pExpected[i].parola, pActual[i].parola)) {
                                nRetVal = TEST_FAILED;
                            }
                        }
                        if (TEST_FAILED == nRetVal) {
                            PrintReport("tokenizzaEmail(\"from:me@myself.i\\n\\nblah blah\\n\\nblah\", (Token*)pBuf, 0x400 / sizeof(Token))",
                                tipo_di_test, nRes, "il contenuto del buffer restituito da tokenizzaEmail e' errato");
                            break;
                        }
                    }
                }
            }
            break;
        case UNKNOWN_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        case NULL_POINTER:
            ALR_SWITCHARG_B
                ALR_CASEARG(0, tokenizzaEmail(NULL, (Token*)pBuf, -1), descr[1])
                ALR_CASEARG(1, tokenizzaEmail("from:me@myself.i\n\nbody\nword1 word2", NULL, -1), descr[1])
            ALR_SWITCHARG_E
            break;
        case EMPTY_STRING:
            ALR_CHECKARG(0, NOT_VALID, tokenizzaEmail("", NULL, -1), descr[1])
            break;
        case BUFFER_TOO_SMALL:
            ALR_SWITCHARG_B
                ALR_CASEARGS(1, 2, tokenizzaEmail("from:me@myself.i\n\nbody\nword1 word2", (Token*)pBuf, -1), descr[2])
            ALR_SWITCHARG_E
            break;
        case HEADER_NOT_FOUND:
            ALR_CHECKARG(0, NOT_VALID, tokenizzaEmail("\nblah blah blah", NULL, -1), descr[3])
            break;
        case BODY_NOT_FOUND:
            ALR_CHECKARG(0, NOT_VALID, tokenizzaEmail("from: me@myself.i\n\n", (Token*)pBuf, 0x400 / sizeof(Token)), descr[4])
            break;
        case BAD_HEADER:
            ALR_CHECKARG(0, NOT_VALID, tokenizzaEmail("blah\n\nblah blah", (Token*)pBuf, 0x400 / sizeof(Token)), descr[5])
            break;
        case MEM_ALLOC_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        }
        FreeTokens((Token*)pBuf);
        return nRetVal;
    }
    else {
        return ARG_MISSING;
    }
}

int test_printTokens(int tipo_di_test, int argomento) {
    if ((argomento >= 0) && (argomento <= 2)) {
        int nRetVal = NOT_VALID, nRes;
        Token pTokens[6] = {
            {"me", "from:"},
            {"myself", "from:"},
            {"blah", NULL},
            {"blah", NULL},
            {"blah", NULL},
            {NULL, NULL}
        };
        switch (tipo_di_test) {
        case RESULT_OK:
            ALR_CHECKARG(0, NOT_IMPLEMENTED, printTokens(pTokens, 6),
                "la funzione printTokens non stampa correttamente i tokens");
            break;
        case UNKNOWN_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        case BUFFER_TOO_SMALL:
            ALR_CHECKARG(0, NOT_VALID, printTokens(pTokens, -1),
                "la funzione printTokens non controlla la validita' del secondo argomento");
            break;
        case NULL_POINTER:
            ALR_CHECKARG(0, NOT_VALID, printTokens(NULL, -1),
                "la funzione printTokens non controlla la validita' del primo argomento");
            break;
        case MEM_ALLOC_ERROR:
            nRetVal = NOT_IMPLEMENTED;
            break;
        }
        return nRetVal;
    }
    else {
        return ARG_MISSING;
    }
}

char *GetCodeName(int nCode) {
    switch (nCode) {
    case RESULT_OK:
        return "RESULT_OK";
    case UNKNOWN_ERROR:
        return "UNKNOWN_ERROR";
    case NULL_POINTER:
        return "NULL_POINTER";
    case EMPTY_STRING:
        return "EMPTY_STRING";
    case FILE_MISSING:
        return "FILE_MISSING";
    case FILE_UNREADABLE:
        return "FILE_UNREADABLE";
    case FILE_UNWRITABLE:
        return "FILE_UNWRITABLE";
    case FILE_EMPTY:
        return "FILE_EMPTY";
    case BUFFER_TOO_SMALL:
        return "BUFFER_TOO_SMALL";
    case HEADER_NOT_FOUND:
        return "HEADER_NOT_FOUND";
    case BODY_NOT_FOUND:
        return "BODY_NOT_FOUND";
    case BAD_HEADER:
        return "BAD_HEADER";
    case MEM_ALLOC_ERROR:
        return "MEM_ALLOC_ERROR";
    default:
        return "<sconosciuto>";
    }
}

int CheckBuffer(char *pExpected, char *pActual) {
    int i, nRes = 1;
    for (i = 0; pExpected[i] || pExpected[i + 1]; i++) {
        if (pExpected[i] != pActual[i]) {
            nRes = 0;
            break;
        }
    }
    if (nRes) {
        if (pActual[i]) {
            nRes = 0;
        }
    }
    return nRes;
}

void PrintReport(char *pszCallFmt, int nExpectedCode, int nReturnedCode, char *pszDescr) {
    printf("Test fallito:\n\tfunzione testata: %s\n\trisultato atteso: %s\n\trisultato ottenuto: %s\n\tspiegazione: %s\n\n",
        pszCallFmt, GetCodeName(nExpectedCode), GetCodeName(nReturnedCode), pszDescr);
}

void FreeTokens(Token tokenArray[]) {
    int i;
    for (i = 0; tokenArray[i].contesto; i++) {
        if (tokenArray[i].contesto) {
            free(tokenArray[i].contesto);
        }
        if (tokenArray[i].parola) {
            free(tokenArray[i].parola);
        }
    }
}

