#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include "modulo1.h"

/**************************************************************************
 * leggiFile()
 **************************************************************************/

int
leggiFile (char *nomefile, int dimensione, char *testo)
{

  int result;
  struct stat status;

  FILE *fp;

  if (nomefile == NULL || testo == NULL)
    return NULL_POINTER;

  if (nomefile[0] == '\0')
    return EMPTY_STRING;

  if (dimensione < 0)
    return BUFFER_TOO_SMALL;


  result = stat (nomefile, &status);


  if (result == -1)
    return FILE_MISSING;

  if (dimensione < status.st_size)
    return BUFFER_TOO_SMALL;


  fp = fopen (nomefile, "rt");

  if (fp == NULL)

    return FILE_UNREADABLE;


  result = fread (testo, 1, status.st_size, fp);

  testo[result] = '\0';

  fclose (fp);

  return RESULT_OK;
}

/**************************************************************************
 * estraiHeaders()
 **************************************************************************/

int
estraiHeaders (char *email, char *headers, int dimh)
{
	char *p, *l_start, *l_stop;
	int len;
	
/*strstr  una funzione che cerca nel suo primo argomento la prima occorenza della stringa indicata dal suo secondo argomento. In questo caso ricerca la fine dell'header*/
	p = strstr(email, "\n\n");
	
	if (p == NULL)
		return HEADER_NOT_FOUND;
	
	p++; // punta al char successivo
	*p = 0; p++; // azzera il secondo \n, punta al char successivo
	
	// controlla che ogni riga della sezione Header sia di almeno 1 carattere	
	l_start = email;
	l_stop = strchr(l_start, '\n');
	
	while (l_stop != NULL)
	{
		//printf("-%lu-%lu- %lu\n", l_start, l_stop, l_stop-l_start);
		if (l_stop - l_start < 1)
			return BAD_HEADER;
		
		l_start = l_stop + 1;
		l_stop = strchr(l_start, '\n');
	}
	
	len = strlen(email); // ricava la lunghezza dell'header
	
	// controlla che ci sia spazio nel buffer per salvare
	// l'header e lo '\0' finale
	if (dimh-1 < len)
		return BUFFER_TOO_SMALL;
	
	// copia gli headers nel buffer
	strcpy(headers, email);
	
	// ripristina gli \n
	p--; *p = '\n';
	
	return RESULT_OK;
}

/**************************************************************************
 * primoHeader()
 **************************************************************************/
int primoHeader(char * headers, char * primo_header, int dim)
{
	int len;
	char *p, *end;
	
	p = strchr(headers, '\n'); // trova la prima riga
	if (p == NULL)	return BAD_HEADER;
	*p=0; // termina la prima riga con '\0'
	if (strlen(headers) < 1) // controlla che la prima riga non sia nulla
		return BAD_HEADER;
	end = p; // salviamo la posizione in cui finisce la riga
	
	while(1)
	{
		// cerchiamo la prox riga
		if ((p = strchr(end+1, '\n')) == NULL)
			return BAD_HEADER;
		
		*p=0; // termina la riga
	
		// se la riga successiva e' almeno di un carattere e se il primo
		// carattere e' uno spazio o un TAB, allora l'header continua
		if (strlen(end+1)>=1 && (*(end+1)==' ' || *(end+1)=='\t')) {
			*end='\n'; // ripristina lo \n
			end = p;  // salta alla nuova riga
		}
		else // la prima riga dell'header e' terminata, possiamo uscire
		{
			*p='\n';
			break;
		}
	}
	
	// ricava la lunghezza dell'header, +1 perche' lo \n e' stato sostituito
	// con uno \0
	len = strlen(headers)+1; 
	
	// controlla che ci sia spazio nel buffer per salvare
	// l'header e lo '\0' finale
	if (dim-1 < len)
		return BUFFER_TOO_SMALL;
	
	// copia gli headers nel buffer
	*end = '\n';
	strncpy(primo_header, headers, len);
	primo_header[len] = 0;
	
	return RESULT_OK;
}

/**************************************************************************
 * parseheader()
 **************************************************************************/
#define MAX_LEN 10000
int parseHeader(char* header, char* tipo, int dimt, char* valore, int dimv) {
    char buf[MAX_LEN];
    int i = 0;
    
    while(header != ':') 
        buf[i++] = (header)++;
    buf[i] = '\0';
    (header)++;
    /* ora header punta al primo carattere del valore dello header, ad esempio 
    con From: ciao@test.it, header punta a 'c' */
    tipo = (char *)calloc(strlen(buf)+1, sizeof(char));
    strcpy(tipo, buf);
    /* copia in tipo il contenuto di buf */
    
    if (!(*buf)) {
        valore = (char *)calloc(strlen(header)+1, sizeof(char));
        strcpy(valore, header);
    } /* se buf  vuoto, significa che non esiste il tipo, quindi si sta 
    analizzando il testo dell'email, che puo' contenere vari \n */
    else {
        i = 0;
        while(*header != '\n')
            buf[i++] = (header)++;
        buf[i] = '\0';
        valore = (char *)calloc(strlen(buf)+1, sizeof(char));
        strcpy(valore, buf);
        (header)++;
    } /* if-else */
}
              



/**************************************************************************
 * primaParola()
 **************************************************************************/
int primaParola(char* testo, char* parola, int dim){	

    char buf[MAX_LEN];
int i=0;
	do {
    buf[i] = *testo;
    i++;
    testo++;
} while (isalnum(*testo));

buf[i] = '\0';
if (dim < i) return BUFFER_TOO_SMALL;
strcpy(parola, buf);
return RESULT_OK;

}

