/* Antonino Muratore / 1046705 */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>

#include "modulo1.h"

// funzione che copia da un file i primi 'dimensione' caratteri nel blocco di memoria 'testo'
// e mette nella variabile 'dimensione' il numero di caratteri letti
int leggiFile(char * nomefile, int dimensione, char * testo) {
	struct	stat	stfile;
	int	readed = 0;
	FILE	*fp;

	if (nomefile == NULL)
		return NULL_POINTER;
	if (strlen(nomefile) == 0)
		return EMPTY_STRING;

	if (stat(nomefile, &stfile) == -1) {
		if (errno == ENOENT)
			return FILE_MISSING;
	}

	if (stfile.st_size == 0)
		return FILE_EMPTY;

	if ((fp = fopen(nomefile, "r")) == NULL)
		return FILE_UNREADABLE;

	bzero(testo, dimensione + 1);
	fread(testo, dimensione, 1, fp);

	fclose(fp);
	return RESULT_OK;
}

// funzione che individua le intestazioni della email e le copia nel blocco di memoria 'headers' di lunghezza 'dimh'
int estraiHeaders(char * email, char * headers, int dimh) {
	char	*p, *s;

	if (email == NULL)
		return NULL_POINTER;
	if (strlen(email) == 0)
		return EMPTY_STRING;

	s = strdup(email);
	p = strstr(s, "\n\n");

	if ((strlen(email) - strlen(p)) > dimh)
		return BUFFER_TOO_SMALL;

	bzero(headers, dimh);
	if (strlen(p) > 1)
		strncpy(headers, email, strlen(email) - strlen(p));
	else
		return HEADER_NOT_FOUND;

	return RESULT_OK;
}

// funzione che estrae il primo header dagli headers e lo copia in primo_header
int primoHeader(char * headers, char * primo_header, int dim) {
	char	*p, *s, *x, *hd;
	int	ok = 0;

	if (headers == NULL || primo_header == NULL)
		return NULL_POINTER;
	if (strlen(headers) == 0)
		return EMPTY_STRING;

	hd = (char *)malloc(dim);
	bzero(hd, dim);

	s = strdup(headers);
	ok = 0;
	while (p = strtok(s, "\n")) {
		s = NULL;
		if (p[0] == ' ' || p[0] == '\t') {
			strncat(hd, p, dim - 1);
			continue;
		}
		if (ok) break;

		if (strstr(p, " ") == NULL)
			return BAD_HEADER;

		strncpy(hd, p, dim - 1);

		ok = 1;
	}
	strncpy(primo_header, hd, dim);
	return RESULT_OK;
}

// funzione che estrae il tipo ed il valore di uno header
int parseHeader(char * header, char * tipo, int dimt, char * valore, int dimv) {
	char *p, *x;

	if (header == NULL)
		return NULL_POINTER;
	if (strlen(header) == 0)
		return EMPTY_STRING;

	x = strtok(strdup(header), " ");
	if (sizeof(x) > dimt)
		return BUFFER_TOO_SMALL;
	strncpy(tipo, x, dimt);

	p = strchr(header, ' ');
	if (sizeof(p) > dimv)
		return BUFFER_TOO_SMALL;
	strncpy(valore, p, dimv);

	return RESULT_OK;
}

// Funzione che estrae la prima parola (alfanumerica)
// da una stringa e la copia dentro parola con un limite
// sulla dimensione di dim caratteri
int primaParola(char * testo, char * parola, int dim) {
	char	w[dim];
	int	ok = 0, wx = 0, x = 0;

	if (testo == NULL)
		return NULL_POINTER;
	if (strlen(testo) == 0)
		return EMPTY_STRING;

	bzero(w, dim);
	for (x = 0; x < strlen(testo); x++) {
		if (isalnum(testo[x]))
			ok = 1;
		else {
			if (ok) break;
			ok = 0;
		}
		if (ok) {
			w[wx] = testo[x];
			wx++;
		}
	}

	if (strlen(w) < 1)
		return BUFFER_TOO_SMALL;
	strncpy(parola, w, dim);
	return RESULT_OK;
}

// funzione che estrae i primi dimT token dalla email
int tokenizzaEmail(char * email, Token tokenArray[], int dimT) {
	char	*hd, *tipoh, *valh, *parolah, *headerh, *bodyh;
	int	ret = 0, x = 0, hlen = 0, xwordlen = 0, ok = 0;

	if (email == NULL || tokenArray == NULL)
		return NULL_POINTER;
	if (strlen(email) == 0)
		return EMPTY_STRING;

	hd = (char *)malloc(strlen(email));
	bzero(hd, strlen(email));

	if ((ret = estraiHeaders(email, hd, strlen(email))) != RESULT_OK)
		return ret;
	hlen = strlen(hd);

	headerh = (char *)malloc(strlen(email));
	bzero(headerh, strlen(email));
	while ((ret = primoHeader(hd, headerh, strlen(email))) == RESULT_OK && headerh != NULL) {
		tipoh = (char *)malloc(strlen(headerh));
		valh = (char *)malloc(strlen(headerh));
		bzero(tipoh, strlen(headerh));
		bzero(valh, strlen(headerh));

		if ((ret = parseHeader(headerh, tipoh, strlen(headerh), valh, strlen(headerh))) == RESULT_OK && tipoh != NULL) {
			parolah = (char *)malloc(strlen(headerh));
			bzero(parolah, strlen(headerh));
			while (primaParola(valh, parolah, strlen(headerh)) == RESULT_OK && parolah != NULL) {
				if (x == dimT)
					return BUFFER_TOO_SMALL;

				tokenArray[x].parola = (char *)malloc(strlen(tipoh) + 1);
				bzero(tokenArray[x].parola, strlen(tipoh) + 1);
				strncpy(tokenArray[x].parola, tipoh, strlen(tipoh));

				tokenArray[x].contesto = (char *)malloc(strlen(parolah) + 1);
				bzero(tokenArray[x].contesto, strlen(parolah) + 1);
				strncpy(tokenArray[x].contesto, parolah, strlen(parolah));

				valh = strdup(valh += strlen(parolah) + 1);
				bzero(parolah, strlen(headerh));

				x++;
			}
			free(parolah);
			free(tipoh);
			free(valh);
		}

		hd = strdup(hd += strlen(headerh) + 1);
		bzero(headerh, strlen(email));
	}
	free(headerh);

	email += hlen;
	parolah = (char *)malloc(strlen(email));
	bzero(parolah, strlen(email));
	while (primaParola(email, parolah, strlen(email)) == RESULT_OK && parolah != NULL) {
		if (x == dimT)
			return BUFFER_TOO_SMALL;

		ok = 0;
		for (xwordlen = 0; xwordlen < strlen(email); xwordlen++) {
			if (isalnum(email[xwordlen]))
				ok = 1;
			else {
				if (ok) break;
				ok = 0;
			}
		}

		tokenArray[x].parola = (char *)malloc(strlen(parolah) + 1);
		bzero(tokenArray[x].parola, strlen(parolah) + 1);
		strncpy(tokenArray[x].parola, parolah, strlen(parolah));

		tokenArray[x].contesto = NULL;

		email  += xwordlen;

		bzero(parolah, strlen(email));

		x++;
	}
	free(parolah);
	free(hd);

	return RESULT_OK;
}

// Funzione che stampa i token uno per riga nel formato
// 'parola*contesto' (header)
// 'parola'	     (body)
int printTokens(Token tokenArray[], int dimT) {
	int	x = 0;

	if (tokenArray[0].parola == NULL && tokenArray[0].contesto == NULL)
		return NULL_POINTER;

	for (x = 0; x < dimT; x++) {
		if (tokenArray[x].parola == NULL) break;
		if (tokenArray[x].contesto == NULL)
			printf("%s\n", tokenArray[x].parola);
		else
			printf("%s*%s\n", tokenArray[x].parola, tokenArray[x].contesto);
	}
	return RESULT_OK;
}

