#include "modulo1.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#define DIM_BUFFERS 1000
#define MAX_LETTERS 26
#define MAX_DIGITS 10


/****************************************************************************/

/********************** FUNZIONI DI SERVIZIO ***************************/

/* legge una linea da text[] (fino a '\n' o '\0') */
/* e la copia in line[] senza modificare text[]   */
int readline(char* text, char* line, int* start);
/* conta quanti caratteri vi sono nella stringa in entrata */
int alpha(char* str);
/* controlla che un carattere sia o una lettera (maiuscola o minuscola) o una cifra decimale */
int isaldig(int c);
/***********************************************************************/



/* Questa funzione riceve i tre parametri nomefile, dimensione e testo, che indicano rispettivamente:   */
/*  nomefile: il nome del file da aprire e leggere                                                     */
/*  dimensione: il numero di bytes (meno 1 per lo '\0') che deve essere letto da tale file             */
/*  testo: il puntatore ad una zona di memoria grande dimensione caratteri in cui copiare il testo     */
/* letto.                                                                                               */
/* La funzione deve aprire il file e copiarne il contenuto nella zona di memoria testo.                 */
/* La stringa che viene copiata nella variabile testo deve essere terminata dal carattere '\0'.         */
/* Se la dimensione della stringa testo  minore della dimensione del file, si copiano solo i primi     */
/* caratteri.                                                                                           */
/* Opportuni controlli debbono essere fatti sui parametri per assicurarsi che essi siano corretti.      */
/* Eventuali errori verranno comunicati dalla funzione tornando un codice intero che descrive l'errore. */
/* Si veda il file modulo1.h per il valore ed il significato di tali codici di errore.                */
/* Se la funzione termina senza errori il codice da tornare  il valore RESULT_OK (zero).               */
int leggiFile(char* nomeFile, int dimensione, char* testo)
{
   FILE* lettura;
   struct stat statusfile;

   int i = 0,
       c;

/* controllo errore di NULL POINTER */
   if(nomeFile == NULL || testo == NULL)
      return NULL_POINTER;

/* controllo errore di stringa vuota */
   if(strlen(nomeFile) == 0)
      return EMPTY_STRING;

/* controllo errore dimensione buffer (negativa o nulla) */
   if(dimensione <= 0)
      return BUFFER_TOO_SMALL;

/* controllo errore esistenza file in memoria */
   if(stat(nomeFile, &statusfile) != 0)
   {
   /* il file non esiste in memoria */
      if(errno == ENOENT)
         return FILE_MISSING;
   /* non  stato possibile ottenere informazioni sul file specificato */
      else
         return UNKNOWN_ERROR;
   }

/* apro buffer di sola lettura su file*/
   lettura = fopen(nomeFile, "r");

/* controllo che il file sia stato aperto: se cos non  */
/* allora esso  protetto da lettura, perci lancio una */
/* eccezione di FILE_UNREADABLE che ritorno subito in uscita */
   if(!lettura)
      return FILE_UNREADABLE;

/* leggo il file carattere per carattere e lo memorizzo in testo[] */
   while((c = fgetc(lettura)) != EOF && i < dimensione - 1)
   {
      testo[i] = c;
      i++;
   }
/* termino la stringa appena caricata */
   testo[i] = '\0';

/* la lunghezza della stringa letta\copiata dal file */
/*  di lunghezza 0, quindi il file risulta essere vuoto */
   if(strlen(testo) == 0)
      return FILE_EMPTY;

/* ritorno codice di terminazione corretta della funzione */
   return RESULT_OK;
}



/* Questa funzione riceve in ingresso un blocco di testo che contiene una email e ne copia il blocco      */
/* delle intestazioni nella stringa headers. Gli argomenti sono:                                          */
/*  email: che contiene il testo della email, terminato dal carattere '\0'                               */
/*  headers:  un blocco di memoria grande dimh caratteri in cui copiare le intestazioni della email,    */
/*  dimh: dimensione della stringa headers in caratteri                                                  */
/* La funzione deve leggere dalla email contenuta nel parametro email gli headers e copiarne il           */
/* contenuto nel blocco di memoria headers. La stringa headers dev'essere terminata dal carattere '\0'.   */
/* Si faccia attenzione a non copiare pi caratteri di quanto spazio  disponibile. Se lo spazio dimh non */
/*  sufficiente si torni il giusto codice di errore.                                                     */
/* Se la funzione termina senza errori il codice da tornare  il valore RESULT_OK (zero).                 */
/* Come si distinguono gli headers dal corpo del messaggio? Gli headers sono linee di testo che           */
/* contengono almeno un carattere prima di arrivare all'accapo, sono seguiti da una linea vuota, poi      */
/* segue il corpo del messaggio.                                                                          */
int estraiHeaders(char* email, char* headers, int dimh)
{
   char* line;
   int ce = 0,    /* contatore email[] */
       ch = 0,    /* contatore headers[] */
       cl,        /* contatore line[] */
       lines = 0; /* contatore linee (correttamente) copiate */

/* controllo errore di null pointer */
   if(email == NULL || headers == NULL)
      return NULL_POINTER;

/* controllo errore di stringa vuota */
   if(strlen(email) == 0)
      return EMPTY_STRING;

/* controllo errore dimensione buffer negativa o nulla */
   if(dimh <= 0)
      return BUFFER_TOO_SMALL;

/* alloco memoria per la stringa di appoggio */
   line = (char*) malloc(sizeof(char) * DIM_BUFFERS);

/* controllo errore di allocazione memoria */
   if(line == NULL)
      return MEM_ALLOC_ERROR;

/* leggo la prima riga da email[] e lancio un'eccezione in */
/* caso la funzione readline[] non vada a buon fine */
   if(readline(email, line, &ce) == BUFFER_TOO_SMALL)
      return BUFFER_TOO_SMALL;


/* alla prima riga senza alcun carattere termino la lettura di email[] */
   while(alpha(line) > 0)
   {
      for(cl = 0; cl < strlen(line) && ch < dimh; cl++, ch++)
         headers[ch] = line[cl];

   /* il contatore 'ch'  stato incrementato fino a 'dimh', perci */
   /* il buffer risulta essere troppo piccolo per contenere gli headers */
      if(ch == dimh)
         return BUFFER_TOO_SMALL;

   /* ho copiato con successo un'altra linea di testo */
      lines++;

   /* incremento il contatore di email[] per evitare un loop infinito */
   /* della funzione readline su un carattere di newline; quindi vado a */
   /* verificare se sono alla fine di email[] (o oltre): se si, esco dal ciclo */
      if(++ce >= strlen(email))
         break;

   /* leggo la riga successiva da email[] e lancio un'eccezione */
   /* in caso la funzione readline non sia andata a buon fine */
      if(readline(email, line, &ce) == BUFFER_TOO_SMALL)
         return BUFFER_TOO_SMALL;
   }
/* termino la stringa appena caricata */
   headers[ch] = '\0';

/* se non ho copiato alcuna riga (lines = 0), allora ritorno un'eccezione */
   if(!lines)
      return HEADER_NOT_FOUND;

/* libero la memoria occupata da line[] */
   free(line);


/* ritorno codice di terminazione corretta della funzione */
   return RESULT_OK;
}



int readline(char* text, char* line, int* ct)
{
   int cl;

   for(cl = 0; text[*ct] != '\n' && (*ct) < strlen(text) && cl < DIM_BUFFERS; cl++, (*ct)++)
      line[cl] = text[*ct];

/* controllo errore di BUFFER_TOO_SMALL */
   if(cl == DIM_BUFFERS)
      return BUFFER_TOO_SMALL;

/* assegno a line in ultima posizione o il carattere di newline o quello di terminazione */
   line[cl] = text[*ct];

/* incremento il contatore per porre '\0' alla fine della stringa */
/* e verifico se sono arrivato in fondo al buffer. Se  cos, */
/* lancio immediatamente un'eccezione di BUFFER_TOO_SMALL */
   if(++cl == DIM_BUFFERS)
      return BUFFER_TOO_SMALL;

/* termino con '\0' la stringa appena copiata */
   line[cl] = '\0';

/* ritorno codice di terminazione corretta della funzione */
   return RESULT_OK;
}



int alpha(char* str)
{
/* lunghezza della stringa in entrata */
   int lungz = strlen(str);
/* contatore scansione stringa in entrata */
   int i;
/* contatore dei caratteri */
   int chars = 0;

/* per tutti i caratteri, tranne il carattere di accapo, incremento il contatore */
   for(i = 0; i < lungz; i++)
      if(isaldig(str[i]))
         chars++;

/* ritorno il numero di caratteri al chiamante */
   return chars;
}



int isaldig(int c)
{
   char simbols[] = {"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                     "abcdefghijklmnopqrstuvwxyz"
                     "0123456789"};
   int i;
   int MAX = (MAX_LETTERS * 2) + MAX_DIGITS;

   for(i = 0; i < MAX; i++)
      if(simbols[i] == c)
         return 1;

   return 0;
}



/* Questa funzione riceve in ingresso gli argomenti:                                                     */
/*  headers: stringa che contiene tutti gli headers della email, terminata da '\0'                      */
/*  primo_header: stringa grande dim caratteri in cui copiare il primo header.                          */
/*  dim: dimensione in caratteri della stringa primo_header.                                            */
/* La funzione esamina la stringa degli headers, e copia il primo header nella stringa primo_header.     */
/* La stringa deve essere terminata dal carattere '\0'. Dato che primo_header  grande dim caratteri, si */
/* torni un opportuno codice di errore (vedi modulo1.h) se questi non sono sufficienti.                */
/* La funzione deve controllare i parametri in modo che siano corretti, e tornare un codice di errore se */
/* non lo sono (si veda il file modulo1.h per i codici di errore).                                     */
/* Se la funzione non ha avuto errori si torni il risultato RESULT_OK (zero).                            */
/* Come si riconosce uno header?                                                                         */
/* Uno header inizia con una sequenza di caratteri senza spazi seguita opzionalmente dal carattere ':',  */
/* poi seguita da spazio, poi da testo fino ad un accapo.                                                */
/* Lo header pu continuare sulle righe seguenti se ciascuna riga inizia con un carattere spazio o tab.  */
int primoHeader(char* headers, char* primo_header, int dim)
{
   int ch = 0,  /* contatore headers[] */
       cp = 0;  /* contatore primo_header[] */

/* contatore di linee copiate */
/* (1 - se l'header  su una sola riga, > 1 se l'header si sviluppa su pi linee) */
   int lines = 0;

/* controllo errore NULL_POINTER */
   if(headers == NULL || primo_header == NULL)
      return NULL_POINTER;

/* controllo errore EMPTY_STRING */
   if(strlen(headers) == 0)
      return EMPTY_STRING;

/* controllo errore dimensione negativa o nulla */
   if(dim <= 0)
      return BUFFER_TOO_SMALL;


/* ciclo infinito bloccato da condizioni interne */
   for(;;)
   {
   /* copio una riga fino al carattere di accapo */
   /* controllando di non uscire da headers[] e */
   /* anche di non uscire da primo_header[] */
      while(ch < strlen(headers) && headers[ch] != '\n' && cp < dim)
      {
         primo_header[cp] = headers[ch];
         cp++;
         ch++;
      }

   /* dimensione massima di primo_header[] raggiunta: lancio eccezione */
      if(cp == dim)
         return BUFFER_TOO_SMALL;

   /* copio comunque il carattere di accapo in primo_header[] */
      if(headers[ch] == '\n')
      {
         primo_header[cp] = headers[ch];

      /* la condizione di entrata nel while() sovrastante mi assicura */
      /* una protezione contro un'eventuale uscita da primo_header[] */
      /* dopo questo incremento del suo contatore. Non eseguo controlli. */
         cp++;

      /* incremento il contatore di linee */
         lines++;
      }

   /* avanzo il contatore di headers[] per controllare il prossimo carattere */
   /* e, se sono arrivato alla fine, esco diretamente dal ciclo */
      if(++ch >= strlen(headers))
         break;


   /* se il carattere che incontro ora NON E' un carattere di inizio di */
   /* un'altra riga dello header, allora esco subito perch questo  il */
   /* segnale di inizio di un altro header */
      if(headers[ch] != ' ' && headers[ch] != '\t')
         break;
   }

/* controllo errore di BUFFER_TOO_SMALL */
   if(cp >= dim)
      return BUFFER_TOO_SMALL;

/* termino con '\0' la stringa appena copiata */
   primo_header[cp] = '\0';

/* nel caso lo header considerato esista (cio siano state copiate linee\a), */
/* allora sovrascrivo il carattere di newline finale con il carattere '\0' */
   if(lines >= 1)
      primo_header[--cp] = '\0';

/* controllo errore BAD_HEADER */
   if(strlen(primo_header) == 0)
      return HEADER_NOT_FOUND;


/* ritorno codice di terminazione corretta della funzione */
   return RESULT_OK;
}



/* La funzione deve analizzare un header e spezzarlo nelle due parti: tipo e valore.                        */
/* Nell'esempio precedente lo header From: andrea@sterbini.org deve fornire il tipo From: ed il         */
/* valore andrea@sterbini.org.                                                                            */
/* I parametri della funzione sono:                                                                         */
/*  header: stringa terminata da '\0' che contiene tutto lo header,                                        */
/*  tipo: puntatore ad una stringa di dimensioni dimt in cui copiare il tipo dello header,                 */
/*  dimt: dimensioni della stringa tipo,                                                                   */
/*  valore: puntatore ad una stringa di dimensioni dimv in cui copiare il valore dello header,             */
/*  dimv: dimensioni della stringa valore.                                                                 */
/* Sia la stringa copiata in tipo che quella copiata in valore debbono essere terminate dal carattere '\0'. */
/* Si torni un opportuno codice di errore se il numero di caratteri dimt o dimv non  sufficiente a         */
/* contenere la stringa da copiare. Si torni il valore RESULT_OK (zero) altrimenti.                         */
int parseHeader(char* header, char* tipo, int dimT, char* valore, int dimV)
{
   int ch = 0,  /* contatore header[] */
       ct = 0,  /* contatore tipo[] */
       cv = 0;  /* contatore valore[] */

/* controllo errore NULL_POINTER */
   if(header == NULL || tipo == NULL || valore == NULL)
      return NULL_POINTER;

/* controllo errore EMPTY_STRING */
   if(strlen(header) == 0)
      return EMPTY_STRING;

/* controllo errore dimensioni negative o nulle */
   if(dimT <= 0 || dimV <= 0)
      return BUFFER_TOO_SMALL;

/* FASE 1: ESTRAZIONE DEL TIPO DELLO HEADER */
   while(ch < strlen(header) && header[ch] != ' ' && header[ch] != '\t' && ct < dimT)
   {
      tipo[ct] = header[ch];
      ct++;
      ch++;
   }

/* controllo BUFFER_TOO_SMALL per tipo[] */
   if(ct == dimT)
      return BUFFER_TOO_SMALL;

/* termino con '\0' la stringa appena copiata */
   tipo[ct] = '\0';

/* controllo formattazione dello header */
   if(++ch >= strlen(header))
      return BAD_HEADER;

/* se tipo[] ha lunghezza pari a 0, oppure NESSUN carattere */
/* alfanumerico, lancio un'eccezione di BAD_HEADER */
   if(strlen(tipo) == 0 || alpha(tipo) == 0)
      return BAD_HEADER;

/* FASE 2: ESTRAZIONE VALORE DELLO HEADER */
   while(ch < strlen(header) && cv < dimV)
   {
      valore[cv] = header[ch];
      cv++;
      ch++;
   }

/* controllo errore BUFFER_TOO_SMALL su valore[] */
   if(cv == dimV)
      return BUFFER_TOO_SMALL;

/* termino con '\0' la stringa appena copiata */
   valore[cv] = '\0';

/* se valore[] ha lunghezza pari a 0, lancio un'eccezione di BAD_HEADER */
   if(strlen(valore) == 0 || alpha(valore) == 0)
      return BAD_HEADER;

/* ritorno codice di terminazione corretta della funzione */
   return RESULT_OK;
}



/* La funzione deve estrarre dalla stringa testo (terminata dal carattere '\0') la prima parola e copiarla */
/* nella stringa parola di dimensioni dim caratteri.                                                       */
/* La parola dev'essere terminata dal carattere '\0'.                                                      */
/* Se il numero di caratteri dim non  sufficiente si deve ritornare un opportuno codice di errore (si     */
/* veda il file modulo1.h).                                                                              */
/* Se la funzione termina correttamente si torni il codice RESULT_OK (zero).                               */
/* Come si riconosce una parola?                                                                           */
/* Una parola  una sequenza di caratteri alfanumerici, ogni altro carattere va considerato separatore.    */
/* Una eventuale sequenza di separatori iniziale deve essere ignorata.                                     */
int primaParola(char* testo, char* parola, int dim)
{
   int ct = 0,   /* contatore testo[] */
       cp = 0;   /* contatore parola[] */

/* controllo errore di null pointer */
   if(testo == NULL || parola == NULL)
      return NULL_POINTER;

/* controllo errore di stringa vuota su testo[] */
   if(strlen(testo) == 0)
      return EMPTY_STRING;

/* controllo errore di dimensione buffer negativa o nulla */
   if(dim <= 0)
      return BUFFER_TOO_SMALL;

/* salto tutti i caratteri non alfanumerici */
   while(!isaldig(testo[ct]) && ct < strlen(testo))
      ct++;

/* raggiunta fine di testo[]: pongo '\0' come unico carattere di parola[] */
   if(ct == strlen(testo))
      parola[cp] = testo[ct];

/* copio tutti e soli i caratteri alfanumerici in parola[] */
   while(isaldig(testo[ct]) && ct < strlen(testo) && cp < dim)
   {
      parola[cp] = testo[ct];
      cp++;
      ct++;
   }

/* controllo errore di dimensione  buffer */
   if(cp == dim)
      return BUFFER_TOO_SMALL;

/* termino la stringa appena copiata */
   parola[cp] = '\0';


/* ritorno codice di terminazione corretta della funzione */
   return RESULT_OK;
}



/* La funzione riceve come parametro una stringa contenente una email, ed usando le funzioni           */
/* precedenti la analizza e inserisce nel vettore tokenArray di dimensione dimT i token ricavati dallo */
/* header e dal corpo della email. Se la email contiene pi di dimT tokens si deve tornare il codice   */
/* BUFFER_TOO_SMALL per indicare che altri token sono disponibili.                                     */
/* I token vanno inseriti nel vettore tokenArray nell'ordine in cui appaiono nella email.              */
/* La struttura dati Token (definita nel file "modulo1.h")  formata dai due campi parola e contesto   */
/* (di tipo char *) in cui vanno inseriti il contesto e la parola estratta dalla email.                */
/* Se la parola  stata estratta da uno header il contesto sar il tipo di header.                     */
/* Se la parola  stata estratta dal corpo del messaggio il contesto sar il valore NULL.              */
int tokenizzaEmail(char* email, Token tokenArray[], int dimT)
{
/********** STRINGHE BUFFERS PER TOKENIZZAZIONE EMAIL **********/
   char* headers;       /* stringa per blocco headers (1000 caratteri) */
   char* header;        /* stringa contenitore di uno header (1000 caratteri) */
   char* tipo_header;   /***** stringhe contenitrici uno header *****/
   char* valore_header; /***** parsato (1000 caratteri ciascuna) *****/
   char* body;          /* stringa per corpo email (di fatto SOLO puntatore) */
   char* parola;        /* stringa contenente una parola (500 caratteri) */
/***************************************************************/

/**************** CONTATORI DI SCANSIONE *****************/
   int ce   = 0,   /* contatore per email[] */
       chs  = 0,   /* contatore per headers[] */
       ch   = 0,   /* contatore per header[] */
       cv   = 0,   /* contatore per valore_header[] */
       cb   = 0,   /* contatore per body[] */
       cp   = 0,   /* contatore per parola[] */
       cta  = -1;  /* contatore per tokenArray[] */
/*********************************************************/

/******* CONTATORI INIZIO SUCCESSIVI PASSAGGI DI PARAMETRI STRINGA *******/
   int ne  = 0,  /* contatore prossimo passaggio per email[] */
       nhs = 0,  /* contatore prossimo passaggio per headers[] */
       nv  = 0,  /* contatore prossimo passaggio per valore_header[] */
       nb  = 0;  /* contatore prossimo passaggio per body[] */
/*************************************************************************/

/* contenitore risultato elaborazione di una funzione */
   int errore;


/* controllo errore NULL_POINTER */
   if(email == NULL || tokenArray == NULL)
      return NULL_POINTER;

/* controllo errore EMPTY_STRING */
   if(strlen(email) == 0)
      return EMPTY_STRING;

/* controllo errore dimensione negativa o nulla */
   if(dimT <= 0)
      return BUFFER_TOO_SMALL;


/********** FASE 1: ESTRAZIONE BLOCCO DEGLI HEADERS DALLA EMAIL **********/
   headers = (char*) malloc(sizeof(char) * DIM_BUFFERS);

/* controllo errore allocazione memoria */
   if(!headers)
      return MEM_ALLOC_ERROR;

   errore = estraiHeaders(email, headers, DIM_BUFFERS);

/* se ci sono stati errori di elaborazione, ritorno immediatamente l'errore */
   if(errore)
   {
      free(headers);
      return errore;
   }

/* se la lunghezza di headers[]  pari a 0, vuol dire che non  stato trovato */
/* alcun blocco header nel blocco della email. */
/* POSSIBILE CAUSA: L'email inizia con una riga vuota */
   if(strlen(headers) == 0)
   {
      free(headers);
      return HEADER_NOT_FOUND;
   }

/* calcolo inizio prossimo passaggio di parametro stringa */
   ne += strlen(headers) + 1;

/* se dopo il blocco headers sono arrivato in fondo all'email, essa non ha corpo */
/* perci lancio un'ecezione ed esco immediatamente dalla funzione */
   if(ne == strlen(email))
   {
      free(headers);
      return BODY_NOT_FOUND;
   }

/*************** FASE 1: TERMINATA ***************/


/********** FASE 2: ESTRAZIONE SINGOLI HEADER E LORO SPEZZETTAMENTO **********/
   header = (char*) malloc(sizeof(char) * DIM_BUFFERS);
   tipo_header = (char*) malloc(sizeof(char) * DIM_BUFFERS);
   valore_header = (char*) malloc(sizeof(char) * DIM_BUFFERS);
   parola = (char*) malloc(sizeof(char) * (DIM_BUFFERS / 2));

/* controllo errori allocazione memoria */
   if(!header || !tipo_header || !valore_header || !parola)
   {
      free(headers);
      return MEM_ALLOC_ERROR;
   }

/* FASE 2.A: estraggo i singoli header da headers[] e li copio in primo_header[] */
   for(;nhs < strlen(headers);)
   {
      errore = primoHeader(headers + nhs, header, DIM_BUFFERS);

   /* se ci sono stati errori di elaborazione, ritorno immediatamente l'errore */
      if(errore)
      {
      /* libero tutta la memoria allocata */
         free(headers);
         free(header);
         free(tipo_header);
         free(valore_header);
         free(parola);
         return errore;
      }
   /* memorizzo il prossimo punto di inizio per header[] da passare alla funzione */
   /* agiungendo 1 per anare oltre il carattere di newline */
      nhs += strlen(header) + 1;

   /* FASE 2.B: spezzo lo header estratto in tipo e valore */
      errore = parseHeader(header, tipo_header, DIM_BUFFERS, valore_header, DIM_BUFFERS);

   /* se ci sono stati errori di elaborazione, ritorno immediatamente l'errore */
      if(errore)
      {
      /* libero tutta la memoria allocata */
         free(headers);
         free(header);
         free(tipo_header);
         free(valore_header);
         free(parola);
         return errore;
      }

   /* FASE 2.C: spezzo il valore dello header in singole parole */

      nv = 0;  /* azzero il contatore di inizio di valore_header[]...IMPORTANTE! */
      for(;nv < strlen(valore_header);)
      {
      /* salto tutti i caratteri NON ALFANUMERICI prima della prossima parola */
         while(!isalnum(valore_header[nv]) && nv < strlen(valore_header))
            nv++;

      /* se sono arrivato alla fine di valore_header[], vuol dire che non ci */
      /* sono altre parole, perci esco dal ciclo subito (onde evitare errori) */
         if(nv == strlen(valore_header))
            break;

         errore = primaParola(valore_header + nv, parola, DIM_BUFFERS / 2);

      /* se ci sono stati errori di elaborazione, ritorno immediatamente l'errore */
         if(errore)
         {
         /* libero tutta la memoria allocata */
            free(headers);
            free(header);
            free(tipo_header);
            free(valore_header);
            free(parola);
            return errore;
         }
      /* memorizzo il prossimo punto di inizio per valore_header[] da passare alla funzione */
         nv += strlen(parola) + 1;

      /* FASE 2.C.A: copio i valori acquisiti in un nuovo token nell'array */

      /* considero la parola trovata SOLO se essa ha una lunghezza */
      /* pari o superiore a 2 caratteri alfanumerici */
         if(strlen(parola) >= 2)
         {
         /* incremento il contatore dei token per vedere se sono in fondo al buffer disponibile; */
         /* se  cos, lancio un'eccezione di BUFFER_TOO_SMALL, segnalando che ci sono altri token */
            if(++cta >= dimT)
            {
            /* libero tutta la memoria allocata */
               free(headers);
               free(header);
               free(tipo_header);
               free(valore_header);
               free(parola);
               return BUFFER_TOO_SMALL;
            }

         /* alloco memoria per il nuovo token */
            tokenArray[cta].contesto = (char*) malloc(sizeof(char) * (strlen(tipo_header) + 1));
            tokenArray[cta].parola = (char*) malloc(sizeof(char) * (strlen(parola) + 1));

         /* controllo errori allocazione memoria */
            if(!tokenArray[cta].contesto || !tokenArray[cta].parola)
            {
            /* libero tutta la memoria allocata */
               free(headers);
               free(header);
               free(tipo_header);
               free(valore_header);
               free(parola);
               return MEM_ALLOC_ERROR;
            }

          /* copio il valore del tipo dello header nel contesto del token */
          /* e la parola nel campo corrispondente del token */
            strcpy(tokenArray[cta].contesto, tipo_header);
            strcpy(tokenArray[cta].parola, parola);
         } /***** FASE 2.C.A: TERMINATA *****/
      } /***** FASE 2.C: TERMINATA *****/
   } /***** FASE 2.A: TERMINATA *****/

/* libero tutta la memoria allocata ora non pi utile */
   free(headers);
   free(header);
   free(tipo_header);
   free(valore_header);

/*************** FASE 2: TERMINATA ***************/


/********** FASE 3: ESTRAZIONE CORPO DELLA EMAIL E SUO SPEZZETTAMENTO **********/

/* posiziono body alla fine del blocco degli headers */
   body = email + ne;

/* puntando alla locazione di memoria precedentemente assegnata a body */
/* sono arrivato alla fine della email, ma questo vuol dire che, dopo  */
/* gli headers, non c' altro (il corpo): quindi, lancio un'eccezione  */
   if(strlen(body) == 0)
      return BODY_NOT_FOUND;

/* FASE 3.A: spezzo body[] in singole parole passandole ai token */
   for(;nb < strlen(body);)
   {
   /* salto tutti i caratteri NON ALFANUMERICI prima della prossima parola */
      while(!isalnum(body[nb]) && nb < strlen(body))
         nb++;

   /* se sono arrivato alla fine di body[], esco immediatamente dal ciclo */
      if(nb == strlen(body))
         break;

      errore = primaParola(body + nb, parola, DIM_BUFFERS / 2);

   /* se ci sono stati errori di elaborazione dati, ritorno immediatamente l'errore */
      if(errore)
      {
      /* libero la memoria allocata */
         free(parola);
         return errore;
      }

   /* memorizzo inizio prossimo passaggio di parametro stringa */
      nb += strlen(parola) + 1;

   /* considero la parola estratta SOLO se essa ha una lunghezza */
   /* pari o superiore a 2 caratteri alfanumerici */
      if(strlen(parola) >= 2)
      {
      /* incremento il contatore dei token per vedere se sono in fondo al buffer disponibile; */
      /* se  cos, lancio un'eccezione di BUFFER_TOO_SMALL, segnalando che ci sono altri token */
         if(++cta == dimT)
         {
            free(parola);
            return BUFFER_TOO_SMALL;
         }

      /* alloco memoria per il nuovo token, assegnando per al contesto NULL, */
      /* visto che la parola  stata estrapolata dal corpo della email */
         tokenArray[cta].contesto = NULL;
         tokenArray[cta].parola = (char*) malloc(sizeof(char) * (strlen(parola) + 1));

      /* controllo errore allocazione memoria */
         if(tokenArray[cta].parola == NULL)
            return MEM_ALLOC_ERROR;

      /* copio il contenuto della parola estratta nel token allocato */
         strcpy(tokenArray[cta].parola, parola);
      }
   } /***** FASE 3.A: TERMINATA *****/

/*************** FASE 3: TERMINATA ***************/


/********** FASE 4: AGGIUSTO IL VETTORE DI TOKEN **********/
/*** (solo se ha ALMENO un'altra posizone disponibile) ***/
   if(++cta < dimT)
   {
      tokenArray[cta].contesto = NULL;
      tokenArray[cta].parola = NULL;
   }
/*************** FASE 4: TERMINATA **************/


/* ritorno codice di terminazione corretta della funzione */
   return RESULT_OK;
}



/* La funzione stampa su stdout (una per ciascuna riga e nell'ordine di apparizione) le stringhe      */
/* tipo*parola ottenute dai Token passati nel vettore tokenArray concatenando il tipo con la parola */
/* del token. Per i token estratti dal corpo del messaggio (quelli con tipo == NULL) va stampata, una */
/* per riga, solo la parola.                                                                          */
int printTokens(Token tokenArray[], int dimT)
{
   int i;

/* controllo errore di NULL_POINTER */
   if(tokenArray == NULL)
      return NULL_POINTER;

/* controllo errore dimensione negativa o nulla */
   if(dimT <= 0)
      return BUFFER_TOO_SMALL;

/* stampo tokens memorizzati */
   for(i = 0; (i < dimT && (tokenArray[i].contesto != NULL || tokenArray[i].parola != NULL)); i++)
   {
   /* controllo se devo stampare il contesto */
      if(tokenArray[i].contesto != NULL)
      {
      /* controllo errore stringa vuota */
         if(strlen(tokenArray[i].contesto) == 0)
            return EMPTY_STRING;
         else
      /* stampo contesto */
           printf("%s*",tokenArray[i].contesto);
      }

   /* controllo errore di null pointer */
      if(tokenArray[i].parola == NULL)
         return NULL_POINTER;
   /* controllo errore di stringa vuota */
      else if(strlen(tokenArray[i].parola) == 0)
         return EMPTY_STRING;
   /* stampo parola */
      else
         printf("%s\n",tokenArray[i].parola);
   }

/* ritorno codice di terminazione corretta della funzione */
   return RESULT_OK;
}


