#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "testModulo1.h"


#define DIM_BUFFER 1000
#define DIM_BUFFER_SMALL (DIM_BUFFER / 50)
#define ERRORS 13
#define TESTS 15
#define ARGUMENTS 5




/* funzione di test per leggiFile(...) */
int test_leggiFile(int tipo_di_test, int argomento)
{
   char* tests[ERRORS] = {"RESULT_OK",
                          "UNKNOWN_ERROR",
                          "NULL_POINTER",
                          "EMPTY_STRING",
                          "FILE_MISSING",
                          "FILE_UNREADABLE",
                          "FILE_UNWRITABLE",
                          "FILE_EMPTY",
                          "BUFFER_TOO_SMALL",
                          "HEADER_NOT_FOUND",
                          "BODY_NOT_FOUND",
                          "BAD_HEADER",
                          "MEM_ALLOC_ERROR"};

   char* description[TESTS] = {"La funzione ritorna un codice RESULT_OK, ma non svolge correttamente il suo lavoro\n",
                               "Errore di tipo sconosciuto\n",
                               "La funzione non controlla se riceve un puntatore nullo\n",
                               "La funzione non controlla se riceve una stringa vuota\n",
                               "La funzione non controlla l'esistenza del file\n",
                               "La funzione non controlla il permesso di lettura del file\n",
                               "La funzione non controlla il permesso di scrittura sul file\n",
                               "La funzione non controlla se il file letto  vuoto\n",
                               "La funzione non controlla se il buffer  sufficientemene capiente\n",
                               "La funzione non controlla l'esistenza dello header nella email\n",
                               "La funzione non controlla l'esistenza del corpo nella email\n",
                               "La funzione non controlla la giusta sintassi di uno header\n",
                               "La funzione non controlla l'allocazione della memoria\n",
                               "La funzione ritorna un codice esatto anche per parametri errati\n",
                               "La funzione ritorna un codice di errore anche per parametri corretti\n"};

    char* argposition[ARGUMENTS] = {"Primo",
                                    "Secondo",
                                    "Terzo",
                                    "Quarto",
                                    "Quinto"};

/* memorizzano i risultati restituiti dalla funzione */
   int test,
       descrizione = tipo_di_test;
/* stringa usata per i test: allocata dinamicamente */
   char* buffer;
/* segnala un codice di ritorno corretto ma malfunzionamento della funzione */
   int BAD_RESULT_OK = 0;

/* controllo argomento da testare */
   switch(argomento)
   {
   /* argomento considerato: nomeFile[] */
      case 0:
         /* alloco memoria per un buffer di test */
            buffer = (char*) malloc(sizeof(char) * DIM_BUFFER);
         /* sporco il buffer */
            strcpy(buffer, "sdgbfbaeowrh");

         /* allocazione di memoria fallita */
            if(!buffer)
               return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               test = leggiFile("email1.txt", DIM_BUFFER, buffer);

          /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO:\nFunzione testata leggiFile(\"email1.txt\", 1000, buffer)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzione NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer, "From: andrea@sterbini.org\n"
                                    "To: babbo_natale@northpole.org\n"
                                    "Subject: Cosa vorrei per Natale\n"
                                    "\n"
                                    "Caro Babbo Natale, \n"
                                    "quest'anno ho cercato di essere molto buono e di non bocciare nessuno, ma talvolta ho dovuto farlo.\n"
                                    "\n"
                                    "Ti prego di portarmi in regalo un Lego Mindstorms per poter insegnare ai ragazzi del corso a programmare in microcodice Lego.\n"
                                    "\n"
                                    "Ciao\n"
                                    "Andrea") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro con parametri corretti */
               /* controllo se la funzione svolge correttamente il suo lavoro con parametri errati */
                  else
                  {
                     test = leggiFile(NULL, DIM_BUFFER, buffer);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* nessun errore  stato rilevato, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               test = leggiFile(NULL, DIM_BUFFER, buffer);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata leggiFile(NULL, 1000, buffer)\n");
            break;

         /* risultato atteso: EMPTY_STRING */
            case EMPTY_STRING:
               test = leggiFile("", DIM_BUFFER, buffer);

             /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata leggiFile(\"\", 1000, buffer)\n");
            break;

         /* risultato atteso: FILE_MISSING */
            case FILE_MISSING:
               test = leggiFile("ciulizzzneeeciulizzz.woo", DIM_BUFFER, buffer);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata leggiFile(\"ciulizzzzzneeeciulizzzzz.woo\", 1000, buffer)\n");
            break;

         /* risultato atteso: FILE_EMPTY */
            case FILE_EMPTY:
            /* apro e chiudo un file che risulter vuoto */
               fclose(fopen("empty.prv", "w"));

               test = leggiFile("empty.prv", DIM_BUFFER, buffer);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata leggiFile(\"empty.prv\", 1000, buffer)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case FILE_UNWRITABLE:
            case BUFFER_TOO_SMALL:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;


         /* tipo di test non implementato */
            default:
               return NOT_IMPLEMENTED;
         }
      /* libero la memoria occupata da buffer[] */
         free(buffer);
      break;


   /* argomento considerato: dimensione */
      case 1:
         /* alloco memoria per un buffer di test */
            buffer = (char*) malloc(sizeof(char) * DIM_BUFFER);

         /* allocazione di memoria fallita */
            if(!buffer)
               return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               test = leggiFile("email1.txt", DIM_BUFFER, buffer);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata leggiFile(\"email1.txt\", 1000, buffer)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzione NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer, "From: andrea@sterbini.org\n"
                                    "To: babbo_natale@northpole.org\n"
                                    "Subject: Cosa vorrei per Natale\n"
                                    "\n"
                                    "Caro Babbo Natale, \n"
                                    "quest'anno ho cercato di essere molto buono e di non bocciare nessuno, ma talvolta ho dovuto farlo.\n"
                                    "\n"
                                    "Ti prego di portarmi in regalo un Lego Mindstorms per poter insegnare ai ragazzi del corso a programmare in microcodice Lego.\n"
                                    "\n"
                                    "Ciao\n"
                                    "Andrea") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro con parametri corretti */
               /* controllo se la funzione svolge correttamente il suo lavoro con parametri errati */
                  else
                  {
                     test = leggiFile("email1.txt", -1, buffer);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* nessun errore  stato rilevato, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: BUFFER_TOO_SMALL */
            case BUFFER_TOO_SMALL:
               test = leggiFile("email1.txt", -1, buffer);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata leggiFile(\"email1.txt\", -1, buffer)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case NULL_POINTER:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* tipo di test non implementato */
            default:
               return NOT_IMPLEMENTED;
         }
      /* libero la memoria occupata da buffer[] */
         free(buffer);
      break;


   /* argomento considerato: testo[] */
      case 2:
         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
            /* allocazione memoria per n buffer di test */
               buffer = (char*) malloc(sizeof(char) * DIM_BUFFER);

            /* allocazione memoria fallita */
               if(!buffer)
                  return MEM_ALLOC_ERROR;

               test = leggiFile("email1.txt", DIM_BUFFER, buffer);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata leggiFile(\"email1.txt\", 1000, buffer)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzione NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer, "From: andrea@sterbini.org\n"
                                    "To: babbo_natale@northpole.org\n"
                                    "Subject: Cosa vorrei per Natale\n"
                                    "\n"
                                    "Caro Babbo Natale, \n"
                                    "quest'anno ho cercato di essere molto buono e di non bocciare nessuno, ma talvolta ho dovuto farlo.\n"
                                    "\n"
                                    "Ti prego di portarmi in regalo un Lego Mindstorms per poter insegnare ai ragazzi del corso a programmare in microcodice Lego.\n"
                                    "\n"
                                    "Ciao\n"
                                    "Andrea") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro con parametri corretti */
               /* controllo se la funzione svolge correttamente il suo lavoro con parametri errati */
                  else
                  {
                     test = leggiFile("email1.txt", DIM_BUFFER, NULL);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* nessun errore  stato rilevato, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }

            /* libero la memoria occupata da buffer[] */
               free(buffer);
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               test = leggiFile("email1.txt", DIM_BUFFER, NULL);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata leggiFile(\"email1.txt\", 1000, NULL)\n");
            break;

         /* test insensati per questo argomento */
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* tipo di test non implementato */
            default:
               return NOT_IMPLEMENTED;
         }
      break;

   /* numero argomento non valido */
      default:
         return ARG_MISSING;
   }

/* test passato: risultato atteso corretto */
   if(tipo_di_test == test)
      return TEST_PASSED;


/* TEST FALLITO: visualizzazione avviso */
   if(!BAD_RESULT_OK)
   {
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }
   else
/* TEST APPARENTEMENTE CORRETTO: la funzione ha ritornato RESULT_OK, ma non svolge correttamente il suo lavoro */
   {
      printf("TEST APPARENTEMENTE CORRETTO\n");
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }


/* ritorno codice di svolgimento test corretto */
   return TEST_FAILED;
}




/* funzione di test per estraiHeaders(...) */
int test_estraiHeaders(int tipo_di_test, int argomento)
{
   char* tests[TESTS] = {"RESULT_OK",
                         "UNKNOWN_ERROR",
                         "NULL_POINTER",
                         "EMPTY_STRING",
                         "FILE_MISSING",
                         "FILE_UNREADABLE",
                         "FILE_UNWRITABLE",
                         "FILE_EMPTY",
                         "BUFFER_TOO_SMALL",
                         "HEADER_NOT_FOUND",
                         "BODY_NOT_FOUND",
                         "BAD_HEADER",
                         "MEM_ALLOC_ERROR"};

   char* description[TESTS] = {"La funzione ritorna un codice RESULT_OK, ma non svolge correttamente il suo lavoro\n",
                               "Errore di tipo sconosciuto\n",
                               "La funzione non controlla se riceve un puntatore nullo\n",
                               "La funzione non controlla se riceve una stringa vuota\n",
                               "La funzione non controlla l'esistenza del file\n",
                               "La funzione non controlla il permesso di lettura del file\n",
                               "La funzione non controlla il permesso di scrittura sul file\n",
                               "La funzione non controlla se il file letto  vuoto\n",
                               "La funzione non controlla se il buffer  sufficientemene capiente\n",
                               "La funzione non controlla l'esistenza dello header nella email\n",
                               "La funzione non controlla l'esistenza del corpo nella email\n",
                               "La funzione non controlla la giusta sintassi di uno header\n",
                               "La funzione non controlla l'allocazione della memoria\n",
                               "La funzione ritorna un codice esatto anche per parametri errati\n",
                               "La funzione ritorna un codice di errore anche per parametri corretti\n"};

    char* argposition[ARGUMENTS] = {"Primo",
                                    "Secondo",
                                    "Terzo",
                                    "Quarto",
                                    "Quinto"};

/* memorizzano i risultati restituiti dalla funzione   */
   int test,
       descrizione = tipo_di_test;
/* segnala un codice di ritorno corretto ma malfunzionamento della funzione */
   int BAD_RESULT_OK = 0;
/* stringhe usate per i test: allocate dinamicamente */
   char *buffer1,
        *buffer2;

/* controllo argomento da testare */
   switch(argomento)
   {
   /* argomento considerato: email[] */
      case 0:
      /* allocazione due buffer di prova */
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer1 || !buffer2)
            return MEM_ALLOC_ERROR;

      /* copio nei buffer stringhe di prova */
         strcpy(buffer1, "From: davidbenedetti@dav.it\n\n");
         strcpy(buffer2, "hfrtnftfdtbdt");

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               test = estraiHeaders(buffer1, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: estraiHeaders(\"From: davidbenedetti@dav.it\", \"hfrtnftfdtbdt\", 1000)\n");
                  descrizione = 14;
               }
            /* verifico che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzione non svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "From: davidbenedetti@dav.it\n") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro */
               /* controllo che la funzione svolga il suo lavoro anche con parametri errati */
                  else
                  {
                     test = estraiHeaders(NULL, buffer2, DIM_BUFFER);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* nessun errore  stato rilevato, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               test = estraiHeaders(NULL, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: estraiHeaders(NULL, \"hfrtnftfdtbdt\", 1000)\n");
            break;

         /* risultato atteso: EMPTY_STRING */
            case EMPTY_STRING:
               test = estraiHeaders("", buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALITO\nFunzione testata: estraiHeaders(\"\", \"hfrtnftfdtbdt\", 1000)\n");
            break;

         /* risultato atteso: HEADER_NOT_FOUND */
            case HEADER_NOT_FOUND:
               strcpy(buffer2, "bnghewrds");

            /* copio nel primo buffer una stringa in cui vi sia subito un riga vuota */
               strcpy(buffer1, "\nFrom: davidbenedetti@dav.it\n\n");

               test = estraiHeaders(buffer1, buffer2, DIM_BUFFER);

               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: estraiHeaders(\" From: davidbenedetti@dav.it\", \"bnghewrds\", 1000)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case BUFFER_TOO_SMALL:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* tipo di test non implementato */
            default:
               return NOT_IMPLEMENTED;
         }

     /* libero memoria occupata dai due buffer */
         free(buffer1);
         free(buffer2);
      break;

   /* argomento considerato: headers[] */
      case 1:
      /* allocazione dinamica memoria per buffer di test */
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer1)
            return MEM_ALLOC_ERROR;

         strcpy(buffer1, "From: davidbenedetti@dav.it\n\n");

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);

            /* controllo errore di allocazione memoria */
               if(!buffer2)
                  return MEM_ALLOC_ERROR;

               strcpy(buffer2, "vdjmlofoerfdje");

               test = estraiHeaders(buffer1, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: estraiHeaders(\"From: davidbenedetti@dav.it\", \"vdjmlofoerfdje\", 1000)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzione non svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "From: davidbenedetti@dav.it\n") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro */
                  else
                  {
                     test = estraiHeaders(buffer1, NULL, DIM_BUFFER);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* nessun errore  stato rilevato, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }

            /* libero la memoria occupata dal secondo buffer */
               free(buffer2);
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               test = estraiHeaders(buffer1, NULL, DIM_BUFFER);

               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: estraiHeaders(\"From: davidbenedetti@dav.it\", NULL, 1000)\n");
            break;

         /* risultato atteso: BUFFER_TO_SMALL */
            case BUFFER_TOO_SMALL:
               buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER_SMALL);

            /* controllo errore allocazione memoria */
               if(!buffer2)
                  return MEM_ALLOC_ERROR;

               strcpy(buffer2, "bnghewrds");

               test = estraiHeaders(buffer1, buffer2, DIM_BUFFER_SMALL);

               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: estraiHeaders(\"From: davidbenedetti@dav.it\", \"bnghewrds\", 20)\n");

            /* libero la memoria occupata dal secondo buffer */
               free(buffer2);
            break;

         /* test insensati per questo argomento */
            case EMPTY_STRING:
            case UNKNOWN_ERROR:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;

         /* test non implementato */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria occupata dal primo buffer */
         free(buffer1);
      break;

   /* argomento considerato: dimh */
      case 2:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);

         strcpy(buffer1, "From: davidbenedetti@dav.it\n\n");
         strcpy(buffer2, "fddfbfbnrieef");

      /* controllo errore di allocazione memoria */
         if(!buffer1 || !buffer2)
            return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               test = estraiHeaders(buffer1, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: estraiHeaders(\"From: davidbenedetti@dav.it\", \"fddfbfbnrieef\", 1000)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzione non svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "From: davidbenedetti@dav.it\n") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro */
                  else
                  {
                     test = estraiHeaders(buffer1, buffer2, -1);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* nessun errore  stato rilevato, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: BUFFER_TO_SMALL */
            case BUFFER_TOO_SMALL:
               test = estraiHeaders(buffer1, buffer2, -1);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: estraiHeaders(\"From: davidbenedetti@dav.it\", \"fddfbfbnrieef\", -1)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case NULL_POINTER:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;

         /* test non implementato */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria occupata dai due buffer di test */
         free(buffer1);
         free(buffer2);
      break;

   /* argomento non presente */
      default:
         return ARG_MISSING;
   }

/* test passato: risultato atteso corretto */
   if(tipo_di_test == test && !BAD_RESULT_OK)
      return TEST_PASSED;


/* TEST FALLITO: visualizzazione avviso */
   if(!BAD_RESULT_OK)
   {
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }
   else
/* TEST APPARENTEMENTE CORRETTO: la funzione ha ritornato RESULT_OK, ma non svolge correttamente il suo lavoro */
   {
      printf("TEST APPARENTEMENTE CORRETTO\n");
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }


/* ritorno codice di svolgimento test corretto */
   return TEST_FAILED;
}



/* funzione di test per int primoHeader(...) */
int test_primoHeader(int tipo_di_test, int argomento)
{
   char* tests[TESTS] = {"RESULT_OK",
                         "UNKNOWN_ERROR",
                         "NULL_POINTER",
                         "EMPTY_STRING",
                         "FILE_MISSING",
                         "FILE_UNREADABLE",
                         "FILE_UNWRITABLE",
                         "FILE_EMPTY",
                         "BUFFER_TOO_SMALL",
                         "HEADER_NOT_FOUND",
                         "BODY_NOT_FOUND",
                         "BAD_HEADER",
                         "MEM_ALLOC_ERROR"};

   char* description[TESTS] = {"La funzione ritorna un codice RESULT_OK, ma non svolge correttamente il suo lavoro\n",
                               "Errore di tipo sconosciuto\n",
                               "La funzione non controlla se riceve un puntatore nullo\n",
                               "La funzione non controlla se riceve una stringa vuota\n",
                               "La funzione non controlla l'esistenza del file\n",
                               "La funzione non controlla il permesso di lettura del file\n",
                               "La funzione non controlla il permesso di scrittura sul file\n",
                               "La funzione non controlla se il file letto  vuoto\n",
                               "La funzione non controlla se il buffer  sufficientemene capiente\n",
                               "La funzione non controlla l'esistenza dello header nella email\n",
                               "La funzione non controlla l'esistenza del corpo nella email\n",
                               "La funzione non controlla la giusta sintassi di uno header\n",
                               "La funzione non controlla l'allocazione della memoria\n",
                               "La funzione ritorna un codice esatto anche per parametri errati\n",
                               "La funzione ritorna un codice di errore anche per parametri corretti\n"};

    char* argposition[ARGUMENTS] = {"Primo",
                                    "Secondo",
                                    "Terzo",
                                    "Quarto",
                                    "Quinto"};

/* memorizzano i risultati restituiti dalla funzione   */
   int test,
       descrizione = tipo_di_test;
/* segnala un codice di ritorno corretto ma malfunzionamento della funzione */
   int BAD_RESULT_OK = 0;
/* stringhe usate per i test: allocate dinamicamente */
   char *buffer1,
        *buffer2;


/* ARGOMENTI */
   switch(argomento)
   {
   /* argomento considerato: headers[] */
      case 0:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer1 || !buffer2)
            return MEM_ALLOC_ERROR;

      /* 0: TESTS */
         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer1, "From: davidbenedetti@dav.it\n davidbenedetti151.100.XXX\n\trome324450\nTo: eledip@tin.it\nSubject: ciao\n\n");
               strcpy(buffer2, "sdngsewopfjsdfg");

               test = primoHeader(buffer1, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: primoHeader(buffer1, buffer2, 1000)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzione NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "From: davidbenedetti@dav.it\n davidbenedetti151.100.XXX\n\trome324450") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro */
                  else
                  {
                     test = primoHeader(NULL, buffer2, DIM_BUFFER);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* nessun errore  stato rilevato, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               strcpy(buffer2, "rfndbjjhf gch");

               test = primoHeader(NULL, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: primoHeader(NULL, buffer2, 1000)\n");
            break;

         /* risltato atteso: EMPTY_STRING */
            case EMPTY_STRING:
               strcpy(buffer2, "rfndbjjhf gch");

               test = primoHeader("", buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: primoHeader(\"\", buffer2, 1000)\n");
            break;

         /* risultato atteso: BAD_HEADER */
            case HEADER_NOT_FOUND:
               strcpy(buffer1, "\nFrom: davidbenedetti@dav.it");
               strcpy(buffer2, "rfndbjjhf gch");

               test = primoHeader(buffer1, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: primoHeader(buffer1, buffer2, 1000)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case BUFFER_TOO_SMALL:
            case BAD_HEADER:
            case BODY_NOT_FOUND:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato */
            default:
               return NOT_IMPLEMENTED;
         } /* FINE 0: TESTS */

      /* libero la memoria allocata */
         free(buffer1);
         free(buffer2);
      break; /* FINE headers[] */

   /* argomento considerato: primo_header[] */
      case 1:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer1)
            return MEM_ALLOC_ERROR;

      /* 1: TESTS */
         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);

            /* controllo errore allocazione memoria */
               if(!buffer2)
                  return MEM_ALLOC_ERROR;

               strcpy(buffer1, "From: davidbenedetti@dav.it\n davidbenedetti151.100.XXX\n\trome324450\nTo: eledip@tin.it\nSubject: ciao\n\n");
               strcpy(buffer2, "fnvjbjebfrb");

               test = primoHeader(buffer1, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: primoHeader(buffer1, buffer2, 1000)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzione NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "From: davidbenedetti@dav.it\n davidbenedetti151.100.XXX\n\trome324450") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro */
                  else
                  {
                     test = primoHeader(buffer1, NULL, DIM_BUFFER);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* nessun errore  stato rilevato, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               strcpy(buffer1, "From: davidbenedetti@dav.it\n davidbenedetti151.100.XXX\n\trome324450\nTo: eledip@tin.it\nSubject: ciao\n\n");

               test = primoHeader(buffer1, NULL, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: primoHeader(buffer1, NULL, 1000)\n");
            break;

         /* risultato atteso: BUFFER_TOO_SMALL */
            case BUFFER_TOO_SMALL:
               buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER_SMALL);

            /* controllo errore allocazione memoria */
               if(!buffer2)
                  return MEM_ALLOC_ERROR;

               strcpy(buffer1, "From: davidbenedetti@dav.it\n");
               strcpy(buffer2, "srnbngkog");

               test = primoHeader(buffer1, buffer2, DIM_BUFFER_SMALL);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: primoHeader(buffer1, buffer2, 1000)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato */
            default:
               return NOT_IMPLEMENTED;
         } /* FINE 1: TESTS */

      /* libero la memoria allocata */
         free(buffer1);
         free(buffer2);
      break; /* FINE primo_header[] */

   /* argomento considerato: dim */
      case 2:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer1)
            return MEM_ALLOC_ERROR;

      /* TESTS */
         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);
                                                                 
            /* controllo errore allocazione memoria */
               if(!buffer2)
                  return MEM_ALLOC_ERROR;

               strcpy(buffer1, "From: davidbenedetti@dav.it\n davidbenedetti151.100.XXX\n\trome324450\nTo: eledip@tin.it\nSubject: ciao\n\n");
               strcpy(buffer2, "sdnfbgnaegoe");

               test = primoHeader(buffer1, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: primoHeader(buffer1, buffer2, 1000)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzione NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "From: davidbenedetti@dav.it\n davidbenedetti151.100.XXX\n\trome324450") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro */
                  else
                  {
                     test = primoHeader(buffer1, buffer2, -1);

                  /* la funzione NON svolge correttamente il suo lavoro per parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* nessun errore  stato rilevato, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }

            /* libero la memoria allocata */
               free(buffer2);
            break;

         /* risultato atteso: BUFFER_TO_SMALL */
            case BUFFER_TOO_SMALL:
               buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);

            /* controllo errore allocazione memoria */
               if(!buffer2)
                  return MEM_ALLOC_ERROR;

               strcpy(buffer1, "From: davidbenedetti@dav.it\n davidbenedetti151.100.XXX\n\trome324450\nTo: eledip@tin.it\nSubject: ciao\n\n");
               strcpy(buffer2, "sdnfbgnaegoe");

               test = primoHeader(buffer1, buffer2, -1);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: primoHeader(buffer1, buffer2, -1)\n");

               free(buffer2);
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case NULL_POINTER:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato */
            default:
               return NOT_IMPLEMENTED;
         } /* FINE 2: TESTS */

      /* libero la memoria allocata */
         free(buffer1);
      break; /* FINE dim */

   /* argomento non presente */
      default:
         return ARG_MISSING;
   } /* FINE ARGOMENTI */


/* test passato: risultato atteso corretto */
   if(tipo_di_test == test && !BAD_RESULT_OK)
      return TEST_PASSED;


/* TEST FALLITO: visualizzazione avviso */
   if(!BAD_RESULT_OK)
   {
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }
   else
/* TEST APPARENTEMENTE CORRETTO: la funzione ha ritornato RESULT_OK, ma non svolge correttamente il suo lavoro */
   {
      printf("TEST APPARENTEMENTE CORRETTO\n");
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }


/* ritorno codice di svolgimento test corretto */
   return TEST_FAILED;
}



/* funzione di test per parseHeader(...) */
int test_parseHeader(int tipo_di_test, int argomento)
{
   char* tests[TESTS] = {"RESULT_OK",
                         "UNKNOWN_ERROR",
                         "NULL_POINTER",
                         "EMPTY_STRING",
                         "FILE_MISSING",
                         "FILE_UNREADABLE",
                         "FILE_UNWRITABLE",
                         "FILE_EMPTY",
                         "BUFFER_TOO_SMALL",
                         "HEADER_NOT_FOUND",
                         "BODY_NOT_FOUND",
                         "BAD_HEADER",
                         "MEM_ALLOC_ERROR"};

   char* description[TESTS] = {"La funzione ritorna un codice RESULT_OK, ma non svolge correttamente il suo lavoro\n",
                               "Errore di tipo sconosciuto\n",
                               "La funzione non controlla se riceve un puntatore nullo\n",
                               "La funzione non controlla se riceve una stringa vuota\n",
                               "La funzione non controlla l'esistenza del file\n",
                               "La funzione non controlla il permesso di lettura del file\n",
                               "La funzione non controlla il permesso di scrittura sul file\n",
                               "La funzione non controlla se il file letto  vuoto\n",
                               "La funzione non controlla se il buffer  sufficientemene capiente\n",
                               "La funzione non controlla l'esistenza dello header nella email\n",
                               "La funzione non controlla l'esistenza del corpo nella email\n",
                               "La funzione non controlla la giusta sintassi di uno header\n",
                               "La funzione non controlla l'allocazione della memoria\n",
                               "La funzione ritorna un codice esatto anche per parametri errati\n",
                               "La funzione ritorna un codice di errori anche con parametri corretti\n"};

    char* argposition[ARGUMENTS] = {"Primo",
                                    "Secondo",
                                    "Terzo",
                                    "Quarto",
                                    "Quinto"};

/* memorizzano i risultati restituiti dalla funzione   */
   int test,
       descrizione = tipo_di_test;
/* segnala un codice di ritorno corretto ma malfunzionamento della funzione */
   int BAD_RESULT_OK = 0;
/* stringhe usate per i test: allocate dinamicamente */
   char *buffer1,
        *buffer2,
        *buffer3;


/* ARGOMENTI */
   switch(argomento)
   {
   /* argomento considerato: header[] */
      case 0:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer3 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer1 || !buffer2 || !buffer3)
            return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer1, "From: davidbenedetti@dav.it");
               strcpy(buffer2, "nvnefbseoiehg");
               strcpy(buffer3, "efkvbfb sdn a");

               test = parseHeader(buffer1, buffer2, DIM_BUFFER, buffer3, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, buffer2, 1000, buffer3, 1000)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzone NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "From:") != 0 || strcmp(buffer3, "davidbenedetti@dav.it") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro */
                  else
                  {
                     test = parseHeader(NULL, buffer2, DIM_BUFFER, buffer3, DIM_BUFFER);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* non sono stati riscontrati errori, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               test = parseHeader(NULL, buffer2, DIM_BUFFER, buffer3, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: parseHeader(NULL, buffer2, 1000, buffer3, 1000)\n");
            break;

         /* risultato atteso: EMPTY_STRING */
            case EMPTY_STRING:
               test = parseHeader("", buffer2, DIM_BUFFER, buffer3, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: parseHeader(\"\", buffer2, 1000, buffer3, 1000)\n");
            break;

         /* risultato atteso: BAD_HEADER */
            case BAD_HEADER:
               strcpy(buffer1, "\tFrom: davidbenedetti@dav.it");
               strcpy(buffer2, "sdfnjbfg");
               strcpy(buffer3, "fkpowqef");

               test = parseHeader(buffer1, buffer2, DIM_BUFFER, buffer3, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, buffer2, 1000, buffer3, 1000)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case BUFFER_TOO_SMALL:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria allocata */
         free(buffer1);
         free(buffer2);
         free(buffer3);
      break;

   /* argomento considerato: tipo[] */
      case 1:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer3 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer1 || !buffer2 || !buffer3)
            return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer1, "From: davidbenedetti@dav.it");
               strcpy(buffer2, "nvnefbseoiehg");
               strcpy(buffer3, "efkvbfb sdn a");

               test = parseHeader(buffer1, buffer2, DIM_BUFFER, buffer3, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, buffer2, 1000, buffer3, 1000)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzone NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "From:") != 0 || strcmp(buffer3, "davidbenedetti@dav.it") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro */
                  else
                  {
                     test = parseHeader(buffer1, NULL, DIM_BUFFER, buffer3, DIM_BUFFER);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                     else
                  /* non sono stati riscontrati errori, perci faccio in modo di segnalare TEST_PASSED */
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               test = parseHeader(buffer1, NULL, DIM_BUFFER, buffer3, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, NULL, 1000, buffer3, 1000)\n");
            break;

         /* risultato atteso: BUFFER_TOO_SMALL */
            case BUFFER_TOO_SMALL:
               free(buffer2);
               buffer2 = (char*) malloc(sizeof(char) * (DIM_BUFFER_SMALL / 10));

            /* controllo errore allocazione memoria */
               if(!buffer2)
                  return MEM_ALLOC_ERROR;

               strcpy(buffer1, "From: davidbenedetti@dav.it");
               strcpy(buffer2, "jnvjkf");
               strcpy(buffer3, "dfdskn");

               test = parseHeader(buffer1, buffer2, DIM_BUFFER_SMALL / 10, buffer3, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, buffer2, 2, buffer3, 1000)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria allocata */
         free(buffer1);
         free(buffer2);
         free(buffer3);
      break;

   /* argomento considerato: dimt */
      case 2:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer3 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer1 || !buffer2 || !buffer3)
            return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer1, "From: davidbenedetti@dav.it");
               strcpy(buffer2, "nvnefbseoiehg");
               strcpy(buffer3, "efkvbfb sdn a");

               test = parseHeader(buffer1, buffer2, DIM_BUFFER, buffer3, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, buffer2, 1000, buffer3, 1000)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzone NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "From:") != 0 || strcmp(buffer3, "davidbenedetti@dav.it") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro */
                  else
                  {
                     test = parseHeader(buffer1, buffer2, -1, buffer3, DIM_BUFFER);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                     else
                  /* non sono stati riscontrati errori, perci faccio in modo di segnalare TEST_PASSED */
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: BUFFER_TOO_SMALL */
            case BUFFER_TOO_SMALL:
               test = parseHeader(buffer1, buffer2, -1, buffer3, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, buffer2, -1, buffer3, 1000)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case NULL_POINTER:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria allocata */
         free(buffer1);
         free(buffer2);
         free(buffer3);
      break;

   /* argomento considerato: valore[] */
      case 3:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer3 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer1 || !buffer2 || !buffer3)
            return MEM_ALLOC_ERROR;

        switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer1, "From: davidbenedetti@dav.it");
               strcpy(buffer2, "nvnefbseoiehg");
               strcpy(buffer3, "efkvbfb sdn a");

               test = parseHeader(buffer1, buffer2, DIM_BUFFER, buffer3, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, buffer2, 1000, buffer3, 1000)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzone NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "From:") != 0 || strcmp(buffer3, "davidbenedetti@dav.it") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro */
                  else
                  {
                     test = parseHeader(buffer1, buffer2, DIM_BUFFER, NULL, DIM_BUFFER);

                  /* la funzione NON svolge correttamente il suo lavoro per parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* non sono stati riscontrati errori, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               test = parseHeader(buffer1, buffer2, DIM_BUFFER, NULL, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, buffer2, 1000, NULL, 1000)\n");
            break;

         /* risultato atteso: BUFFER_TOO_SMALL */
            case BUFFER_TOO_SMALL:
               free(buffer3);
               buffer3 = (char*) malloc(sizeof(char) * DIM_BUFFER_SMALL);

            /* controllo errore allocazione memoria */
               if(!buffer3)
                  return MEM_ALLOC_ERROR;

               strcpy(buffer1, "From: davidbenedetti@dav.it");
               strcpy(buffer2, "nvnefbseoiehg");
               strcpy(buffer3, "efkvbfb sdn a");

               test = parseHeader(buffer1, buffer2, DIM_BUFFER, buffer3, DIM_BUFFER_SMALL);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, buffer2, 1000, NULL, 1000)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }

      /* lbero la memoria allocata */
         free(buffer1);
         free(buffer2);
         free(buffer3);
      break;

   /* argomento considerato: dimv */
      case 4:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer3 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer1 || !buffer2 || !buffer3)
            return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer1, "From: davidbenedetti@dav.it");
               strcpy(buffer2, "nvnefbseoiehg");
               strcpy(buffer3, "efkvbfb sdn a");

               test = parseHeader(buffer1, buffer2, DIM_BUFFER, buffer3, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, buffer2, 1000, buffer3, 1000)\n");
                  descrizione = 14;
               }
            /* controllo che la funzione abbia svolto correttamente il suo lavoro */
               else
               {
               /* la funzone NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "From:") != 0 || strcmp(buffer3, "davidbenedetti@dav.it") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamente il suo lavoro */
                  else
                  {
                     test = parseHeader(buffer1, buffer2, DIM_BUFFER, buffer3, -1);

                  /* la funzione NON svolge correttamente il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* non sono stati riscontrati errori, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: BUFFER_TOO_SMALL */
            case BUFFER_TOO_SMALL:
               test = parseHeader(buffer1, buffer2, DIM_BUFFER, buffer3, -1);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: parseHeader(buffer1, buffer2, 1000, buffer3, -1)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case NULL_POINTER:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria allocata */
         free(buffer1);
         free(buffer2);
         free(buffer3);
      break;

   /* argomento non presente */
      default:
         return ARG_MISSING;
   }


/* test passato: risultato atteso corretto */
   if(tipo_di_test == test && !BAD_RESULT_OK)
      return TEST_PASSED;


/* TEST FALLITO: visualizzazione avviso */
   if(!BAD_RESULT_OK)
   {
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }
   else
/* TEST APPARENTEMENTE CORRETTO: la funzione ha ritornato RESULT_OK, ma non svolge correttamente il suo lavoro */
   {
      printf("TEST APPARENTEMENTE CORRETTO\n");
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }


/* ritorno codice di svolgimento test corretto */
   return TEST_FAILED;
}



/* funzione di test per primaParola(...) */
int test_primaParola(int tipo_di_test, int argomento)
{
   char* tests[TESTS] = {"RESULT_OK",
                         "UNKNOWN_ERROR",
                         "NULL_POINTER",
                         "EMPTY_STRING",
                         "FILE_MISSING",
                         "FILE_UNREADABLE",
                         "FILE_UNWRITABLE",
                         "FILE_EMPTY",
                         "BUFFER_TOO_SMALL",
                         "HEADER_NOT_FOUND",
                         "BODY_NOT_FOUND",
                         "BAD_HEADER",
                         "MEM_ALLOC_ERROR"};

   char* description[TESTS] = {"La funzione ritorna un codice RESULT_OK, ma non svolge correttamente il suo lavoro\n",
                               "Errore di tipo sconosciuto\n",
                               "La funzione non controlla se riceve un puntatore nullo\n",
                               "La funzione non controlla se riceve una stringa vuota\n",
                               "La funzione non controlla l'esistenza del file\n",
                               "La funzione non controlla il permesso di lettura del file\n",
                               "La funzione non controlla il permesso di scrittura sul file\n",
                               "La funzione non controlla se il file letto  vuoto\n",
                               "La funzione non controlla se il buffer  sufficientemene capiente\n",
                               "La funzione non controlla l'esistenza dello header nella email\n",
                               "La funzione non controlla l'esistenza del corpo nella email\n",
                               "La funzione non controlla la giusta sintassi di uno header\n",
                               "La funzione non controlla l'allocazione della memoria\n",
                               "La funzione ritorna un codice esatto anche per parametri errati\n",
                               "La funzione ritorna un codice di errore anche con parametri corretti\n"};

    char* argposition[ARGUMENTS] = {"Primo",
                                    "Secondo",
                                    "Terzo",
                                    "Quarto",
                                    "Quinto"};

/* memorizzano i risultati restituiti dalla funzione   */
/* test2  usato solo per il secondo test su RESULT_OK */
   int test,
       descrizione = tipo_di_test;
/* segnala un codice di ritorno corretto ma malfunzionamento della funzione */
   int BAD_RESULT_OK = 0;
/* stringhe usate per i test: allocate dinamicamente */
   char *buffer1,
        *buffer2;


/* ARGOMENTI */
   switch(argomento)
   {
   /* argomento considerato: testo[] */
      case 0:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore di allocazione memoria */
         if(!buffer1 || !buffer2)
            return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer1, "%&%&)$$untestperprovareprimaParola12071983ght &/%(()");
               strcpy(buffer2, "efonfgbrfgbeofb");

               test = primaParola(buffer1, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: primaParola(buffer1, buffer2, 1000)\n");
                  descrizione = 14;
               }
            /* controllo se la funzione svolge correttamente il suo lavoro */
               else
               {
               /* la funzione NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "untestperprovareprimaParola12071983ght") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamene il suo lavoro */
                  else
                  {
                     test = primaParola(NULL, buffer2, DIM_BUFFER);

                  /* la svolge NON svolge il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* non sono stati riscontrati errori, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               test = primaParola(NULL, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: primaParola(NULL, buffer2, 1000)\n");
            break;

         /* risultato atteso: EMPTY_STRING */
            case EMPTY_STRING:
               test = primaParola("", buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: primaParola(\"\", buffer2, 1000)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case BUFFER_TOO_SMALL:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria allocata */
         free(buffer1);
         free(buffer2);
      break;

   /* argomento considerato: parola[] */
      case 1:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore di allocazione memoria */
         if(!buffer1 || !buffer2)
            return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer1, "%&%&)$$untestperprovare primaParola12071983ght&/%(()");
               strcpy(buffer2, "efonfgbrfgbeofb");

               test = primaParola(buffer1, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: primaParola(buffer1, buffer2, 1000)\n");
                  descrizione = 14;
               }
               /* controllo se la funzione svolge correttamente il suo lavoro */
               else
               {
               /* la funzione NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "untestperprovare") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamene il suo lavoro */
                  else
                  {
                     test = primaParola(buffer1, NULL, DIM_BUFFER);

                  /* la svolge NON svolge il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* non sono stati riscontrati errori, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               test = primaParola(buffer1, NULL, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: primaParola(buffer1, NULL, 1000)\n");
            break;

         /* risultato atteso: BUFFER_TOO_SMALL */
            case BUFFER_TOO_SMALL:
               free(buffer2);
               buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER_SMALL);

            /* controllo errore allocazione memoria */
               if(!buffer2)
                  return MEM_ALLOC_ERROR;

               strcpy(buffer1, "%&%$&/((&%$ ciulizzzzneeeciulizzzz=)(/&%%/(");
               strcpy(buffer2, "dngjbg");

               test = primaParola(buffer1, buffer2, DIM_BUFFER_SMALL);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: primaParola(buffer1, buffer2, 20)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria allocata */
         free(buffer1);
         free(buffer2);
      break;

   /* argomento considerato: dim */
      case 2:
         buffer1 = (char*) malloc(sizeof(char) * DIM_BUFFER);
         buffer2 = (char*) malloc(sizeof(char) * DIM_BUFFER);

      /* controllo errore di allocazione memoria */
         if(!buffer1 || !buffer2)
            return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer1, "%&%&)$$ primaParola12071983ght&/%(()");
               strcpy(buffer2, "efonfgbrfgbeofb");

               test = primaParola(buffer1, buffer2, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
               {
                  printf("TEST FALLITO\nFunzione testata: primaParola(buffer1, buffer2, 1000)\n");
                  descrizione = 14;
               }
               /* controllo se la funzione svolge correttamente il suo lavoro */
               else
               {
               /* la funzione NON svolge correttamente il suo lavoro */
                  if(strcmp(buffer2, "primaParola12071983ght") != 0)
                  {
                     BAD_RESULT_OK = 1;
                     descrizione = 0;
                  }
               /* la funzione svolge correttamene il suo lavoro */
                  else
                  {
                     test = primaParola(buffer1, buffer2, -1);

                  /* la svolge NON svolge il suo lavoro con parametri errati */
                     if(tipo_di_test == test)
                     {
                        BAD_RESULT_OK = 1;
                        descrizione = 13;
                     }
                  /* non sono stati riscontrati errori, perci faccio in modo di segnalare TEST_PASSED */
                     else
                        test = tipo_di_test;
                  }
               }
            break;

         /* risultato atteso: BUFFER_TOO_SMALL */
            case BUFFER_TOO_SMALL:
               test = primaParola(buffer1, buffer2, -1);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: primaParola(buffer1, buffer2, -1)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case NULL_POINTER:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria allocata */
         free(buffer1);
         free(buffer2);
      break;

   /* argomento non presente */
      default:
         return ARG_MISSING;
   }


/* test passato: risultato atteso corretto */
   if(tipo_di_test == test && !BAD_RESULT_OK)
      return TEST_PASSED;


/* TEST FALLITO: visualizzazione avviso */
   if(!BAD_RESULT_OK)
   {
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }
   else
/* TEST APPARENTEMENTE CORRETTO: la funzione ha ritornato RESULT_OK, ma non svolge correttamente il suo lavoro */
   {
      printf("TEST APPARENTEMENTE CORRETTO\n");
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }


/* ritorno codice di svolgimento test corretto */
   return TEST_FAILED;
}



/* funzione di test per tokenizzaEmail(...) */
int test_tokenizzaEmail(int tipo_di_test, int argomento)
{
   char* tests[TESTS] = {"RESULT_OK",
                         "UNKNOWN_ERROR",
                         "NULL_POINTER",
                         "EMPTY_STRING",
                         "FILE_MISSING",
                         "FILE_UNREADABLE",
                         "FILE_UNWRITABLE",
                         "FILE_EMPTY",
                         "BUFFER_TOO_SMALL",
                         "HEADER_NOT_FOUND",
                         "BODY_NOT_FOUND",
                         "BAD_HEADER",
                         "MEM_ALLOC_ERROR"};

   char* description[TESTS] = {"La funzione ritorna un codice RESULT_OK, ma non svolge correttamente il suo lavoro\n",
                               "Errore di tipo sconosciuto\n",
                               "La funzione non controlla se riceve un puntatore nullo\n",
                               "La funzione non controlla se riceve una stringa vuota\n",
                               "La funzione non controlla l'esistenza del file\n",
                               "La funzione non controlla il permesso di lettura del file\n",
                               "La funzione non controlla il permesso di scrittura sul file\n",
                               "La funzione non controlla se il file letto  vuoto\n",
                               "La funzione non controlla se il buffer  sufficientemene capiente\n",
                               "La funzione non controlla l'esistenza dello header nella email\n",
                               "La funzione non controlla l'esistenza del corpo nella email\n",
                               "La funzione non controlla la giusta sintassi di uno header\n",
                               "La funzione non controlla l'allocazione della memoria\n",
                               "La funzione ritorna un codice esatto anche per parametri errati\n",
                               "La funzione ritorna un codice di errore anche con parametri corretti\n"};

    char* argposition[ARGUMENTS] = {"Primo",
                                    "Secondo",
                                    "Terzo",
                                    "Quarto",
                                    "Quinto"};

/* memorizzano i risultati restituiti dalla funzione   */
/* test2  usato solo per il secondo test su RESULT_OK */
   int test,
       descrizione = tipo_di_test;
/* segnala un codice di ritorno corretto ma malfunzionamento della funzione */
   int BAD_RESULT_OK = 0;
/* stringa usata per i test: allocata dinamicamente */
   char *buffer;
/* array di tokens per memorizzare le singole parole trovate nella email */
   Token* array;


/* ARGOMENTI */
   switch(argomento)
   {
   /* argomento considerato: email[] */
      case 0:
         buffer = (char*) malloc(sizeof(char) * DIM_BUFFER);
         array  = (Token*) malloc(sizeof(Token) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer || !array)
            return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer, "From: andrea@sterbini.org\n"
                              "To: babbo_natale@northpole.org\n"
                              "Subject: Cosa vorrei per Natale\n"
                              "\n"
                              "Caro Babbo Natale,\n"
                              "quest'anno ho cercato di essere molto buono e di non bocciare nessuno, ma talvolta ho dovuto farlo.\n"
                              "\n"
                              "Ti prego di portarmi in regalo un Lego Mindstorms per poter insegnare ai ragazzi del corso a programmare in microcodice Lego.\n"
                              "\n"
                              "Ciao\n"
                              "Andrea");

                test = tokenizzaEmail(buffer, array, DIM_BUFFER);

             /* test fallito */
                if(tipo_di_test != test)
                {
                   printf("TEST FALLITO\nFunzione testata: tokenizzaEmail(buffer, array, 1000)\n");
                   descrizione = 14;
                }
             /* controllo che la funzione abbia eseguito correttamente il suo lavoro */
                else
                {
                /* la funzione NON svolge correttamente il suo lavoro  */
                   if(strcmp(array[0].contesto, "From:") != 0 || strcmp(array[0].parola, "andrea") != 0 ||
                      strcmp(array[1].contesto, "From:") != 0 || strcmp(array[1].parola, "sterbini") != 0 ||
                      strcmp(array[2].contesto, "From:") != 0 || strcmp(array[2].parola, "org") != 0 ||
                      strcmp(array[3].contesto, "To:") != 0 || strcmp(array[3].parola, "babbo") != 0 ||
                      strcmp(array[4].contesto, "To:") != 0 || strcmp(array[4].parola, "natale") != 0 ||
                      strcmp(array[5].contesto, "To:") != 0 || strcmp(array[5].parola, "northpole") != 0 ||
                      strcmp(array[6].contesto, "To:") != 0 || strcmp(array[6].parola, "org") != 0 ||
                      strcmp(array[7].contesto, "Subject:") != 0 || strcmp(array[7].parola, "Cosa") != 0 ||
                      strcmp(array[8].contesto, "Subject:") != 0 || strcmp(array[8].parola, "vorrei") != 0 ||
                      strcmp(array[9].contesto, "Subject:") != 0 || strcmp(array[9].parola, "per") != 0 ||
                      strcmp(array[10].contesto, "Subject:") != 0 || strcmp(array[10].parola, "Natale") != 0 ||
                      array[11].contesto != NULL || strcmp(array[11].parola, "Caro") != 0 ||
                      array[12].contesto != NULL || strcmp(array[12].parola, "Babbo") != 0 ||
                      array[13].contesto != NULL || strcmp(array[13].parola, "Natale") != 0 ||
                      array[14].contesto != NULL || strcmp(array[14].parola, "quest") != 0 ||
                      array[15].contesto != NULL || strcmp(array[15].parola, "anno") != 0 ||
                      array[16].contesto != NULL || strcmp(array[16].parola, "ho") != 0 ||
                      array[17].contesto != NULL || strcmp(array[17].parola, "cercato") != 0 ||
                      array[18].contesto != NULL || strcmp(array[18].parola, "di") != 0 ||
                      array[19].contesto != NULL || strcmp(array[19].parola, "essere") != 0 ||
                      array[20].contesto != NULL || strcmp(array[20].parola, "molto") != 0 ||
                      array[21].contesto != NULL || strcmp(array[21].parola, "buono") != 0 ||
                      array[22].contesto != NULL || strcmp(array[22].parola, "di") != 0 ||
                      array[23].contesto != NULL || strcmp(array[23].parola, "non") != 0 ||
                      array[24].contesto != NULL || strcmp(array[24].parola, "bocciare") != 0 ||
                      array[25].contesto != NULL || strcmp(array[25].parola, "nessuno") != 0 ||
                      array[26].contesto != NULL || strcmp(array[26].parola, "ma") != 0 ||
                      array[27].contesto != NULL || strcmp(array[27].parola, "talvolta") != 0 ||
                      array[28].contesto != NULL || strcmp(array[28].parola, "ho") != 0 ||
                      array[29].contesto != NULL || strcmp(array[29].parola, "dovuto") != 0 ||
                      array[30].contesto != NULL || strcmp(array[30].parola, "farlo") != 0 ||
                      array[31].contesto != NULL || strcmp(array[31].parola, "Ti") != 0 ||
                      array[32].contesto != NULL || strcmp(array[32].parola, "prego") != 0 ||
                      array[33].contesto != NULL || strcmp(array[33].parola, "di") != 0 ||
                      array[34].contesto != NULL || strcmp(array[34].parola, "portarmi") != 0 ||
                      array[35].contesto != NULL || strcmp(array[35].parola, "in") != 0 ||
                      array[36].contesto != NULL || strcmp(array[36].parola, "regalo") != 0 ||
                      array[37].contesto != NULL || strcmp(array[37].parola, "un") != 0 ||
                      array[38].contesto != NULL || strcmp(array[38].parola, "Lego") != 0 ||
                      array[39].contesto != NULL || strcmp(array[39].parola, "Mindstorms") != 0 ||
                      array[40].contesto != NULL || strcmp(array[40].parola, "per") != 0 ||
                      array[41].contesto != NULL || strcmp(array[41].parola, "poter") != 0 ||
                      array[42].contesto != NULL || strcmp(array[42].parola, "insegnare") != 0 ||
                      array[43].contesto != NULL || strcmp(array[43].parola, "ai") != 0 ||
                      array[44].contesto != NULL || strcmp(array[44].parola, "ragazzi") != 0 ||
                      array[45].contesto != NULL || strcmp(array[45].parola, "del") != 0 ||
                      array[46].contesto != NULL || strcmp(array[46].parola, "corso") != 0 ||
                      array[47].contesto != NULL || strcmp(array[47].parola, "programmare") != 0 ||
                      array[48].contesto != NULL || strcmp(array[48].parola, "in") != 0 ||
                      array[49].contesto != NULL || strcmp(array[49].parola, "microcodice") != 0 ||
                      array[50].contesto != NULL || strcmp(array[50].parola, "Lego") != 0 ||
                      array[51].contesto != NULL || strcmp(array[51].parola, "Ciao") != 0 ||
                      array[52].contesto != NULL || strcmp(array[52].parola, "Andrea") != 0)
                   {
                      BAD_RESULT_OK = 1;
                      descrizione = 0;
                   }
                /* la funzione svolge correttamente il suo lavoro */
                   else
                   {
                      test = tokenizzaEmail(NULL, array, DIM_BUFFER);

                   /* la funzione NON svolge il suo lavoro con parametri errati */
                      if(tipo_di_test == test)
                      {
                         BAD_RESULT_OK = 1;
                         descrizione = 13;
                      }
                   /* non sono stati riscontrati errori, perci faccio in modo di segnalare TEST_PASSED */
                      else
                         test = tipo_di_test;
                   }
                }
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               test = tokenizzaEmail(NULL, array, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: tokenizzaEmail(NULL, buffer, 1000)\n");
            break;

         /* risultato atteso: EMPTY_STRING */
            case EMPTY_STRING:
               test = tokenizzaEmail("", array, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: tokenizzaEmail(\"\", buffer, 1000)\n");
            break;

         /* risultato atteso: HEADER_NOT_FOUND */
            case HEADER_NOT_FOUND:
               strcpy(buffer, "\n"
                              "From: andrea@sterbini.org\n"
                              "To: babbo_natale@northpole.org\n"
                              "Subject: Cosa vorrei per Natale\n"
                              "\n"
                              "Caro Babbo Natale,\n"
                              "quest'anno ho cercato di essere molto buono e di non bocciare nessuno, ma talvolta ho dovuto farlo.\n"
                              "\n"
                              "Ti prego di portarmi in regalo un Lego Mindstorms per poter insegnare ai ragazzi del corso a programmare in microcodice Lego.\n"
                              "\n"
                              "Ciao\n"
                              "Andrea");

                test = tokenizzaEmail(buffer, array, DIM_BUFFER);

             /* test fallito */
                if(tipo_di_test != test)
                   printf("TEST FALLITO\nFunzione testata: tokenizzaEmail(buffer, array, 1000)\n");
            break;

         /* risultato atteso: BODY_NOT_FOUND */
            case BODY_NOT_FOUND:
               strcpy(buffer, "From: andrea@sterbini.org\n"
                              "To: babbo_natale@northpole.org\n"
                              "Subject: Cosa vorrei per Natale\n"
                              "\n");

                test = tokenizzaEmail(buffer, array, DIM_BUFFER);

             /* test fallito */
                if(tipo_di_test != test)
                   printf("TEST FALLITO\nFunzione testata: tokenizzaEmail(buffer, array, 1000)\n");
            break;

         /* risultato atteso: BAD_HEADER */
            case BAD_HEADER:
               strcpy(buffer, "\tFrom: andrea@sterbini.org\n"
                              "To: babbo_natale@northpole.org\n"
                              "Subject: Cosa vorrei per Natale\n"
                              "\n"
                              "Caro Babbo Natale,\n"
                              "quest'anno ho cercato di essere molto buono e di non bocciare nessuno, ma talvolta ho dovuto farlo.\n"
                              "\n"
                              "Ti prego di portarmi in regalo un Lego Mindstorms per poter insegnare ai ragazzi del corso a programmare in microcodice Lego.\n"
                              "\n"
                              "Ciao\n"
                              "Andrea");

                test = tokenizzaEmail(buffer, array, DIM_BUFFER);

             /* test fallito */
                if(tipo_di_test != test)
                   printf("TEST FALLITO\nFunzione testata: tokenizzaEmail(buffer, array, 1000)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case BUFFER_TOO_SMALL:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria allocata */
         free(buffer);
         free(array);
      break;

   /* argomento considerato: tokenArray[] */
      case 1:
         buffer = (char*) malloc(sizeof(char) * DIM_BUFFER);
         array  = (Token*) malloc(sizeof(Token) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer || !array)
            return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer, "From: andrea@sterbini.org\n"
                              "To: babbo_natale@northpole.org\n"
                              "Subject: Cosa vorrei per Natale\n"
                              "\n"
                              "Caro Babbo Natale, \n"
                              "quest'anno ho cercato di essere molto buono e di non bocciare nessuno, ma talvolta ho dovuto farlo.\n"
                              "\n"
                              "Ti prego di portarmi in regalo un Lego Mindstorms per poter insegnare ai ragazzi del corso a programmare in microcodice Lego.\n"
                              "\n"
                              "Ciao\n"
                              "Andrea");

                test = tokenizzaEmail(buffer, array, DIM_BUFFER);

             /* test fallito */
                if(tipo_di_test != test)
                {
                   printf("TEST FALLITO\nFunzione testata: tokenizzaEmail(buffer, array, 1000)\n");
                   descrizione = 14;
                }
             /* controllo che la funzione abbia eseguito correttamente il suo lavoro */
                else
                {
                /* la funzione NON svolge correttamente il suo lavoro  */
                   if(strcmp(array[0].contesto, "From:") != 0 || strcmp(array[0].parola, "andrea") != 0 ||
                      strcmp(array[1].contesto, "From:") != 0 || strcmp(array[1].parola, "sterbini") != 0 ||
                      strcmp(array[2].contesto, "From:") != 0 || strcmp(array[2].parola, "org") != 0 ||
                      strcmp(array[3].contesto, "To:") != 0 || strcmp(array[3].parola, "babbo") != 0 ||
                      strcmp(array[4].contesto, "To:") != 0 || strcmp(array[4].parola, "natale") != 0 ||
                      strcmp(array[5].contesto, "To:") != 0 || strcmp(array[5].parola, "northpole") != 0 ||
                      strcmp(array[6].contesto, "To:") != 0 || strcmp(array[6].parola, "org") != 0 ||
                      strcmp(array[7].contesto, "Subject:") != 0 || strcmp(array[7].parola, "Cosa") != 0 ||
                      strcmp(array[8].contesto, "Subject:") != 0 || strcmp(array[8].parola, "vorrei") != 0 ||
                      strcmp(array[9].contesto, "Subject:") != 0 || strcmp(array[9].parola, "per") != 0 ||
                      strcmp(array[10].contesto, "Subject:") != 0 || strcmp(array[10].parola, "Natale") != 0 ||
                      array[11].contesto != NULL || strcmp(array[11].parola, "Caro") != 0 ||
                      array[12].contesto != NULL || strcmp(array[12].parola, "Babbo") != 0 ||
                      array[13].contesto != NULL || strcmp(array[13].parola, "Natale") != 0 ||
                      array[14].contesto != NULL || strcmp(array[14].parola, "quest") != 0 ||
                      array[15].contesto != NULL || strcmp(array[15].parola, "anno") != 0 ||
                      array[16].contesto != NULL || strcmp(array[16].parola, "ho") != 0 ||
                      array[17].contesto != NULL || strcmp(array[17].parola, "cercato") != 0 ||
                      array[18].contesto != NULL || strcmp(array[18].parola, "di") != 0 ||
                      array[19].contesto != NULL || strcmp(array[19].parola, "essere") != 0 ||
                      array[20].contesto != NULL || strcmp(array[20].parola, "molto") != 0 ||
                      array[21].contesto != NULL || strcmp(array[21].parola, "buono") != 0 ||
                      array[22].contesto != NULL || strcmp(array[22].parola, "di") != 0 ||
                      array[23].contesto != NULL || strcmp(array[23].parola, "non") != 0 ||
                      array[24].contesto != NULL || strcmp(array[24].parola, "bocciare") != 0 ||
                      array[25].contesto != NULL || strcmp(array[25].parola, "nessuno") != 0 ||
                      array[26].contesto != NULL || strcmp(array[26].parola, "ma") != 0 ||
                      array[27].contesto != NULL || strcmp(array[27].parola, "talvolta") != 0 ||
                      array[28].contesto != NULL || strcmp(array[28].parola, "ho") != 0 ||
                      array[29].contesto != NULL || strcmp(array[29].parola, "dovuto") != 0 ||
                      array[30].contesto != NULL || strcmp(array[30].parola, "farlo") != 0 ||
                      array[31].contesto != NULL || strcmp(array[31].parola, "Ti") != 0 ||
                      array[32].contesto != NULL || strcmp(array[32].parola, "prego") != 0 ||
                      array[33].contesto != NULL || strcmp(array[33].parola, "di") != 0 ||
                      array[34].contesto != NULL || strcmp(array[34].parola, "portarmi") != 0 ||
                      array[35].contesto != NULL || strcmp(array[35].parola, "in") != 0 ||
                      array[36].contesto != NULL || strcmp(array[36].parola, "regalo") != 0 ||
                      array[37].contesto != NULL || strcmp(array[37].parola, "un") != 0 ||
                      array[38].contesto != NULL || strcmp(array[38].parola, "Lego") != 0 ||
                      array[39].contesto != NULL || strcmp(array[39].parola, "Mindstorms") != 0 ||
                      array[40].contesto != NULL || strcmp(array[40].parola, "per") != 0 ||
                      array[41].contesto != NULL || strcmp(array[41].parola, "poter") != 0 ||
                      array[42].contesto != NULL || strcmp(array[42].parola, "insegnare") != 0 ||
                      array[43].contesto != NULL || strcmp(array[43].parola, "ai") != 0 ||
                      array[44].contesto != NULL || strcmp(array[44].parola, "ragazzi") != 0 ||
                      array[45].contesto != NULL || strcmp(array[45].parola, "del") != 0 ||
                      array[46].contesto != NULL || strcmp(array[46].parola, "corso") != 0 ||
                      array[47].contesto != NULL || strcmp(array[47].parola, "programmare") != 0 ||
                      array[48].contesto != NULL || strcmp(array[48].parola, "in") != 0 ||
                      array[49].contesto != NULL || strcmp(array[49].parola, "microcodice") != 0 ||
                      array[50].contesto != NULL || strcmp(array[50].parola, "Lego") != 0 ||
                      array[51].contesto != NULL || strcmp(array[51].parola, "Ciao") != 0 ||
                      array[52].contesto != NULL || strcmp(array[52].parola, "Andrea") != 0)
                   {
                      BAD_RESULT_OK = 1;
                      descrizione = 0;
                   }
                /* la funzione svolge correttamente il suo lavoro */
                   else
                   {
                      test = tokenizzaEmail(buffer, NULL, DIM_BUFFER);

                   /* la funzione NON svolge il suo lavoro con parametri errati */
                      if(tipo_di_test == test)
                      {
                         BAD_RESULT_OK = 1;
                         descrizione = 13;
                      }
                   /* non sono stati riscontrati errori, perci faccio in modo di segnalare TEST_PASSED */
                      else
                         test = tipo_di_test;
                   }
                }
            break;

         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               strcpy(buffer, "From: andrea@sterbini.org\n"
                              "To: babbo_natale@northpole.org\n"
                              "Subject: Cosa vorrei per Natale\n"
                              "\n"
                              "Caro Babbo Natale,\n"
                              "quest'anno ho cercato di essere molto buono e di non bocciare nessuno, ma talvolta ho dovuto farlo.\n"
                              "\n"
                              "Ti prego di portarmi in regalo un Lego Mindstorms per poter insegnare ai ragazzi del corso a programmare in microcodice Lego.\n"
                              "\n"
                              "Ciao\n"
                              "Andrea");

               test = tokenizzaEmail(buffer, NULL, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: tokenizzaEmail(buffer, NULL, 1000)\n");
            break;

         /* risultato atteso: BUFFER_TOO_SMALL */
            case BUFFER_TOO_SMALL:
               free(array);
               array = (Token*) malloc(sizeof(Token) * DIM_BUFFER_SMALL);

            /* controllo errore allocazione memoria */
               if(!array)
                  return MEM_ALLOC_ERROR;

               strcpy(buffer, "From: andrea@sterbini.org\n"
                              "To: babbo_natale@northpole.org\n"
                              "Subject: Cosa vorrei per Natale\n"
                              "\n"
                              "Caro Babbo Natale,\n"
                              "quest'anno ho cercato di essere molto buono e di non bocciare nessuno, ma talvolta ho dovuto farlo.\n"
                              "\n"
                              "Ti prego di portarmi in regalo un Lego Mindstorms per poter insegnare ai ragazzi del corso a programmare in microcodice Lego.\n"
                              "\n"
                              "Ciao\n"
                              "Andrea");

                test = tokenizzaEmail(buffer, array, DIM_BUFFER_SMALL);

             /* test fallito */
                if(tipo_di_test != test)
                   printf("TEST FALLITO\nFunzione testata: tokenizzaEmail(buffer, array, 20)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria allocata */
         free(buffer);
         free(array);
      break;

   /* argomento considerato: dimT */
      case 2:
         buffer = (char*) malloc(sizeof(char) * DIM_BUFFER);
         array  = (Token*) malloc(sizeof(Token) * DIM_BUFFER);

      /* controllo errore allocazione memoria */
         if(!buffer || !array)
            return MEM_ALLOC_ERROR;

         switch(tipo_di_test)
         {
         /* risultato atteso: RESULT_OK */
            case RESULT_OK:
               strcpy(buffer, "From: andrea@sterbini.org\n"
                              "To: babbo_natale@northpole.org\n"
                              "Subject: Cosa vorrei per Natale\n"
                              "\n"
                              "Caro Babbo Natale,\n"
                              "quest'anno ho cercato di essere molto buono e di non bocciare nessuno, ma talvolta ho dovuto farlo.\n"
                              "\n"
                              "Ti prego di portarmi in regalo un Lego Mindstorms per poter insegnare ai ragazzi del corso a programmare in microcodice Lego.\n"
                              "\n"
                              "Ciao\n"
                              "Andrea");

                test = tokenizzaEmail(buffer, array, DIM_BUFFER);

             /* test fallito */
                if(tipo_di_test != test)
                {
                   printf("TEST FALLITO\nFunzione testata: tokenizzaEmail(buffer, array, 1000)\n");
                   descrizione = 14;
                }
             /* controllo che la funzione abbia eseguito correttamente il suo lavoro */
                else
                {
                /* la funzione NON svolge correttamente il suo lavoro  */
                   if(strcmp(array[0].contesto, "From:") != 0 || strcmp(array[0].parola, "andrea") != 0 ||
                      strcmp(array[1].contesto, "From:") != 0 || strcmp(array[1].parola, "sterbini") != 0 ||
                      strcmp(array[2].contesto, "From:") != 0 || strcmp(array[2].parola, "org") != 0 ||
                      strcmp(array[3].contesto, "To:") != 0 || strcmp(array[3].parola, "babbo") != 0 ||
                      strcmp(array[4].contesto, "To:") != 0 || strcmp(array[4].parola, "natale") != 0 ||
                      strcmp(array[5].contesto, "To:") != 0 || strcmp(array[5].parola, "northpole") != 0 ||
                      strcmp(array[6].contesto, "To:") != 0 || strcmp(array[6].parola, "org") != 0 ||
                      strcmp(array[7].contesto, "Subject:") != 0 || strcmp(array[7].parola, "Cosa") != 0 ||
                      strcmp(array[8].contesto, "Subject:") != 0 || strcmp(array[8].parola, "vorrei") != 0 ||
                      strcmp(array[9].contesto, "Subject:") != 0 || strcmp(array[9].parola, "per") != 0 ||
                      strcmp(array[10].contesto, "Subject:") != 0 || strcmp(array[10].parola, "Natale") != 0 ||
                      array[11].contesto != NULL || strcmp(array[11].parola, "Caro") != 0 ||
                      array[12].contesto != NULL || strcmp(array[12].parola, "Babbo") != 0 ||
                      array[13].contesto != NULL || strcmp(array[13].parola, "Natale") != 0 ||
                      array[14].contesto != NULL || strcmp(array[14].parola, "quest") != 0 ||
                      array[15].contesto != NULL || strcmp(array[15].parola, "anno") != 0 ||
                      array[16].contesto != NULL || strcmp(array[16].parola, "ho") != 0 ||
                      array[17].contesto != NULL || strcmp(array[17].parola, "cercato") != 0 ||
                      array[18].contesto != NULL || strcmp(array[18].parola, "di") != 0 ||
                      array[19].contesto != NULL || strcmp(array[19].parola, "essere") != 0 ||
                      array[20].contesto != NULL || strcmp(array[20].parola, "molto") != 0 ||
                      array[21].contesto != NULL || strcmp(array[21].parola, "buono") != 0 ||
                      array[22].contesto != NULL || strcmp(array[22].parola, "di") != 0 ||
                      array[23].contesto != NULL || strcmp(array[23].parola, "non") != 0 ||
                      array[24].contesto != NULL || strcmp(array[24].parola, "bocciare") != 0 ||
                      array[25].contesto != NULL || strcmp(array[25].parola, "nessuno") != 0 ||
                      array[26].contesto != NULL || strcmp(array[26].parola, "ma") != 0 ||
                      array[27].contesto != NULL || strcmp(array[27].parola, "talvolta") != 0 ||
                      array[28].contesto != NULL || strcmp(array[28].parola, "ho") != 0 ||
                      array[29].contesto != NULL || strcmp(array[29].parola, "dovuto") != 0 ||
                      array[30].contesto != NULL || strcmp(array[30].parola, "farlo") != 0 ||
                      array[31].contesto != NULL || strcmp(array[31].parola, "Ti") != 0 ||
                      array[32].contesto != NULL || strcmp(array[32].parola, "prego") != 0 ||
                      array[33].contesto != NULL || strcmp(array[33].parola, "di") != 0 ||
                      array[34].contesto != NULL || strcmp(array[34].parola, "portarmi") != 0 ||
                      array[35].contesto != NULL || strcmp(array[35].parola, "in") != 0 ||
                      array[36].contesto != NULL || strcmp(array[36].parola, "regalo") != 0 ||
                      array[37].contesto != NULL || strcmp(array[37].parola, "un") != 0 ||
                      array[38].contesto != NULL || strcmp(array[38].parola, "Lego") != 0 ||
                      array[39].contesto != NULL || strcmp(array[39].parola, "Mindstorms") != 0 ||
                      array[40].contesto != NULL || strcmp(array[40].parola, "per") != 0 ||
                      array[41].contesto != NULL || strcmp(array[41].parola, "poter") != 0 ||
                      array[42].contesto != NULL || strcmp(array[42].parola, "insegnare") != 0 ||
                      array[43].contesto != NULL || strcmp(array[43].parola, "ai") != 0 ||
                      array[44].contesto != NULL || strcmp(array[44].parola, "ragazzi") != 0 ||
                      array[45].contesto != NULL || strcmp(array[45].parola, "del") != 0 ||
                      array[46].contesto != NULL || strcmp(array[46].parola, "corso") != 0 ||
                      array[47].contesto != NULL || strcmp(array[47].parola, "programmare") != 0 ||
                      array[48].contesto != NULL || strcmp(array[48].parola, "in") != 0 ||
                      array[49].contesto != NULL || strcmp(array[49].parola, "microcodice") != 0 ||
                      array[50].contesto != NULL || strcmp(array[50].parola, "Lego") != 0 ||
                      array[51].contesto != NULL || strcmp(array[51].parola, "Ciao") != 0 ||
                      array[52].contesto != NULL || strcmp(array[52].parola, "Andrea") != 0)
                   {
                      BAD_RESULT_OK = 1;
                      descrizione = 0;
                   }
                /* la funzione svolge correttamente il suo lavoro */
                   else
                   {
                      test = tokenizzaEmail(buffer, array, -1);

                   /* la funzione NON svolge il suo lavoro con parametri errati */
                      if(tipo_di_test == test)
                      {
                         BAD_RESULT_OK = 1;
                         descrizione = 13;
                      }
                   /* non sono stati riscontrati errori, perci faccio in modo di segnalare TEST_PASSED */
                      else
                         test = tipo_di_test;
                   }
                }
            break;

         /* risultato atteso: BUFFER_TOO_SMALL */
            case BUFFER_TOO_SMALL:
               test = tokenizzaEmail(buffer, array, -1);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: tokenizzaEmail(buffer, array, -1)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case NULL_POINTER:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }

      /* libero la memoria allocata */
         free(buffer);
         free(array);
      break;

   /* argomento non presente */
      default:
         return ARG_MISSING;
   }


/* test passato: risultato atteso corretto */
   if(tipo_di_test == test && !BAD_RESULT_OK)
      return TEST_PASSED;


/* TEST FALLITO: visualizzazione avviso */
   if(!BAD_RESULT_OK)
   {
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }
   else
/* TEST APPARENTEMENTE CORRETTO: la funzione ha ritornato RESULT_OK, ma non svolge correttamente il suo lavoro */
   {
      printf("TEST APPARENTEMENTE CORRETTO\n");
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[descrizione]);
   }


/* ritorno codice di svolgimento test corretto */
   return TEST_FAILED;
}



/* funzione di test per printTokens(...) */
int test_printTokens(int tipo_di_test, int argomento)
{
   char* tests[TESTS] = {"RESULT_OK",
                         "UNKNOWN_ERROR",
                         "NULL_POINTER",
                         "EMPTY_STRING",
                         "FILE_MISSING",
                         "FILE_UNREADABLE",
                         "FILE_UNWRITABLE",
                         "FILE_EMPTY",
                         "BUFFER_TOO_SMALL",
                         "HEADER_NOT_FOUND",
                         "BODY_NOT_FOUND",
                         "BAD_HEADER",
                         "MEM_ALLOC_ERROR"};

   char* description[TESTS] = {"La funzione ritorna un codice RESULT_OK, ma non svolge correttamente il suo lavoro\n",
                               "Errore di tipo sconosciuto\n",
                               "La funzione non controlla se riceve un puntatore nullo\n",
                               "La funzione non controlla se riceve una stringa vuota\n",
                               "La funzione non controlla l'esistenza del file\n",
                               "La funzione non controlla il permesso di lettura del file\n",
                               "La funzione non controlla il permesso di scrittura sul file\n",
                               "La funzione non controlla se il file letto  vuoto\n",
                               "La funzione non controlla se il buffer  sufficientemene capiente\n",
                               "La funzione non controlla l'esistenza dello header nella email\n",
                               "La funzione non controlla l'esistenza del corpo nella email\n",
                               "La funzione non controlla la giusta sintassi di uno header\n",
                               "La funzione non controlla l'allocazione della memoria\n",
                               "La funzione ritorna un codice esatto anche per parametri errati\n"};

    char* argposition[ARGUMENTS] = {"Primo",
                                    "Secondo",
                                    "Terzo",
                                    "Quarto",
                                    "Quinto"};

/* memorizzano i risultati restituiti dalla funzione */
   int test;
/* segnala un codice di ritorno corretto ma malfunzionamento della funzione */
   int BAD_RESULT_OK = 0;
/* array di tokens per memorizzare le singole parole trovate nella email */
   Token* array;


/* ARGOMENTI */
   switch(argomento)
   {
   /* argomento considerato:  tokenArray[] */
      case 0:
         switch(tipo_di_test)
         {
         /* risultato atteso: NULL_POINTER */
            case NULL_POINTER:
               test = printTokens(NULL, DIM_BUFFER);

            /* test fallito */
               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: printTokens(NULL, 1000)\n");
            break;

         /* test insensati per questo argomento */
            case UNKNOWN_ERROR:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case BUFFER_TOO_SMALL:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }
      break;

   /* argomento considerato: dimT */
      case 1:
         switch(tipo_di_test)
         {
         /* risultato atteso: BUFFER_TOO_SMALL */
            case BUFFER_TOO_SMALL:
               array = (Token*) malloc(sizeof(Token) * 20);

               test = printTokens(array, -1);

               if(tipo_di_test != test)
                  printf("TEST FALLITO\nFunzione testata: printTokens(tokenArray, -1)\n");

               free(array);
            break;

         /* test insensati per questo argomento */
            case NULL_POINTER:
            case UNKNOWN_ERROR:
            case EMPTY_STRING:
            case FILE_MISSING:
            case FILE_UNREADABLE:
            case FILE_UNWRITABLE:
            case FILE_EMPTY:
            case HEADER_NOT_FOUND:
            case BODY_NOT_FOUND:
            case BAD_HEADER:
            case MEM_ALLOC_ERROR:
               return NOT_VALID;
            break;

         /* test non implementato per questo argomento */
            default:
               return NOT_IMPLEMENTED;
         }
      break;

   /* argomento non presente */
      default:
         return ARG_MISSING;
   }


/* test passato: risultato atteso corretto */
   if(tipo_di_test == test && !BAD_RESULT_OK)
      return TEST_PASSED;


/* TEST FALLITO: visualizzazione avviso */
   if(!BAD_RESULT_OK)
   {
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[tipo_di_test]);
   }
   else
/* TEST APPARENTEMENTE CORRETTO: la funzione ha ritornato RESULT_OK, ma non svolge correttamente il suo lavoro */
   {
      printf("TEST APPARENTEMENTE CORRETTO\n");
      printf("Argomento in esame: %s\n",argposition[argomento]);
      printf("Risultato atteso: %s\n",tests[tipo_di_test]);
      printf("Risultato ottenuto: %s\n",tests[test]);
      printf("Possibile causa: %s\n",description[test]);
   }


/* ritorno codice di svolgimento test corretto */
   return TEST_FAILED;
}

