#include "modulo1.h"
#include <stdio.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>

int leggiFile(char* nomefile, int dimensione, char* testo){
  struct stat s;
  FILE* fptr;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su nomefile
  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  // nomefile  un puntatore NULL 
  if(nomefile == NULL)
    return NULL_POINTER;
  // nomefile  una stringa vuota
  if(*nomefile == '\0')
    return EMPTY_STRING;
  // il file "nomefile" non esiste
  errno = 0;
  if(stat(nomefile, &s) != 0){
    if(errno == ENOENT)
      return FILE_MISSING;
    return UNKNOWN_ERROR; // c' stato un problema in stat
  }
  // il file "nomefile" non pu essere aperto in lettura
  if((fptr = fopen(nomefile, "r")) == NULL)
    return FILE_UNREADABLE;
  // il file "nomefile"  vuoto
  if(s.st_size == 0)
    return FILE_EMPTY;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su dimensione
  ///////////////////////////////////////////////////////////////////////////////////////////////

  if(dimensione <= 0)
    return BUFFER_TOO_SMALL;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su testo
  ///////////////////////////////////////////////////////////////////////////////////////////////

  // testo  un puntatore NULL
  if(testo == NULL)
    return NULL_POINTER;
  
  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  fread(testo, sizeof(char), dimensione - 1, fptr);
  testo[dimensione] = '\0';
  if(fclose(fptr) != 0)
    return UNKNOWN_ERROR;
  return RESULT_OK;
}

int estraiHeaders(char* email, char* headers, int dimh){
  int c;
  int i;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su email
  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  // email  un puntatore NULL 
  if(email == NULL)
    return NULL_POINTER;
  // email  una stringa vuota
  if(*email == '\0')
    return EMPTY_STRING;
  // email non contiene headers
  if(email[0] == '\n')
    return HEADER_NOT_FOUND;
  for(c = 0; email[c] != '\0'; c++)
    if(email[c] == '\n' && email[c + 1] == '\n')
      break;
  if(email[c] == '\0')
    return HEADER_NOT_FOUND;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su headers
  ///////////////////////////////////////////////////////////////////////////////////////////////

  // headers  un puntatore NULL
  if(headers == NULL)
    return NULL_POINTER;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su dimh
  ///////////////////////////////////////////////////////////////////////////////////////////////

  // dimh  troppo piccolo
  if(dimh <= 0 || dimh <= c + 1)
    return BUFFER_TOO_SMALL;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  i = 0;
  while(i <= c){
    headers[i] = email[i];
    i++;
  }
  headers[i] = '\0';
  return RESULT_OK;
}

int primoHeader(char* headers, char* primo_header, int dim){
  int i;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su headers
  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  // headers  un puntatore NULL 
  if(headers == NULL)
    return NULL_POINTER;
  // headers  una stringa vuota
  if(*headers == '\0')
    return EMPTY_STRING;
  
  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su primo_header
  ///////////////////////////////////////////////////////////////////////////////////////////////

  // primo_header  un puntatore NULL
  if(primo_header == NULL)
    return NULL_POINTER;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su dim
  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  // dim <= 0
  if(dim <= 0)
    return BUFFER_TOO_SMALL;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  for(i = 0; headers[i] != '\0'; i++){
    if(dim > i + 1){
      primo_header[i] = headers[i];
      if(headers[i] == '\n'){
	if(headers[i + 1] == ' ' || headers[i + 1] == '\t')
	  ;
	else{
	  primo_header[i] = '\0';
	  return RESULT_OK;
	}
      }
    }
    else
      return BUFFER_TOO_SMALL;
  }
  return UNKNOWN_ERROR;
}

int parseHeader(char* header, char* tipo, int dimt, char* valore, int dimv){
  int i;
  int j = 0;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su header
  ///////////////////////////////////////////////////////////////////////////////////////////////

  // header  un puntatore NULL 
  if(header == NULL)
    return NULL_POINTER;
  // header  una stringa vuota
  if(*header == '\0')
    return EMPTY_STRING;
  // header contiene un header con sintassi errata
  if(*header == ' ' || *header == '\t')
    return BAD_HEADER;
  for(i = 0; header[i] != '\n' && header[i] != '\0'; i++)
    if(header[i] == ' ' || header[i] == '\t')
      j = 1;
  if(j != 1)
    return BAD_HEADER;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su tipo
  ///////////////////////////////////////////////////////////////////////////////////////////////

  // tipo  un puntatore NULL
  if(tipo == NULL)
    return NULL_POINTER;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su dimt
  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  // dimt <= 0
  if(dimt <= 0)
    return BUFFER_TOO_SMALL;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su valore
  ///////////////////////////////////////////////////////////////////////////////////////////////

  // valore  un puntatore NULL
  if(valore == NULL)
    return NULL_POINTER;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su dimv
  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  // dimv <= 0
  if(dimv <= 0)
    return BUFFER_TOO_SMALL;

  ///////////////////////////////////////////////////////////////////////////////////////////////

  for(i = 0; header[i] != ' ' && header[i] != '\t'; i++)
    if(dimt > i + 1)
      tipo[i] = header[i];
    else
      return BUFFER_TOO_SMALL;
  tipo[i] = '\0';
  i++;
  j = 0;
  while(header[i] != '\0'){
    if(dimv > j + 1)
      valore[j] = header[i];
    else
      return BUFFER_TOO_SMALL;
    i++;
    j++;
  }
  valore[j] = '\0';
  return RESULT_OK;
}

int primaParola(char* testo, char* parola, int dim){
  int i, j, flag;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su testo
  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  // testo  un puntatore NULL 
  if(testo == NULL)
    return NULL_POINTER;
  // testo  una stringa vuota
  if(*testo == '\0')
    return EMPTY_STRING;
  
  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su parola
  ///////////////////////////////////////////////////////////////////////////////////////////////

  // parola  un puntatore NULL
  if(parola == NULL)
    return NULL_POINTER;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su dim
  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  // dim <= 0
  if(dim <= 0)
    return BUFFER_TOO_SMALL;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  flag = 0;
  j = 0;
  for(i = 0; testo[i] != '\0' && flag != 1; i++){
    if(testo[i + 1] == '\0' && j <= 1){
      j = 0;
      break;
    }
    if((testo[i] >= 48 && testo[i] <= 57) || (testo[i] >= 65 && testo[i] <= 90) || (testo[i] >= 97 && testo[i] <= 122)){
      if(j == dim - 1)
	return BUFFER_TOO_SMALL;
      parola[j] = testo[i];
      j++;
    }
    else{
      if(j > 1)
	flag = 1;
      else
	j = 0;
    }
  }
  parola[j] = '\0';
  return RESULT_OK;
}

int tokenizzaEmail(char* email, Token tokenArray[], int dimT){
  char* headers;
  char* hdr;
  char* tipo;
  char* valore;
  char* parola;
  int dim_H, dimh, dimt, dimv, dimp, i, j, k, l, r;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su email
  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  // email  un puntatore NULL 
  if(email == NULL)
    return NULL_POINTER;

  //email  una stringa vuota
  if(*email == '\0')
    return EMPTY_STRING;

  //////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su tokenArray
  ///////////////////////////////////////////////////////////////////////////////////////////////

  // tokenArray  un puntatore NULL 
  if(tokenArray == NULL)
    return NULL_POINTER;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su dimT
  ///////////////////////////////////////////////////////////////////////////////////////////////

  // dimT <= 0
  if(dimT <= 0)
    return BUFFER_TOO_SMALL;

  ///////////////////////////////////////////////////////////////////////////////////////////////

  dim_H = 50;
  dimh = 20;
  dimt = 5;
  dimv = 10;
  dimp = 10;
  if((headers = (char*) malloc(dim_H)) == NULL)
    return MEM_ALLOC_ERROR;
  if((hdr = (char*) malloc(dimh)) == NULL)
    return MEM_ALLOC_ERROR;
  if((tipo = (char*) malloc(dimt)) == NULL)
    return MEM_ALLOC_ERROR;
  if((valore = (char*) malloc(dimv)) == NULL)
    return MEM_ALLOC_ERROR;
  if((parola = (char*) malloc(dimp)) == NULL)
    return MEM_ALLOC_ERROR;

  i = 0;
  j = 0;
  k = 0;
  l = 0;

  while((r = estraiHeaders(email, headers, dim_H)) == BUFFER_TOO_SMALL){
    dim_H += 10;
    headers = (char*) realloc(headers, dim_H);
  }
  if(r != RESULT_OK){
    free(headers);
    free(hdr);
    free(tipo);
    free(valore);
    free(parola);
    return r;
  }
  l += strlen(headers) + 1;
  if(email[l] == '\0')
    return BODY_NOT_FOUND;
  while(headers[i] != '\0' && j < dimT){
    dimh = 20;
    hdr = (char*) realloc(hdr, dimh);
    while((r = primoHeader(headers + i, hdr, dimh)) == BUFFER_TOO_SMALL){
      dimh += 5;
      hdr = (char*) realloc(hdr, dimh);
    }
    if(r != RESULT_OK){
      free(headers);
      free(hdr);
      free(tipo);
      free(valore);
      free(parola);
      return r;
    }
    dimt = 5;
    dimv = 10;
    tipo = (char*) realloc(tipo, dimt);
    valore = (char*) realloc(valore, dimv);
    while((r = parseHeader(hdr, tipo, dimt, valore, dimv)) == BUFFER_TOO_SMALL){
      dimt += 5;
      dimv += 10;
      tipo = (char*) realloc(tipo, dimt);
      valore = (char*) realloc(valore, dimv);
    }
    if(r != RESULT_OK){
      free(headers);
      free(hdr);
      free(tipo);
      free(valore);
      free(parola);
      return r;
    }
    k = 0;
    while(valore[k] != '\0' && j < dimT){
      while(valore[k] != '\0' && !((valore[k] >= 48 && valore[k] <= 57) || (valore[k] >= 65 && valore[k] <= 90) || (valore[k] >= 97 && valore[k] <= 122)))
	k++;
      if(valore[k] == '\0')
	break;
      if(((valore[k] >= 48 && valore[k] <= 57) || (valore[k] >= 65 && valore[k] <= 90) || (valore[k] >= 97 && valore[k] <= 122)) && !((valore[k + 1] >= 48 && valore[k + 1] <= 57) || (valore[k + 1] >= 65 && valore[k + 1] <= 90) || (valore[k + 1] >= 97 && valore[k + 1] <= 122)))
	k += 2;
      tokenArray[j].contesto = (char*) malloc(dimt);
      tokenArray[j].contesto = strncpy(tokenArray[j].contesto, tipo, dimt);
      dimp = 10;
      parola = (char*) realloc(parola, dimp);
      while((r = primaParola(valore + k, parola, dimp)) == BUFFER_TOO_SMALL){
	dimp += 5;
	parola = (char*) realloc(parola, dimp);
      }
      if(r != RESULT_OK){
	free(headers);
	free(hdr);
	free(tipo);
	free(valore);
	free(parola);
	return r;
      }
      tokenArray[j].parola = (char*) malloc(dimp);
      tokenArray[j].parola = strncpy(tokenArray[j].parola, parola, dimp);
      j++;
      k += strlen(parola);
    }
    i += strlen(hdr) + 1;
  }
  if(j >= dimT){
    free(headers);
    free(hdr);
    free(tipo);
    free(valore);
    return BUFFER_TOO_SMALL;
  }
  while(email[l] != '\0' && j < dimT){
    while(email[l] != '\0' && !((email[l] >= 48 && email[l] <= 57) || (email[l] >= 65 && email[l] <= 90) || (email[l] >= 97 && email[l] <= 122)))
      l++;
    if(email[l] == '\0'){
      free(headers);
      free(hdr);
      free(tipo);
      free(valore);
      free(parola);
      tokenArray[j].contesto = NULL;
      tokenArray[j].parola = NULL;
      return RESULT_OK;
    }
    if(((email[l] >= 48 && email[l] <= 57) || (email[l] >= 65 && email[l] <= 90) || (email[l] >= 97 && email[l] <= 122)) && !((email[l + 1] >= 48 && email[l + 1] <= 57) || (email[l + 1] >= 65 && email[l + 1] <= 90) || (email[l + 1] >= 97 && email[l + 1] <= 122)))
      l += 2;
    tokenArray[j].contesto = NULL;
    dimp = 10;
    parola = (char*) realloc(parola, dimp);
    while((r = primaParola(email + l, parola, dimp)) == BUFFER_TOO_SMALL){
      dimp += 5;
      parola = (char*) realloc(parola, dimp);
    }
    if(r != RESULT_OK){
      free(headers);
      free(hdr);
      free(tipo);
      free(valore);
      free(parola);
      return r;
    }
    tokenArray[j].parola = (char*) malloc(dimp);
    tokenArray[j].parola = strncpy(tokenArray[j].parola, parola, dimp);
    j++;
    l += strlen(parola);
  }
  if(j < dimT){
    tokenArray[j].contesto = NULL;
    tokenArray[j].parola = NULL;
  }
  else{
    free(headers);
    free(hdr);
    free(tipo);
    free(valore);
    free(parola);
    return BUFFER_TOO_SMALL;
  }
  free(headers);
  free(hdr);
  free(tipo);
  free(valore);
  free(parola);
  return RESULT_OK;
}

int printTokens(Token tokenArray[], int dimT){
  int i;

  ///////////////////////////////////////////////////////////////////////////////////////////////
  // Controlli su tokenArray
  ///////////////////////////////////////////////////////////////////////////////////////////////
  
  // tokenArray  un puntatore NULL 
  if(tokenArray == NULL)
    return NULL_POINTER;

  ///////////////////////////////////////////////////////////////////////////////////////////////

  for(i = 0; (tokenArray[i].parola != NULL || tokenArray[i].contesto != NULL) && i < dimT; i++){
    if(tokenArray[i].contesto == NULL)
      printf("%s\n", tokenArray[i].parola);
    else{
      printf("%s*%s\n", tokenArray[i].contesto, tokenArray[i].parola);
    }
  }
  return RESULT_OK;
}

