#include "testModulo1.h"
#include <stdio.h>
#include <string.h>

#define GIOCO_PRINT_FAILURE(f, ra, ro, msg) printf("Test fallito: funzione testata: %s\n\trisultato atteso: %s\n\trisultato \
ottenuto: %s\n\tspiegazione: %s\n", f, ra, ro, msg)

int test_leggiFile(int tipo_di_test, int n_argomento){
  char* buffer;
  FILE* fptr;
  char* testo = "Testo di test.";
  int retCode;
  char* cR_vett[] = {"RESULT_OK", "UNKNOWN_ERROR", "NULL_POINTER", "EMPTY_STRING", "FILE_MISSING", "FILE_UNREADABLE",
			   "FILE_UNWRITABLE", "FILE_EMPTY", "BUFFER_TOO_SMALL", "HEADER_NOT_FOUND", "BODY_NOT_FOUND", 
			   "BAD_HEADER", "MEM_ALLOC_ERROR"};

  if(n_argomento < 0 || n_argomento > 2)
    return ARG_MISSING;

  switch(tipo_di_test){

  case RESULT_OK:

    if(n_argomento == 0){
      buffer = (char*) malloc(15);
      fptr = fopen("GIOCOtestfile.txt", "w");
      fputs(testo, fptr);
      fclose(fptr);
      if((retCode = leggiFile("GIOCOtestfile.txt", 15, buffer)) == RESULT_OK && strcmp(testo, buffer) == 0){
	free(buffer);
	return TEST_PASSED;
      }
      free(buffer);
      GIOCO_PRINT_FAILURE("leggiFile(\"GIOCOtestfile.txt\", 15, buffer)", "RESULT_OK", cR_vett[retCode], "La funzione leggiFile \
fallisce se gli vengono passati 3 argomenti corretti.");
      return TEST_FAILED;
    }
    return NOT_IMPLEMENTED;

  case NULL_POINTER:

    switch(n_argomento){

    case 0:
      buffer = (char*) malloc(15);
      retCode = leggiFile(NULL, 15, buffer);
      free(buffer);
      if(retCode == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("leggiFile(NULL, 15, buffer)", "NULL_POINTER", cR_vett[retCode], "La funzione leggiFile non controlla \
se il primo argomento  un puntatore nullo.");
      return TEST_FAILED;

    case 2:
      fptr = fopen("GIOCOtestfile.txt", "w");
      fputs(testo, fptr);
      fclose(fptr);
      if((retCode = leggiFile("GIOCOtestfile.txt", 15, NULL)) == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("leggiFile(\"GIOCOtestfile.txt\", 15, NULL)", "NULL_POINTER", cR_vett[retCode], "La funzione leggiFile non \
controlla se il terzo argomento  un puntatore nullo.");
      return TEST_FAILED;

    default:
      return NOT_VALID;
    }

  case EMPTY_STRING:

    switch(n_argomento){

    case 0:
      buffer = (char*) malloc(15);
      retCode = leggiFile("", 15, buffer);
      free(buffer);
      if(retCode == EMPTY_STRING)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("leggiFile(\"\", 15, buffer)", "EMPTY_STRING", cR_vett[retCode], "La funzione leggiFile non controlla \
se il primo argomento  una stringa vuota.");
      return TEST_FAILED;

    default:
      return NOT_VALID;
    }

  case FILE_MISSING:

    switch(n_argomento){

    case 0:
      buffer = (char*) malloc(15);
      retCode = leggiFile("file-missing", 15, buffer);
      free(buffer);
      if(retCode == FILE_MISSING)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("leggiFile(\"file-missing\", 15, buffer)", "FILE_MISSING", cR_vett[retCode], "La funzione \
leggiFile non controlla se il primo argomento  un nome di file inesistente.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
    }
    
  case FILE_UNREADABLE:
    
    switch(n_argomento){
      
    case 0:
      buffer = (char*) malloc(15);
      retCode = leggiFile("file-unreadeble", 15, buffer);
      free(buffer);
      if(retCode == FILE_UNREADABLE)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("leggiFile(\"file-unreadable\", 15, buffer)", "FILE_UNREADABLE", cR_vett[retCode], "La funzione \
leggiFile non controlla se il primo argomento  un nome di file dal quale non si pu leggere.");
      return TEST_FAILED;

    default:
      return NOT_VALID;

    }

  case FILE_UNWRITABLE:
    
    switch(n_argomento){
      
    case 0:
      return NOT_IMPLEMENTED;
      
    default:
      return NOT_VALID;
      
    }
    
  case FILE_EMPTY:
    
    switch(n_argomento){
      
    case 0:
      buffer = (char*) malloc(15);
      fptr = fopen("GIOCOfilevuoto.txt", "w");
      fclose(fptr);
      retCode = leggiFile("GIOCOfilevuoto.txt", 15, buffer);
      free(buffer);
      if(retCode == FILE_EMPTY)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("leggiFile(\"GIOCOfilevuoto.txt\", 15, buffer)", "FILE_EMPTY", cR_vett[retCode], "La funzione leggiFile non \
controlla se il primo argomento  un nome di file vuoto.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
      
    }

  case BUFFER_TOO_SMALL:

    switch(n_argomento){
      
    case 1:
      buffer = (char*) malloc(15);
      fptr = fopen("testfile.txt", "w");
      fputs(testo, fptr);
      fclose(fptr);
      retCode = leggiFile("testfile.txt", -1, buffer);
      free(buffer);
      if(retCode == BUFFER_TOO_SMALL)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("leggiFile(\"testfile.txt\", -1, buffer)", "BUFFER_TOO_SMALL", cR_vett[retCode], "La funzione \
leggiFile non controlla se il secondo argomento  una dimensione <= 0.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;

    }
    
  default:
    return NOT_VALID;
  }
}

int test_estraiHeaders(int tipo_di_test, int n_argomento){
  char* buffer;
  char* email;
  int retCode;
  char* cR_vett[] = {"RESULT_OK", "UNKNOWN_ERROR", "NULL_POINTER", "EMPTY_STRING", "FILE_MISSING", "FILE_UNREADABLE",
			   "FILE_UNWRITABLE", "FILE_EMPTY", "BUFFER_TOO_SMALL", "HEADER_NOT_FOUND", "BODY_NOT_FOUND", 
			   "BAD_HEADER", "MEM_ALLOC_ERROR"};

  if(n_argomento < 0 || n_argomento > 2)
    return ARG_MISSING;

  switch(tipo_di_test){
    
  case RESULT_OK:

    if(n_argomento == 0){
      buffer = (char*) malloc(25);
      email = "Questi\nsono\ngli headers\n\nquesto  il body";
      if((retCode = estraiHeaders(email, buffer, 25)) == RESULT_OK && strcmp(buffer, "Questi\nsono\ngli headers\n") == 0){
	free(buffer);
	return TEST_PASSED;
      }
      free(buffer);
      GIOCO_PRINT_FAILURE("estraiHeaders(email, buffer, 25)", "RESULT_OK", cR_vett[retCode], "La funzione estraiHeaders \
fallisce se gli vengono passati 3 argomenti corretti.");
      return TEST_FAILED;
    }
    return NOT_IMPLEMENTED;

  case NULL_POINTER:

    switch(n_argomento){

    case 0:
      buffer = (char*) malloc(2);
      retCode = estraiHeaders(NULL, buffer, 2);
      free(buffer);
      if(retCode == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("estraiHeaders(NULL, buffer, 2)", "NULL_POINTER", cR_vett[retCode], "La funzione estraiHeaders non \
controlla se il primo argomento  un puntatore nullo.");
      return TEST_FAILED;

    case 1:
      email = "xxx\n\nxxx";
      if((retCode = estraiHeaders(email, NULL, 2)) == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("estraiHeaders(email, NULL, 2)", "NULL_POINTER", cR_vett[retCode], "La funzione estraiHeaders non \
controlla se il secondo argomento  un puntatore nullo.");
      return TEST_FAILED;

    default:
      return NOT_VALID;
    }
    
  case EMPTY_STRING:
    
    switch(n_argomento){
      
    case 0:
      buffer = (char*) malloc(2);
      retCode = estraiHeaders("", buffer, 2);
      free(buffer);
      if(retCode == EMPTY_STRING)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("estraiHeaders(\"\", buffer, 2)", "EMPTY_STRING", cR_vett[retCode], "La funzione estraiHeaders non \
controlla se il primo argomento  una stringa vuota.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
    }

  case BUFFER_TOO_SMALL:
    
    switch(n_argomento){
      
    case 2:
      email = "Questi\nsono\ngli headers\n\nquesto  il body";
      buffer = (char*) malloc(2);
      retCode = estraiHeaders(email, buffer, 2);
      free(buffer);
      if(retCode == BUFFER_TOO_SMALL)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("estraiHeaders(email, buffer, 2)", "BUFFER_TOO_SMALL", cR_vett[retCode], "La funzione estraiHeaders \
non controlla se il terzo argomento  una dimensione sufficiente a contenere il blocco degli headers.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
      
    }
    
  case HEADER_NOT_FOUND:
    
    switch(n_argomento){
      
    case 0:
      email = "Non c'\nnessun header";
      buffer = (char*) malloc(22);
      retCode = estraiHeaders(email, buffer, 22);
      free(buffer);
      if(retCode == HEADER_NOT_FOUND)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("estraiHeaders(email, buffer, 22)", "HEADER_NOT_FOUND", cR_vett[retCode], "La funzione estraiHeaders \
non controlla se il primo argomento  una email priva del blocco degli headers.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
      
    }

  case BODY_NOT_FOUND:
    
    switch(n_argomento){
      
    case 0:
      return NOT_IMPLEMENTED;
      
    default:
      return NOT_VALID;
      
    }

  default:
    return NOT_VALID;
  }
}

int test_primoHeader(int tipo_di_test, int n_argomento){
  char* buffer;
  char* headers = "Questo\n\t il\n primo\n";
  int retCode;
  char* cR_vett[] = {"RESULT_OK", "UNKNOWN_ERROR", "NULL_POINTER", "EMPTY_STRING", "FILE_MISSING", "FILE_UNREADABLE",
			   "FILE_UNWRITABLE", "FILE_EMPTY", "BUFFER_TOO_SMALL", "HEADER_NOT_FOUND", "BODY_NOT_FOUND", 
			   "BAD_HEADER", "MEM_ALLOC_ERROR"};

  if(n_argomento < 0 || n_argomento > 2)
    return ARG_MISSING;

  switch(tipo_di_test){
    
  case RESULT_OK:

    if(n_argomento == 0){
      buffer = (char*) malloc(21);
      if((retCode = primoHeader(headers, buffer, 21)) == RESULT_OK && strcmp(buffer, "Questo\n\t il\n primo") == 0){
	free(buffer);
	return TEST_PASSED;
      }
      free(buffer);
      GIOCO_PRINT_FAILURE("primoHeader(headers, buffer, 21)", "RESULT_OK", cR_vett[retCode], "La funzione primoHeader fallisce \
se gli vengono passati 3 argomenti corretti.");
      return TEST_FAILED;
    }
    return NOT_IMPLEMENTED;
    
  case NULL_POINTER:

    switch(n_argomento){

    case 0:
      buffer = (char*) malloc(2);
      retCode = primoHeader(NULL, buffer, 2);
      free(buffer);
      if(retCode == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("primoHeader(NULL, buffer, 2)", "NULL_POINTER", cR_vett[retCode], "La funzione primoHeader non \
controlla se il primo argomento  un puntatore nullo.");
      return TEST_FAILED;

    case 1:
      if((retCode = primoHeader(headers, NULL, 2)) == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("primoHeader(headers, NULL, 2)", "NULL_POINTER", cR_vett[retCode], "La funzione primoHeader non \
controlla se il secondo argomento  un puntatore nullo.");
      return TEST_FAILED;

    default:
      return NOT_VALID;
    }
    
  case EMPTY_STRING:
    
    switch(n_argomento){
      
    case 0:
      buffer = (char*) malloc(2);
      retCode = primoHeader("", buffer, 2);
      free(buffer);
      if(retCode == EMPTY_STRING)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("primoHeader(\"\", buffer, 2)", "EMPTY_STRING", cR_vett[retCode], "La funzione primoHeader non \
controlla se il primo argomento  una stringa vuota.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
    }

  case BUFFER_TOO_SMALL:
    
    switch(n_argomento){
      
    case 2:
      buffer = (char*) malloc(2);
      retCode = primoHeader(headers, buffer, 2);
      free(buffer);
      if(retCode == BUFFER_TOO_SMALL)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("primoHeader(headers, buffer, 2)", "BUFFER_TOO_SMALL", cR_vett[retCode], "La funzione primoHeader non \
controlla se il terzo argomento  una dimensione sufficiente a contenere il primo header.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
      
    }
    
  default:
    return NOT_VALID;
  }
}

int test_parseHeader(int tipo_di_test, int n_argomento){
  char* buffer_T;
  char* buffer_V;
  char* header = "Tipo: valore\n\theader";
  int retCode;
  char* cR_vett[] = {"RESULT_OK", "UNKNOWN_ERROR", "NULL_POINTER", "EMPTY_STRING", "FILE_MISSING", "FILE_UNREADABLE",
			   "FILE_UNWRITABLE", "FILE_EMPTY", "BUFFER_TOO_SMALL", "HEADER_NOT_FOUND", "BODY_NOT_FOUND", 
			   "BAD_HEADER", "MEM_ALLOC_ERROR"};

  if(n_argomento < 0 || n_argomento > 4)
    return ARG_MISSING;

  switch(tipo_di_test){
    
  case RESULT_OK:

    if(n_argomento == 0){
      buffer_T = (char*) malloc(6);
      buffer_V = (char*) malloc(15);
      if((retCode = parseHeader(header, buffer_T, 6, buffer_V, 15)) == RESULT_OK && strcmp(buffer_T, "Tipo:") == 0 && strcmp(buffer_V, "valore\n\theader") == 0){
	free(buffer_T);
	free(buffer_V);
	return TEST_PASSED;
      }
      free(buffer_T);
      free(buffer_V);
      GIOCO_PRINT_FAILURE("parseHeader(header, buffer_t, 6, buffer_V, 15)", "RESULT_OK", cR_vett[retCode], "La funzione \
parseHeader fallisce se gli vengono passati 5 argomenti corretti.");
      return TEST_FAILED;
    }
    return NOT_IMPLEMENTED;

  case NULL_POINTER:

    switch(n_argomento){

    case 0:
      buffer_T = (char*) malloc(2);
      buffer_V = (char*) malloc(2);
      retCode = parseHeader(NULL, buffer_T, 2, buffer_V, 2);
      free(buffer_T);
      free(buffer_V);
      if(retCode == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("parseHeader(NULL, buffer_T, 2, buffer_V, 2)", "NULL_POINTER", cR_vett[retCode], "La funzione \
parseHeader non controlla se il primo argomento  un puntatore nullo.");
      return TEST_FAILED;

    case 1:
      buffer_V = (char*) malloc(2);
      retCode = parseHeader(header, NULL, 2, buffer_V, 2);
      free(buffer_V);
      if(retCode == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("parseHeader(header, NULL, 2, buffer_V, 2)", "NULL_POINTER", cR_vett[retCode], "La funzione \
parseHeader non controlla se il secondo argomento  un puntatore nullo.");
      return TEST_FAILED;

    case 3:
      buffer_T = (char*) malloc(2);
      retCode = parseHeader(header, buffer_T, 2, NULL, 2);
      free(buffer_T);
      if(retCode == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("parseHeader(header, buffer_T, 2, NULL, 2)", "NULL_POINTER", cR_vett[retCode], "La funzione \
parseHeader non controlla se il quarto argomento  un puntatore nullo.");
      return TEST_FAILED;

    default:
      return NOT_VALID;
    }
    
  case EMPTY_STRING:
    
    switch(n_argomento){
      
    case 0:
      buffer_T = (char*) malloc(2);
      buffer_V = (char*) malloc(2);
      retCode = parseHeader("", buffer_T, 2, buffer_V, 2);
      free(buffer_T);
      free(buffer_V);
      if(retCode == EMPTY_STRING)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("parseHeader(\"\", buffer_T, 2, buffer_V, 2)", "EMPTY_STRING", cR_vett[retCode], "La funzione \
parseHeader non controlla se il primo argomento  una stringa vuota.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
    }

  case BUFFER_TOO_SMALL:
    
    switch(n_argomento){
      
    case 2:
      buffer_T = (char*) malloc(2);
      buffer_V = (char*) malloc(15);
      retCode = parseHeader(header, buffer_T, 2, buffer_V, 15);
      free(buffer_T);
      free(buffer_V);
      if(retCode == BUFFER_TOO_SMALL)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("parseHeader(headers, buffer_T, 2, buffer_V, 15)", "BUFFER_TOO_SMALL", cR_vett[retCode], "La funzione \
parseHeader non controlla se il terzo argomento  una dimensione sufficiente a contenere il tipo dello header.");
      return TEST_FAILED;

    case 4:
      buffer_T = (char*) malloc(6);
      buffer_V = (char*) malloc(2);
      retCode = parseHeader(header, buffer_T, 6, buffer_V, 2);
      free(buffer_T);
      free(buffer_V);
      if(retCode == BUFFER_TOO_SMALL)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("parseHeader(headers, buffer_T, 6, buffer_V, 2)", "BUFFER_TOO_SMALL", cR_vett[retCode], "La funzione \
parseHeader non controlla se il quinto argomento  una dimensione sufficiente a contenere il valore dello header.");
      return TEST_FAILED;

    default:
      return NOT_VALID;
    }
 
  case BAD_HEADER:
    
    switch(n_argomento){
      
    case 0:
      buffer_T = (char*) malloc(2);
      buffer_V = (char*) malloc(2);
      retCode = parseHeader(":pippo", buffer_T, 2, buffer_V, 2);
      free(buffer_T);
      free(buffer_V);
      if(retCode == BAD_HEADER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("parseHeader(\":pippo\", buffer_T, 2, buffer_V, 2)", "BAD_HEADER", cR_vett[retCode], "La funzione \
parseHeader non controlla se il primo argomento  uno header con sintassi errata.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
    }
    
  default:
    return NOT_VALID;
  }
}

int test_primaParola(int tipo_di_test, int n_argomento){
  char* buffer;
  int retCode;
  char* cR_vett[] = {"RESULT_OK", "UNKNOWN_ERROR", "NULL_POINTER", "EMPTY_STRING", "FILE_MISSING", "FILE_UNREADABLE",
			   "FILE_UNWRITABLE", "FILE_EMPTY", "BUFFER_TOO_SMALL", "HEADER_NOT_FOUND", "BODY_NOT_FOUND", 
			   "BAD_HEADER", "MEM_ALLOC_ERROR"};

  if(n_argomento < 0 || n_argomento > 2)
    return ARG_MISSING;

  switch(tipo_di_test){
    
  case RESULT_OK:

    if(n_argomento == 0){
      buffer = (char*) malloc(8);
      if((retCode = primaParola("$%$&%Pippo96$%$%", buffer, 8)) == RESULT_OK && strcmp(buffer, "Pippo96") == 0){
	if((retCode = primaParola("%&$!", buffer, 8)) == RESULT_OK && strcmp(buffer, "") == 0){
	  if((retCode = primaParola("$%a e i", buffer, 8)) == RESULT_OK && strcmp(buffer, "") == 0){
	    free(buffer);
	    return TEST_PASSED;
	  }
	  else{
	    free(buffer);
	    GIOCO_PRINT_FAILURE("primaParola(\"$%a e i\", buffer, 8)", "RESULT_OK", cR_vett[retCode], "La funzione primaParola \
fallisce se gli vengono passati 3 argomenti corretti.");
	    return TEST_FAILED;
	  }
	}
	else{
	  free(buffer);
	  GIOCO_PRINT_FAILURE("primaParola(\"%&$!\", buffer, 8)", "RESULT_OK", cR_vett[retCode], "La funzione primaParola \
fallisce se gli vengono passati 3 argomenti corretti.");
	return TEST_FAILED;
	}
      }
      else{
	free(buffer);
	GIOCO_PRINT_FAILURE("primaParola(\"$%$&%Pippo96$%$%\", buffer, 8)", "RESULT_OK", cR_vett[retCode], "La funzione \
primaParola fallisce se gli vengono passati 3 argomenti corretti.");
	return TEST_FAILED;
      }
    }
    return NOT_IMPLEMENTED;

  case NULL_POINTER:

    switch(n_argomento){

    case 0:
      buffer = (char*) malloc(2);
      retCode = primaParola(NULL, buffer, 2);
      free(buffer);
      if(retCode == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("primaParola(NULL, buffer, 2)", "NULL_POINTER", cR_vett[retCode], "La funzione primaParola non \
controlla se il primo argomento  un puntatore nullo.");
      return TEST_FAILED;

    case 1:
      if((retCode = primaParola("$%$&%Pippo96$%$%", NULL, 2)) == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("primaParola(\"$%$&%Pippo96$%$%\", NULL, 2)", "NULL_POINTER", cR_vett[retCode], "La funzione \
primaParola non controlla se il secondo argomento  un puntatore nullo.");
      return TEST_FAILED;

    default:
      return NOT_VALID;
    }
    
  case EMPTY_STRING:
    
    switch(n_argomento){
      
    case 0:
      buffer = (char*) malloc(2);
      retCode = primaParola("", buffer, 2);
      free(buffer);
      if(retCode == EMPTY_STRING)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("primaParola(\"\", buffer, 2)", "EMPTY_STRING", cR_vett[retCode], "La funzione primaParola non \
controlla se il primo argomento  una stringa vuota.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
    }

  case BUFFER_TOO_SMALL:
    
    switch(n_argomento){
      
    case 2:
      buffer = (char*) malloc(2);
      retCode = primaParola("$%$&%Pippo96$%$%", buffer, 2);
      free(buffer);
      if(retCode == BUFFER_TOO_SMALL)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("primaParola(\"$%$&%Pippo96$%$%\", buffer, 2)", "BUFFER_TOO_SMALL", cR_vett[retCode], "La funzione \
primaParola non controlla se il terzo argomento  una dimensione sufficiente a contenere la prima parola.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
      
    }
    
  default:
    return NOT_VALID;
  }
}

int test_tokenizzaEmail(int tipo_di_test, int n_argomento){
  Token* tokenArray;
  char* email = "From: %%%andrea@sterbini.org!!!\nTo: |g_babbo_natale@northpole.org\n\nCaro \
Babbo Natale,\nquest'anno ho cercato di essere molto buono e di non bocciare + nessuno";
  char* tokens[] = {"From:", "andrea", "From:", "sterbini", "To:", "babbo", "To:", "natale", "To:", "northpole", "To:", "org", "Caro", "Babbo", "Natale", "quest", "anno", "ho", "cercato", "di", "essere", "molto", "buono", "di", "non", "bocciare", "nessuno"};
  int retCode, i, j, flag;
  char* cR_vett[] = {"RESULT_OK", "UNKNOWN_ERROR", "NULL_POINTER", "EMPTY_STRING", "FILE_MISSING", "FILE_UNREADABLE",
			   "FILE_UNWRITABLE", "FILE_EMPTY", "BUFFER_TOO_SMALL", "HEADER_NOT_FOUND", "BODY_NOT_FOUND", 
			   "BAD_HEADER", "MEM_ALLOC_ERROR"};

  if(n_argomento < 0 || n_argomento > 2)
    return ARG_MISSING;

  switch(tipo_di_test){
    
  case RESULT_OK:
    if(n_argomento == 0){
      tokenArray = (Token*) malloc(30 * sizeof(Token));
      retCode = tokenizzaEmail(email, tokenArray, 30);
      //printTokens(tokenArray, 30);
      flag = 0;
      for(i = 0; (tokenArray[i].contesto != NULL || tokenArray[i].parola != NULL) && flag == 0; i++){
	if(tokenArray[i].contesto != NULL)
	  if(strcmp(tokenArray[i].contesto, tokens[j]) != 0){
	     flag = 1;
	     break;
	  }
	   else
	     j++;
	if(strcmp(tokenArray[i].parola, tokens[j]) != 0)
	  flag = 1;
	else
	  j++;
      }
      for(i = 0; tokenArray[i].contesto != NULL || tokenArray[i].parola != NULL ; i++){
	if(tokenArray[i].contesto != NULL)
	  free(tokenArray[i].contesto);
	if(tokenArray[i].parola != NULL)
	  free(tokenArray[i].parola);
      }
      free(tokenArray);
      printf("\n%d\n", flag);
      if(retCode == RESULT_OK && flag == 0)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("tokenizzaEmail(email, tokenArray, 30)", "RESULT_OK", cR_vett[retCode], "La funzione tokenizzaEmail \
fallisce se gli vengono passati 3 argomenti corretti.");
      return TEST_FAILED;
    }
    return NOT_IMPLEMENTED;
    
  case NULL_POINTER:

    switch(n_argomento){

    case 0:
      tokenArray = (Token*) malloc(30 * sizeof(Token));
      retCode = tokenizzaEmail(NULL, tokenArray, 30);
      free(tokenArray);
      if(retCode == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("tokenizzaEmail(NULL, tokenArray, 30)", "NULL_POINTER", cR_vett[retCode], "La funzione tokenizzaEmail \
non controlla se il primo argomento  un puntatore nullo.");
      return TEST_FAILED;

    case 1:
      retCode = tokenizzaEmail("ab ca\n\ndef", NULL, 30);
      if(retCode == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("tokenizzaEmail(\"ab ca\n\ndef\", NULL, 30)", "NULL_POINTER", cR_vett[retCode], "La funzione \
tokenizzaEmail non controlla se il secondo argomento  un puntatore nullo.");
      return TEST_FAILED;

    default:
      return NOT_VALID;
    }

  case EMPTY_STRING:
    
    switch(n_argomento){
      
    case 0:
      tokenArray = (Token*) malloc(30 * sizeof(Token));
      retCode = tokenizzaEmail("", tokenArray, 30);
      free(tokenArray);
      if(retCode == EMPTY_STRING)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("tokenizzaEmail(\"\", tokenArray, 30)", "EMPTY_STRING", cR_vett[retCode], "La funzione tokenizzaEmail \
non controlla se il primo argomento  una stringa vuota.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
    }

  case BUFFER_TOO_SMALL:
    
    switch(n_argomento){
      
    case 2:
      tokenArray = (Token*) malloc(2 * sizeof(Token));
      retCode = tokenizzaEmail(email, tokenArray, 2);
      free(tokenArray);
      if(retCode == BUFFER_TOO_SMALL)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("tokenizzaEmail(email, tokenArray, 2)", "BUFFER_TOO_SMALL", cR_vett[retCode], "La funzione \
tokenizzaEmail non controlla se il terzo argomento  una dimensione sufficiente a contenere la prima parola.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
      
    }

  case BODY_NOT_FOUND:
    
    switch(n_argomento){
      
    case 0:
      tokenArray = (Token*) malloc(5 * sizeof(Token));
      retCode = tokenizzaEmail("abc de\n\n", tokenArray, 5);
      free(tokenArray);
      if(retCode == BODY_NOT_FOUND)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("tokenizzaEmail(\"abc de\\n\\n\", tokenArray, 5)", "BODY_NOT_FOUND", cR_vett[retCode], "La funzione \
tokenizzaEmail non controlla se il primo argomento  una email priva di body.");
      return TEST_FAILED;
      
    default:
      return NOT_VALID;
      
    }

  default:
    return NOT_VALID;
  }
}

int test_printTokens(int tipo_di_test, int n_argomento){
  Token tokenArray[8];
  char* output;
  FILE* fptr;
  int retCode;
  char* cR_vett[] = {"RESULT_OK", "UNKNOWN_ERROR", "NULL_POINTER", "EMPTY_STRING", "FILE_MISSING", "FILE_UNREADABLE",
			   "FILE_UNWRITABLE", "FILE_EMPTY", "BUFFER_TOO_SMALL", "HEADER_NOT_FOUND", "BODY_NOT_FOUND", 
			   "BAD_HEADER", "MEM_ALLOC_ERROR"};

  if(n_argomento < 0 || n_argomento > 1)
    return ARG_MISSING;

  switch(tipo_di_test){
    
  case RESULT_OK:
    if(n_argomento == 0){
      tokenArray[0].contesto = "From:";
      tokenArray[0].parola = "andrea";
      tokenArray[1].contesto = "From:";
      tokenArray[1].parola = "sterbini";
      tokenArray[2].contesto = "From:";
      tokenArray[2].parola = "org";
      tokenArray[3].contesto = "To:";
      tokenArray[3].parola = "babbo";
      tokenArray[4].contesto = NULL;
      tokenArray[4].parola = "Caro";
      tokenArray[5].contesto = NULL;
      tokenArray[5].parola = NULL;
      fptr = freopen("temp.txt", "w", stdout);
      retCode = printTokens(tokenArray, 8);
      fclose(fptr);
      fptr = fopen("temp.txt", "r");
      output = (char*) malloc(53);
      fread(output, sizeof(char), 52, fptr);
      output[52] = '\0';
      stdout = freopen("/dev/tty", "w", fptr);
      //printTokens(tokenArray, 8);
      //printf("\n%s\n", output);
      if(retCode == RESULT_OK && strcmp(output, "From:*andrea\nFrom:*sterbini\nFrom:*org\nTo:*babbo\nCaro") == 0){
	free(output);
	return TEST_PASSED;
      }
      free(output);
      GIOCO_PRINT_FAILURE("printTokens(tokenArray, 8)", "RESULT_OK", cR_vett[retCode], "La funzione printTokens \
fallisce se gli vengono passati 2 argomenti corretti.");
      return TEST_FAILED;
    }
    return NOT_IMPLEMENTED;
    
  case NULL_POINTER:

    if(n_argomento == 0){
      
      if((retCode = printTokens(NULL, 8)) == NULL_POINTER)
	return TEST_PASSED;
      GIOCO_PRINT_FAILURE("printTokens(NULL, 8)", "NULL_POINTER", cR_vett[retCode], "La funzione printTokens \
non controlla se il primo argomento  un puntatore nullo.");
      return TEST_FAILED;
    }
    else
      return NOT_VALID;
    
  default:
    return NOT_VALID;
  }
}

