#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include "modulo1.h"

#define ML_ALPHANUM "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"

extern int errno;
int free_and_exit(int error, int n, ...);
int use_func(int func(), char *src, char **dest, int n, char **dest2);
int intotoken(Token *t, char *cont, char *word);
int find_words(char *string);

/* funzione che copia da un file i primi 'dimensione' caratteri nel blocco di memoria 'testo'
   e mette nella variabile 'dimensione' il numero di caratteri letti */
int leggiFile(char * fname, int dim, char * text) {
	FILE *fptr = NULL;

	errno = 0;

	/* check sugli argomenti */
	if(!fname || !text) return NULL_POINTER;

	/* check sulla lunghezza del nome degli argomenti */
	/* dimensione 0  considerato empty string? */
	if (dim <= 0) return BUFFER_TOO_SMALL;

	if (strlen(fname) == 0) return EMPTY_STRING;

	if (strlen(fname) > FILENAME_MAX) return UNKNOWN_ERROR;

	/* check sul file */
	if((fptr = fopen(fname, "r")) == NULL)
		switch(errno) {
			case ENOENT:
				return FILE_MISSING;
			case EACCES:
				return FILE_UNREADABLE;
			default:
				return UNKNOWN_ERROR;
		}

	/* copiare da file tot dimensione in text, aggiungendo \0 alla fine. 
	   se file > dim => copiare fino a dim */
	if((dim = fread(text, sizeof(char), dim - 1, fptr)) == 0)
		return FILE_EMPTY;
	text[dim] = '\0';
	fclose(fptr);

	return RESULT_OK;
}



/* funzione che individua le intestazioni della email e le copia nel blocco 
   di memoria 'headers' di lunghezza 'dimh' */
int estraiHeaders(char *text, char *head, int dimh) {
	int i;
	char *s;
	if(!text || !head) return NULL_POINTER;

	/* stringa vuota: vedi sopra */
	if(strlen(text) == 0) return EMPTY_STRING;

	/* dimensione negativa o 0 */
	if(dimh <= 0) return BUFFER_TOO_SMALL;
	
	/* che ne faccio dell'\n' iniziale?! */
	/* il '\n' finale  considerato negli headers? */
	if(!(s = strstr(text, "\n\n")) || text[0] == '\n') return HEADER_NOT_FOUND;

	/* il gap tra l'inizio della stringa e \n\n  troppo ampio */
	if((i = ++s - text) >= dimh) return BUFFER_TOO_SMALL;
	head = strncpy(head, text, i);
	head[i] = '\0';

	return RESULT_OK;
}



/* funzione che estrae il primo header dagli headers e lo copia in primo_header */
int primoHeader(char * head, char * p_head, int dim) {
	int i;
	char *s = head;

	/* ... */
	if(!head || !p_head) return NULL_POINTER;

	/* ...... */
	if(strlen(head) == 0) return EMPTY_STRING;

	/* s punta ogni linea (a partire dalla seconda) */
	do {
		/* posizione di \n*/
		if(!(s = strchr(s, '\n')) || s == head) return HEADER_NOT_FOUND;

		/* i = posizione di \n - posizione 0  */
		if( (i = ++s - head) > dim ) return BUFFER_TOO_SMALL;
	}
	while(strlen(head) > i && ( (head[i] == '\t') || (head[i] == ' ') ) );
	/* con il -1 non copia il \n finale */
	p_head = strncpy(p_head, head, i-1);
	p_head[i-1] = '\0';
	
	return RESULT_OK;
}




/* funzione che estrae il tipo ed il valore di un header */
int parseHeader(char * head, char * type, int dimt, char * val, int dimv) {
	char *s;
	int i;

	/* eh beh */
	if(!head || !type || !val) return NULL_POINTER;

	/* ... */
	if(strlen(head) == 0) return EMPTY_STRING;

	/* non c' uno spazio o un tab => bad header */
	if(!(s = strpbrk(head, " \t") )) return BAD_HEADER;
	
	/* dim t > s - head (cos da metterci il '\0'. stessa cosa per dimv */
	if((i = s - head) > dimt || (strchr(head, '\0') - s) > dimv)
		return BUFFER_TOO_SMALL;

	strncpy(type, head, i);
	type[i] = '\0';
	i = strlen(++s);
	strncpy(val, s, i);
	val[i] = '\0';

	if(strpbrk(val, ML_ALPHANUM) == NULL || strcspn(type, ":") == 0)
		return BAD_HEADER;
	
	return RESULT_OK;
}



/* funzione che estrae la prima parola da una stringa */
int primaParola(char *text, char *word, int dim) {
	char *s = text;
	int i;
	
	/* vabb */
	if(!text || !word) return NULL_POINTER;

	if(strlen(text) == 0) return EMPTY_STRING;

	/* vede quanti caratteri deve copiare ( > 1) */
	do {
		/* ignora i caratteri nulli */
		if(!(s = strpbrk(s, ML_ALPHANUM)) ) return EMPTY_STRING;
		i = strspn(s, ML_ALPHANUM);
		s++;
	} while (i < 2 );

	/* check sul buffer */
	if(i < dim) {
		strncpy(word, s-1, i);
		/* terminatore */
		word[i] = '\0';
	} else
		/* buffer troppo piccolo */
		return BUFFER_TOO_SMALL;
	
	
	return RESULT_OK;
}






/* funzione che estrae i primi dimT token dalla email */
int tokenizzaEmail(char *email_text, Token tokens[], int dimT) {
	char *email = email_text, *headers, *p_head, *type, *val, *word, *tmp, *headers_base;
	int i= 0, error;
			
	if(!email_text || !tokens) return NULL_POINTER;


	if(dimT <= 0) return BUFFER_TOO_SMALL;
	tokens[0].contesto = tokens[0].parola = NULL;

	/* estrae gli headers */

	if((error = use_func(estraiHeaders, email, &headers_base, 0, NULL)) != RESULT_OK)
		return free_and_exit(error, 1, headers_base);

	headers = headers_base;

	while(strlen(headers) > 0 && i < dimT) {

		/* prende il primo header */
		if((error = use_func(primoHeader, headers, &p_head, 0, NULL)) != RESULT_OK)
			return free_and_exit(error, 2, headers_base, p_head);

		/*  e fa il parsing */
		if((error = use_func(parseHeader, p_head, &type, 1, &val)) != RESULT_OK)
			return free_and_exit(error, 4, headers_base, p_head, type, val);
		tmp = val;

		/* aggiunge gli elementi ai tokens */
		for(; i < dimT && find_words(tmp); i++) {

			/*mette la prima parola in word */
			if((error = use_func(primaParola, tmp, &word, 0, NULL)) > UNKNOWN_ERROR)
				return free_and_exit(error, 5, headers_base, p_head, type, val, word);
			/* inserisce le parole nel token */
			if(intotoken(&(tokens[i]), type, word) == MEM_ALLOC_ERROR)
				return free_and_exit(MEM_ALLOC_ERROR, 5, headers_base, p_head, type, val, word);

			tmp = strstr(tmp, word) + strlen(word);
			free(word);

		} /* for -> parsing 1 header */

		/* rimuove il primo header (compreso di \n finale) */
		headers = headers + strlen(p_head) + 1;
		/* libero le zone allocate */
		free(p_head); free(val); free(type);

	} /* while -> parsing all headers */

	/* rimuove gli headers dal corpo dell'email */
	email = email + strlen(headers_base) + 1;
	free(headers_base);

	if(strlen(email) < 2) return BODY_NOT_FOUND;

	/* tokenizza il corpo dell'email */
	for( ; i < dimT && find_words(email) > 0; i++) {
		/* in caso di errore permette di fare la free */
		tokens[i].parola = NULL;
		
		/* prende la prima parola */
		if((error = use_func(primaParola, email, &word, 0, NULL)) != RESULT_OK)
			return free_and_exit(error, 1, word);
		/* setta testo e contesto */
		if( intotoken(&(tokens[i]), NULL, word) == MEM_ALLOC_ERROR)
			return free_and_exit(MEM_ALLOC_ERROR, 1, word);
		/* avanza il putatore */
		email = strstr(email, word) + strlen(word);
		free(word);
	}

	/* il buffer non  bastato */
	if(i >= dimT) return BUFFER_TOO_SMALL;

	/* chiude gli ultimi token */
	tokens[i].parola = tokens[i].contesto = NULL;

	return RESULT_OK;
}




/* funzione che stampa i token uno per riga nel formato 'contesto*parola' oppure 'parola' */
int printTokens(Token tokenArray[], int dimT) {
	int i = 0;
	
	/* null pointer */
	if(!tokenArray) return NULL_POINTER;

	/* dimensione negativa */
	if(dimT <= 0) return BUFFER_TOO_SMALL;

	while(i < dimT && tokenArray[i].parola != NULL) {
		if(tokenArray[i].contesto) {
			if(strlen(tokenArray[i].contesto) == 0)
				return EMPTY_STRING;
			printf("%s*", tokenArray[i].contesto);
		}
		
		if(strlen(tokenArray[i].parola) < 2) return EMPTY_STRING;
		printf("%s\n", tokenArray[i].parola);
		i++;
	}
	
	return RESULT_OK;
}




/* funzione che mi evita molte righe di codice */
int use_func(int func(), char *src, char **dest, int n, char **dest2) {
	
	/* alloca per dest */
	if(!(*dest = (char *)malloc(sizeof(char) * (strlen(src) + 1) ) ) )
		return MEM_ALLOC_ERROR;
	if(n) {
		/* alloca per dest 2 */
		if(!(*dest2 = (char *)malloc(sizeof(char) * (strlen(src) + 1) ) ) )
			return MEM_ALLOC_ERROR;
		n = func(src, *dest, strlen(src) + 1, *dest2, strlen(src) + 1);
	} else
		n = func(src, *dest, strlen(src) + 1);

	return n;
}


/* free & exit */
int free_and_exit(int error, int n, ...) {
	int i;
	va_list lst;

	va_start(lst, n);
	for(i = 0; i < n; i++)
		free(va_arg(lst, char*));
	va_end(lst);

	return error;
}


/* inserisce il contesto e la parola nel token */
int intotoken(Token *t, char *cont, char *word) {

	/* copia il contesto (se c') */
	if(cont) {
		if(!(t->contesto = malloc(sizeof(char) * (strlen(cont + 1) ) ) ))
			return MEM_ALLOC_ERROR;
		strncpy(t->contesto, cont, strlen(cont) + 1);
	} else
		t->contesto = NULL;
	
	/* copia la parola */
	if(!(t->parola = malloc(sizeof(char) * (strlen(word + 1) ) ) ))
		return MEM_ALLOC_ERROR;

	strncpy(t->parola, word, strlen(word) + 1);
	
	return RESULT_OK;

}



/* check sulle stringhe di parole > di 2 caratteri */
int find_words(char *s) {
	char *tmp = s;
	int i;

	while(strlen(tmp) > 0) {
		tmp = strpbrk(tmp, ML_ALPHANUM);
		if(!tmp) return 0;
		if((i = strspn(tmp, ML_ALPHANUM)) >= 2)	return 1;
		tmp = tmp + i;
	}
	
	return 0;
}


