#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include "testModulo1.h"


#define ML_MAXARGS 5

#define ML_STRHEAD	"From localhost\n\t127.0.0.1\n\tasdasdasd\nFrom: ciccio@formaggio.org\nTo: secco@prociutto.org\nSubject: 1, 2, 3 prova\n\n"
#define ML_STRH1		"From"
#define ML_STRH2		"localhost\n\t127.0.0.1\n\tasdasdasd"

typedef struct a_test {
	int (*func)();
	int n_arg;
	char mask_free;
	unsigned short mask_test[ML_MAXARGS];
	void *arg[ML_MAXARGS];
	char *motivo;
	char *nomefun;
	int result;
} test;

	/* 	RESULT_OK			0	1
		UNKNOWN_ERROR		1	2
		NULL_POINTER		2	4
		EMPTY_STRING		3	8
		FILE_MISSING		4	16
		FILE_UNREADABLE		5	32
		FILE_UNWRITABLE		6	64
		FILE_EMPTY			7	128
		BUFFER_TOO_SMALL	8	256
		HEADER_NOT_FOUND	9	512
		BODY_NOT_FOUND		10	1024
		BAD_HEADER			11	2048
		MEM_ALLOC_ERROR		12	4096	*/

void gen_email(char *fname, int size);
char *gen_mem(int size, int ctrl);
int check_res(test *t, int t_type);
void change_arg(test *t, int n, void *data);
char *intocode(int code);
void printfailed(test *t, int t_type);
void free_tokens(Token *t);

/* test leggiFile */
int test_leggiFile(int t_type, int arg) {
	test t;
	int result;

	t.func = leggiFile;
	t.nomefun = "leggiFile";
	t.n_arg = 3;
	t.mask_free = 0;
	t.mask_test[0] = 191;
	t.mask_test[1] = 259;
	t.mask_test[2] = 7;

	
	if(arg >= t.n_arg) return ARG_MISSING;

	/* vede se il test  valido */
	if(!(t.mask_test[arg] & (1 << t_type)))
		return NOT_VALID;

	/* cambia gli argomenti base del test e genera l'email*/
	t.arg[0] = "email1.txt";
	t.arg[1] = (void *)300;
	t.arg[2] = gen_mem(300, 0);
	t.mask_free |= 1 << 3;
	t.motivo = "il caso base";

	/* cambia gli argomenti necessari */
	switch(t_type) {
		case NULL_POINTER:
			change_arg(&t, arg, NULL);
			t.motivo = "il null pointer";
			break;
		case EMPTY_STRING:
			change_arg(&t, arg, "");
			t.motivo = "la stringa vuota";
			break;
		case FILE_UNREADABLE:
			change_arg(&t, arg, "file-unreadable");
			t.motivo = "file illeggibili";
			break;
		case FILE_MISSING:
			change_arg(&t, arg, gen_mem(12, 1));
			t.mask_free |= 1 << (arg + 1);
			t.motivo = "file inesistenti";
			break;
		case UNKNOWN_ERROR:
			if(arg != 0) return NOT_IMPLEMENTED;
			change_arg(&t, arg, gen_mem(FILENAME_MAX + 10, 1));
			t.motivo = "nomi file troppo lunghi";
			t.mask_free |= 1 << (arg + 1);
			break;
		case BUFFER_TOO_SMALL:
			t.arg[arg] = (void *)-5;
			t.motivo = "dimensione negativa";
			break;
		case FILE_EMPTY:
			change_arg(&t, arg, "file-empty");
			t.motivo = "file vuoti";
			break;
		case RESULT_OK:
			break;
		default:
			return NOT_IMPLEMENTED;
			
	}	
	
	result = check_res(&t, t_type);
	if(result == TEST_PASSED) {
		if(	t_type == RESULT_OK) {
			if(arg == 1) {
				/* continua a fare test sulla dimensione */
				t.arg[1] = (void *)350;
				change_arg(&t, 2, gen_mem(350, 0));
				t.motivo = "dimensione uguale all'email";
				result = check_res(&t, t_type);
				if(result == TEST_PASSED) {
					t.arg[1] = (void *)500;
					change_arg(&t, 2, gen_mem(500, 0));
					t.motivo = "dimensione maggiori all'email";
					result = check_res(&t, t_type);
				}
			}
			/*result = falsepositive(&t);*/
		}
	}

	if(result == TEST_FAILED) printfailed(&t, t_type);

	for(t.n_arg -= 1; t.n_arg > 0; t.n_arg -= 1)
		change_arg(&t, t.n_arg, NULL);

	return result;
}




/* test estraiHeaders */
int test_estraiHeaders(int t_type, int arg) {
	test t;
	int result = -1;

	t.func = estraiHeaders;
	t.nomefun = "estraiHeaders";
	t.n_arg = 3;
	t.mask_free = 0;
	t.mask_test[0] = 527;
	t.mask_test[1] = 7;
	t.mask_test[2] = 259;

	
	if(arg >= t.n_arg) return ARG_MISSING;

	/* vede se il test  valido */
	if(!(t.mask_test[arg] & (1 << t_type)))
		return NOT_VALID;

	/* cambia gli argomenti base del test e genera l'email*/
	t.arg[0] = gen_mem(400, 0);
	leggiFile("email1.txt", 400, t.arg[0]);
	t.arg[1] = gen_mem(1000, 0);
	t.mask_free = 6;
	t.arg[2] = (void *) 1000;
	t.motivo = "il caso base";

	/* cambia gli argomenti necessari */
	switch(t_type) {
		case RESULT_OK:
			break;
		case NULL_POINTER:
			change_arg(&t, arg, NULL);
			t.motivo = "il null pointer";
			break;
		case EMPTY_STRING:
			change_arg(&t, arg, "");
			t.motivo = "una stringa vuota";
			break;
		case BUFFER_TOO_SMALL:
			t.arg[arg] = (void *)-5;
			t.motivo = "dimensione negativa";
			result = check_res(&t, t_type);
			if(result == TEST_PASSED) {
				t.arg[arg] = (void *)10;
				t.motivo = "buffer troppo piccolo";
				result = check_res(&t, t_type);
			}		
			break;
		case HEADER_NOT_FOUND:
			change_arg(&t, arg, "\n\n");
			t.motivo = "un header inesisente";
			break;
		default:
			result = NOT_IMPLEMENTED;
			
	}	
	
	if(result == -1) {
		result = check_res(&t, t_type);
		
		if(result == TEST_PASSED) {
			if(	t_type == RESULT_OK) {
				if(arg == 2) {
					/* fa un altro test sull'header */
					t.arg[2] = (void *)strlen(ML_STRHEAD);
					change_arg(&t, 1, gen_mem((int)strlen(ML_STRHEAD), 0));
					t.motivo = "buffer preciso per gli header";
					result = check_res(&t, t_type);
				}
				/*result = falsepositive(&t);*/
			}
		}
	} /* result gi ritornato */

	if(result == TEST_FAILED) printfailed(&t, t_type);

	for(t.n_arg -= 1; t.n_arg > 0; t.n_arg -= 1)
		change_arg(&t, t.n_arg, NULL);
	
	return result;
}


/* test per primoHeader */
int test_primoHeader(int t_type, int arg) {
	test t;
	int result = -1;

	t.func = primoHeader;
	t.nomefun = "primoHeader";
	t.n_arg = 3;
	t.mask_free = 0;
	t.mask_test[0] = 527;
	t.mask_test[1] = 7;
	t.mask_test[2] = 259;

	
	if(arg >= t.n_arg) return ARG_MISSING;

	/* vede se il test  valido */
	if(!(t.mask_test[arg] & (1 << t_type)))
		return NOT_VALID;

	/* cambia gli argomenti base del test e genera l'email*/
	t.arg[0] = ML_STRHEAD;
	t.arg[1] = gen_mem(50, 0);
	t.mask_free = 4;
	t.arg[2] = (void *) 50;
	t.motivo = "il caso base";

	/* cambia gli argomenti necessari */
	switch(t_type) {
		case RESULT_OK:
			break;
		case NULL_POINTER:
			change_arg(&t, arg, NULL);
			t.motivo = "il null pointer";
			break;
		case EMPTY_STRING:
			change_arg(&t, arg, "");
			t.motivo = "una stringa vuota";
			break;
		case BUFFER_TOO_SMALL:
			t.arg[arg] = (void *)-5;
			t.motivo = "dimensione negativa";
			result = check_res(&t, t_type);
			if(result == TEST_PASSED) {
				t.arg[arg] = (void *)10;
				t.motivo = "buffer troppo piccolo";
				result = check_res(&t, t_type);
			}		
			break;
		case HEADER_NOT_FOUND:
			change_arg(&t, arg, "iononsonounheader");
			t.motivo = "un header inesisente";
			break;
		default:
			result = NOT_IMPLEMENTED;
			
	}	
	
	if(result == -1) {
		result = check_res(&t, t_type);
		
		if(result == TEST_PASSED) {
			if(	t_type == RESULT_OK) {
				if(arg == 2) {
					/* fa un altro test sull'header */
					t.arg[2] = (void *)(strlen(ML_STRH1) + strlen(ML_STRH2) + 2);
					change_arg(&t, 1, gen_mem((int)(strlen(ML_STRH1) + strlen(ML_STRH2) + 2), 0));
					t.motivo = "buffer preciso per il primo header";
					result = check_res(&t, t_type);
				}
			}
		}
	} /* result gi ritornato */

	if(result == TEST_FAILED) printfailed(&t, t_type);

	for(t.n_arg -= 1; t.n_arg > 0; t.n_arg -= 1)
		change_arg(&t, t.n_arg, NULL);

	return result;
}



/* test per parseHeader */
int test_parseHeader(int t_type, int arg) {
	test t;
	int result = -1;

	t.func = parseHeader;
	t.nomefun = "parseHeader";
	t.n_arg = 5;
	t.mask_free = 0;
	t.mask_test[0] = 2063;
	t.mask_test[3] = t.mask_test[1] = 7;
	t.mask_test[4] = t.mask_test[2] = 259;
	

	
	if(arg >= t.n_arg) return ARG_MISSING;

	/* vede se il test  valido */
	if(!(t.mask_test[arg] & (1 << t_type)))
		return NOT_VALID;

	/* cambia gli argomenti base del test e genera l'email*/
	t.arg[0] = gen_mem(150, 0);
	primoHeader(ML_STRHEAD, t.arg[0], 150);
	t.arg[1] = gen_mem(strlen(ML_STRH1) + 15, 0);
	t.arg[2] = (void *) (strlen(ML_STRH1) + 15);
	t.arg[3] = gen_mem(strlen(ML_STRH2) + 15, 0);
	t.arg[4] = (void *) (strlen(ML_STRH2) + 15);
	t.mask_free = 22;
	t.motivo = "il caso base";

	/* cambia gli argomenti necessari */
	switch(t_type) {
		case RESULT_OK:
			break;
		case NULL_POINTER:
			change_arg(&t, arg, NULL);
			t.motivo = "il null pointer";
			break;
		case EMPTY_STRING:
			change_arg(&t, arg, "");
			t.motivo = "una stringa vuota";
			break;
		case BUFFER_TOO_SMALL:
			t.arg[arg] = (void *)-5;
			t.motivo = "dimensione negativa";
			result = check_res(&t, t_type);
			if(result == TEST_PASSED) {
				t.arg[arg] = (void *)(strlen(ML_STRH1) - 2);
				t.motivo = "un buffer troppo piccolo";
				result = check_res(&t, t_type);
			}		
			break;
		case BAD_HEADER:
			change_arg(&t, arg, " ");
			t.motivo = "un header fatto male";
			result = check_res(&t, t_type);
			if(result == TEST_PASSED) {
				change_arg(&t, arg, ": %!prova");
				result = check_res(&t, t_type);
			}
			if(result == TEST_PASSED) {
				change_arg(&t, arg, "From\t%=");
				result = check_res(&t, t_type);
			}
			if(result == TEST_PASSED) {
				change_arg(&t, arg, "From()= &_");
				result = check_res(&t, t_type);
			}
			if(result == TEST_PASSED) {
				change_arg(&t, arg, " beh");
				result = check_res(&t, t_type);
			}
			break;
		default:
			result = NOT_IMPLEMENTED;
			
	}	
	
	if(result == -1) {
		result = check_res(&t, t_type);
		
		if(result == TEST_PASSED) {
			if(	t_type == RESULT_OK) {
				if(arg == 2 || arg == 4) {
					/* fa un altro test sull'header */
					t.arg[arg] = (void *)strlen((arg == 2) ? ML_STRH1 : ML_STRH2) + 1;
					change_arg(&t, arg - 1, gen_mem((int)strlen((arg == 2) ? ML_STRH1 : ML_STRH2) + 1, 0));
					t.motivo = "buffer preciso per gli argomenti";
					result = check_res(&t, t_type);
				}
				/*result = falsepositive(&t);*/
				if(arg == 0) {
					/* fa un altro test sull'header */
					change_arg(&t, 0, "From -ciao");
					t.motivo = "un header sintatticamente corretto";
					result = check_res(&t, t_type);
				}

			}
		}
	} /* result gi ritornato */

	if(result == TEST_FAILED) printfailed(&t, t_type);

	for(t.n_arg -= 1; t.n_arg > 0; t.n_arg -= 1)
		change_arg(&t, t.n_arg, NULL);

	return result;
}



/* test per primaparola */
int test_primaParola(int t_type, int arg) {
	test t;
	int result = -1;

	t.func = primaParola;
	t.nomefun = "primaParola";
	t.n_arg = 3;
	t.mask_free = 0;
	t.mask_test[0] = 15;
	t.mask_test[1] = 7;
	t.mask_test[2] = 259;
	

	
	if(arg >= t.n_arg) return ARG_MISSING;

	/* vede se il test  valido */
	if(!(t.mask_test[arg] & (1 << t_type)))
		return NOT_VALID;

	/* cambia gli argomenti base del test e genera l'email*/
	t.arg[0] = gen_mem(150, 2);
	t.arg[1] = gen_mem(15, 0);
	t.arg[2] = (void *) 15;
	t.mask_free = 6;
	t.motivo = "il caso base";

	/* cambia gli argomenti necessari */
	switch(t_type) {
		case RESULT_OK:
			break;
		case NULL_POINTER:
			change_arg(&t, arg, NULL);
			t.motivo = "il null pointer";
			break;
		case EMPTY_STRING:
			change_arg(&t, arg, "");
			t.motivo = "una stringa vuota";
			break;
		case BUFFER_TOO_SMALL:
			t.arg[arg] = (void *)-5;
			t.motivo = "dimensione negativa";
			result = check_res(&t, t_type);
			if(result == TEST_PASSED) {
				t.arg[arg] = (void *)1;
				t.motivo = "un buffer troppo piccolo";
				result = check_res(&t, t_type);
			}		
			break;
		default:
			result = NOT_IMPLEMENTED;
			
	}	
	
	if(result == -1) {
		result = check_res(&t, t_type);
	} /* result gi ritornato */

	if(result == TEST_FAILED) printfailed(&t, t_type);

	for(t.n_arg -= 1; t.n_arg > 0; t.n_arg -= 1)
		change_arg(&t, t.n_arg, NULL);

	return result;
}


/* test per tokenizzaEmail */
int test_tokenizzaEmail(int t_type, int arg) {
	test t;
	int result = -1;

	t.func = tokenizzaEmail;
	t.nomefun = "tokenizzaEmail";
	t.n_arg = 3;
	t.mask_free = 0;
	t.mask_test[0] = 7695;
	t.mask_test[1] = 7;
	t.mask_test[2] = 259;
	

	
	if(arg >= t.n_arg) return ARG_MISSING;

	/* vede se il test  valido */
	if(!(t.mask_test[arg] & (1 << t_type)))
		return NOT_VALID;

	/* cambia gli argomenti base del test e genera l'email*/
	t.arg[0] = gen_mem(400, 0);
	leggiFile("email1.txt" , 400, t.arg[0]);
	t.arg[1] = (Token *)gen_mem(1000 * sizeof(Token), 0);
	t.arg[2] = (void *) 1000;
	t.mask_free = 6;
	t.motivo = "il caso base";

	/* cambia gli argomenti necessari */
	switch(t_type) {
		case RESULT_OK:
			t.mask_free |= (1 << 7);
			break;
		case NULL_POINTER:
			change_arg(&t, arg, NULL);
			t.motivo = "il null pointer";
			break;
		case EMPTY_STRING:
			change_arg(&t, arg, "");
			t.motivo = "una stringa vuota";
			break;
		case BUFFER_TOO_SMALL:
			t.arg[arg] = (void *)-5;
			t.motivo = "dimensione negativa";
			result = check_res(&t, t_type);
			if(result == TEST_PASSED) {
				t.arg[arg] = (void *)1;
				t.motivo = "un buffer troppo piccolo";
				result = check_res(&t, t_type);
			}		
			break;
		case BODY_NOT_FOUND:
			change_arg(&t, arg, ML_STRHEAD);
			t.motivo = "il body inesisente";
			break;
		case HEADER_NOT_FOUND:
			change_arg(&t, arg, "\n\n");
			t.motivo = "l'header mancante";
			break;
		case BAD_HEADER:
			change_arg(&t, arg, "kaosdkoaks\n\n");
			break;
		default:
			result = NOT_IMPLEMENTED;
		
	}	
	
	if(result == -1) {
		result = check_res(&t, t_type);

	} /* result gi ritornato */

	if(result == TEST_FAILED) printfailed(&t, t_type);

	if(t.mask_free & (1 << 7)) free_tokens(t.arg[1]);

	for(t.n_arg -= 1; t.n_arg > 0; t.n_arg -= 1)
		change_arg(&t, t.n_arg, NULL);

	return result;
}



/* test per i tokens */
int test_printTokens(int t_type, int arg) {

	/*FILE *fptr = stdout; */
	test t;
	char *email;
	int result = -1;

	t.func = printTokens;
	t.nomefun = "printTokens";
	t.n_arg = 2;
	t.mask_free = 0;
	t.mask_test[0] = 15;
	t.mask_test[1] = 259;
	

	
	if(arg >= t.n_arg) return ARG_MISSING;

	/* vede se il test  valido */
	if(!(t.mask_test[arg] & (1 << t_type)))
		return NOT_VALID;

	/* cambia gli argomenti base del test e genera l'email*/
	t.arg[0] = (Token *)gen_mem(1000 * sizeof(Token), 0);
	t.arg[1] = (void *) 400;
	email = (char*)malloc(sizeof(char) * 400);
	leggiFile("email1.txt", 400, email);
	tokenizzaEmail(email, t.arg[0], 1000);
	free(email);
	t.mask_free = 130;
	t.motivo = "il caso base";

	freopen(".stdout", "a+", stdout);
	
	/* cambia gli argomenti necessari */
	switch(t_type) {
		case RESULT_OK:
			break;
		case NULL_POINTER:
			free_tokens(t.arg[0]);
			change_arg(&t, arg, NULL);
			t.mask_free &= ~(1 << 7);
			t.motivo = "il null pointer";
			break;
		case BUFFER_TOO_SMALL:
			t.arg[arg] = (void *)-5;
			t.motivo = "dimensione negativa";
			break;
		default:
			result = NOT_IMPLEMENTED;
			
	}	
	
	if(result == -1) {
		result = check_res(&t, t_type);
	} /*result gi ritornato */

	freopen("/dev/tty", "a+", stdout);
	
	remove(".stdout");

	if(result == TEST_FAILED) printfailed(&t, t_type);

	if(t.mask_free & (1 << 7)) free_tokens(t.arg[0]);

	for(t.n_arg -= 1; t.n_arg > 0; t.n_arg -= 1)
		change_arg(&t, t.n_arg, NULL);

	return result;
/*	return NOT_IMPLEMENTED; */
}




/* ******************** *
 * altre funzioni utili *
 * ******************** */

void free_tokens(Token *t) {
	int i = 0;
	while( t[i].parola ) {
		if(t[i].contesto)
			free(t[i].contesto);
		free(t[i].parola);
		i++;
	}
	
}

/* cambia l'argomento */
void change_arg(test *t, int n, void *data) {
	/* vede se  da fare la free */
	if(t->mask_free & (1 << (n + 1) ) ) {
		free(t->arg[n]);
		t->mask_free &= ~(1 << (n + 1));
	}
	t->arg[n] = data;
}


/* esegue la funzione */
int check_res(test *t, int t_type) {
	
	if(t->n_arg == 2)
		t->result = t->func(t->arg[0], t->arg[1]);
	else
		if(t->n_arg == 3)
			t->result = t->func(t->arg[0], t->arg[1], t->arg[2]);
		else
			t->result = t->func(t->arg[0], t->arg[1], t->arg[2], t->arg[3], t->arg[4]);

	if(t_type == t->result) return TEST_PASSED;
	else return TEST_FAILED;

}


/* stampa il test fallito */
void printfailed(test *t, int t_type) {
	printf("\nTest fallito:\n");
	printf("\tfunzione testata: %s\n", t->nomefun);
	printf("\trisultato atteso: %s\n", intocode(t_type));
	printf("\trisultato ottenuto: %s\n", intocode(t->result));
	printf("\tspiegazione: la funzione non gestisce correttamente\n\t\t%s\n", t->motivo);
}



/* genera un nome pseudo casuale */
/* ctrl = 0 -> area di memoria e basta;
   ctrl = 1 -> array casuale con lettere e punti (nomefile)
   ctrl > 1 -> array completamente casuale */
char *gen_mem(int size, int ctrl) {
	int i;
	char *name;
	
	if(!(name = (char *)malloc(sizeof(char) * size) ) )
		exit(1);

	/* genera una sequenza casuale di lettere */
	if(ctrl != 0) {
		srand(time(NULL));

		if(ctrl == 1) {
			name[0] = '.';
			for(i = 1; i < size - 1; i++)
				name[i] = 'a' + (rand() % 25);
		} else {
			for(i = 0; i < size - 1; i++)
				name[i] = 33 + (rand() % 125);
		}

		name[i] = '\0';
	}

	return name;
}


/* ritorna il codice d'errore */
char *intocode(int code) {
	char *a_result[14];
	a_result[0] = "RESULT_OK";
	a_result[1] = "UNKNOWN_ERROR";
	a_result[2] = "NULL_POINTER";
	a_result[3] = "EMPTY_STRING";
	a_result[4] = "FILE_MISSING";
	a_result[5] = "FILE_UNREADABLE";
	a_result[6] = "FILE_UNWRITABLE";
	a_result[7] = "FILE_EMPTY";
	a_result[8] = "BUFFER_TOO_SMALL";
	a_result[9] = "HEADER_NOT_FOUND";
	a_result[10] = "BODY_NOT_FOUND";
	a_result[11] = "BAD_HEADER";
	a_result[12] = "MEM_ALLOC_ERROR";
	a_result[13] = "falso positivo o errore sconosciuto";

	return a_result[code];
}



