#include "modulo1.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#define BUFSIZE 128

extern int errno;

int leggiFile( char *nomefile, int dimensione, char *testo ) {
	
	FILE *f_in;
	int n, file_dim;
	struct stat statbuf;
		
	// Controllo parametri.
	if ( nomefile == NULL || testo == NULL ) {
		return ( NULL_POINTER );
	}

	// Controllo se il nome del file  una stringa vuota.
	if ( nomefile[0] == '\0' ) {
		return ( EMPTY_STRING );
	}
		
	// Controllo se il file esiste.
	if ( stat( nomefile, &statbuf ) < 0 ) {
		if ( ENOENT == errno ) {
			return ( FILE_MISSING );
		}
		else {
			return ( UNKNOWN_ERROR );
		}
	}

	// Controllo della dimensione del blocco di memoria.
	file_dim = statbuf.st_size;
	if ( file_dim >= dimensione ) {
		return ( BUFFER_TOO_SMALL );		
	}
		
	// Controllo se il file  un file regolare
	if ( !S_ISREG( statbuf.st_mode ) ) {
		return ( FILE_UNREADABLE );
	}
	
	// Si prova ad aprire il file.
	if ( (f_in = fopen( nomefile, "r" )) == NULL ) {
		return ( FILE_UNREADABLE );
	}

	// Controllo se il file e' un file vuoto.
	if ( statbuf.st_size == 0 ) {
		fclose( f_in );
		return ( FILE_EMPTY );
	}
	
	// Controllo se il file ha i permessi in scrittura.
	if ( !(statbuf.st_mode & ( S_IWUSR|S_IWGRP|S_IWOTH )) ) {
		fclose( f_in );
		return ( FILE_UNWRITABLE ); 
	}

	// Si prova a copiare il contenuto in 'testo'.
	if ( (n = fread( (void *) testo, 1, file_dim, f_in )) < file_dim ) {
		fclose( f_in );
		return ( FILE_UNREADABLE );
	}
	
	// Chiusura del file.
	fclose( f_in );

	// Terminatore in 'testo'.
	testo[dimensione-1] = '\0';
	
	return ( RESULT_OK );
}


int estraiHeaders( char *email, char *headers, int dimh )
{		
	int i;
	char *p;
			
	// Controllo parametri.
	if ( email == NULL || headers == NULL ) {
		return ( NULL_POINTER );
	}

	// Controllo sulla presenza del testo.
	if ( strlen( email ) == 0 ) {
		return ( EMPTY_STRING );
	}
	
	// Conta i caratteri fina a che non trova la separazione
	// dagli headers e il corpo del messaggio.
	for ( i = 0; email[i] != '\0'; ++i ) {
		if ( email[i] == '\n' && email[i+1] == '\n' ) {
			// incremento "i" di 1 cosi' viene considerato anche 
			// un ritono accapo
			i++;
			break; // interrompo il ciclo
			
		}
	}
	// Al termine del blocco superiore il valore della variabile "i"
	// equivale esattamente al numero di caratteri da copiare nella
	// stringa "headers".

// TODO: Chiedere il tipo di errore se non c' la separazione.
	if ( email[i] == '\0' ) {
		return ( HEADER_NOT_FOUND );
	}

	// Controllo se la dimensione della porzione di memoria che
	// ho a disposizione e' sufficente per copiare al suo interno
	// gli headers.
	if ( i >= dimh ) {
		return ( BUFFER_TOO_SMALL );
	}

	// Copio gli headers!
	strncpy( headers, email, i );
	headers[i]='\0';

	// Distruggo la parte degli headers, incremento di 1
	// perche' salto i l' accapo che divide gli headers 
	// dal body.
	p = email + i + 1;
	for ( i = 0; p[i] != '\0'; ++i ) {
		email[i] = p[i];
	}
	email[i] = '\0';
		
	return ( RESULT_OK );
}

int primoHeader( char *headers, char *primo_header, int dim ) {
	
		
	// b  un flag usato per vedere la validit del header
	// b = 0, STRINGA VALIDA.
	// b = 1, STRINGA NON VALIDA.
	short b_flag = 0; // vedi sopra.
	short done = 0; //  un flag per determinare la fine del ciclo.
	int c_count = 0; //  un contatore di caratteri.
	int i;
	char *p;
	
	// Controllo dei parametri.
	if ( headers == NULL || primo_header == NULL ) {
		return ( NULL_POINTER );
	}

	// Controllo se headers  una stringa vuota.
	if ( headers[0] == '\0' ) {
		return ( EMPTY_STRING );
	}
	
	// Estraggo il primo headers.
	for ( i = 0; headers[i] != '\0'; ++i ) {
		if ( headers[i] == '\n' ) {
			// Se dopo il ritorno accapo non trovo uno spazio vuoto o
			// un tab. significa che comincia un nuovo header.
			if ( ( headers[i+1] == ' ' || headers[i+1] == '\t' ) ) {
				continue;
			}
			else {
				break;
			}
		}
	}

	// Controllo se ho abbastanza spazio per copiare
	// il primo header.
	if ( i >= dim ) {
		return ( BUFFER_TOO_SMALL );
	}

	// Copio il primo header.
	strncpy( primo_header, headers, i );
	primo_header[i] = '\0';

	if ( headers[i] != '\n' ) return BAD_HEADER;
	
	// Distruggo la parde dell'header che non mi 
	// serve piu'. (i+1 per scavalcare /n)
	p =  headers + i + 1;
	for ( i = 0; p[i] != '\0'; ++i ) {
		headers[i] = p[i];
	}
	headers[i] = '\0';


	// Controllo se la stringa inizia con un carattere alfanumerico
	if ( primo_header[0] == '\t' || primo_header[0] == ' ' || primo_header[0] == ':') {
		return ( BAD_HEADER );
	}
	
	// controllo il rimanente
	for ( i = 0; !done; ++i ) {
		
		// Se trovo uno spazio, imposto il flag in modo
		// da capire se gi ho trovato un'altro spazio.
		if ( primo_header[i] == ' ' || primo_header[i] == '\t' ) {
			b_flag = 1;
		}
		// QUI LA STORIA E' COMPLESSA:
		else if ( primo_header[i] == '\n' ) {
			if ( b_flag == 1 ) {
				b_flag = 0; // ad ogni nuova linea bisogna contare un nuovo spazio.
				c_count = 0; // ad ogni nuova linea azzero il numero di caratteri contati.
			}
			else {
				return ( BAD_HEADER );
			}
		}
		else if ( primo_header[i] == '\0' ) {
			if ( b_flag == 0 ) {
				return ( BAD_HEADER );
			}
			// Se non trovo caratteri "buoni sulla nuova linea c' 
			// qualcosa che non v.
			else if ( c_count == 0 ) {
				return ( BAD_HEADER );
			}
			else {
				done = 1;
			}
		}
		else {
			c_count++;
		}
	}

	return ( RESULT_OK );
}


///////////////////////
int parseHeader( char *header, char *tipo, int dimt, char *valore, int dimv ) {
	
	char *tmp_buf, *p;
	char *t, *v;
	int i;
			
	// Controllo dei parametri.
	if ( header == NULL || tipo == NULL || valore == NULL ) {
		return ( NULL_POINTER );
	}

	// Controllo se l'header  una stringa vuota.
	if ( header[0] == '\0' ) {
		return ( EMPTY_STRING );
	}

	// Crea una copia di header per estrarre i valori senza modificare
	// la stringa originale.
	tmp_buf = strdup( header );
	if ( tmp_buf == NULL ) {
		return ( MEM_ALLOC_ERROR );
	}

	// Posizioni i puntatori per la ricenrca del carattere vuoto.
	t = v = tmp_buf;
	
	// Estraggo il valore: alla fine di questo blocco v 
	// puntera' esattamente all'inizio del valore, mentre t
	// punta all'inizio del tipo, e tutte e due le stringhe 
	// hanno in carattere terminatore.
	for ( v = tmp_buf; *v != ' '; v++ );
	*(v) = '\0'; v=v+1;
	
	// Controlla la dimensione della stringa tipo. 
	if ( strlen( t ) >= dimt ) {
		return ( BUFFER_TOO_SMALL );
	}
	else {
		strcpy( tipo, tmp_buf ); 
	}
	
	// Controllo la dimensione della stringa valore.
	if ( strlen( v ) >= dimv ) {
		return ( BUFFER_TOO_SMALL );
	}
	else {
		strcpy( valore, v );
	}

	// Libero la memoria.
	free( tmp_buf );

	return ( RESULT_OK );
}

///////////////
int primaParola( char *testo, char *parola, int dim ) {

	short done = 0;
	int i, k;
	char *p;
	
	// Controllo parametri.
	if ( testo == NULL || parola == NULL ) { 
		return ( NULL_POINTER );
	}

	// Controllo stringa vuota.
	if ( testo[0] == '\0' ) { 
		return ( EMPTY_STRING );
	}

	// Controllo per non far impazzire il ciclo.
	if ( dim > 0 ) {
		parola[0] = '\0';
	}
	else
	{
		return ( BUFFER_TOO_SMALL );
	}
	

	// Ciclo per estrarre la parola.
	for ( i = 0, k = 0; testo[k] != '\0' && done == 0; ++k ) 
	{
		// controllo se  un carattere
		if( ( testo[k] >= 'a' && testo[k] <= 'z' ) ||
			( testo[k] >= 'A' && testo[k] <= 'Z' ) ||
			( testo[k] >= '0' && testo[k] <= '9' ) ) 
		{
			// Controllo se la dimensione di 'parola' e' sufficente.
			if ( i >= dim ) {
				return ( BUFFER_TOO_SMALL );
			}
				
			// Inserisce il carattere nel vettore 
			// e postincremento del contatore del 
			// carattere corrente.
			parola[i++] = testo[k];	
		}
		else if( parola[0] != '\0' ) {
			done = 1;
		}
	}

	// Controllo se la dimensione di testo e' sufficente.
	if ( i >= dim ) {
		return ( BUFFER_TOO_SMALL );
	}

	// Termino la parola.
	parola[i] = '\0';

	// distruggo la parte precedente del testo con tutte schifezze
	p = testo + k;
	for ( i = 0; p[i] != '\0'; ++i ) {
		testo[i] = p[i];
	}
	testo[i] = '\0';
	
		
	return ( RESULT_OK );
}


//////////////////
int tokenizzaEmail( char *email, Token tokenArray[], int dimT ) {
	
	char *buf; // puntatore al buffer
	int buf_size; // dimensione del buffer
	int errcode; // codice dell'errore
	char *tmpPtr; // puntatore per muovermi nella memoria
	char header_buf[BUFSIZE*2];
	char contesto_buf[BUFSIZE];
	char valori_buf[BUFSIZE];
	char parola_buf[BUFSIZE];
	int i = 0; // Deve essere inizializzato a zero!!!!!
	
	// Controllo parametri.
	if ( email == NULL || tokenArray == NULL ) {
		return ( NULL_POINTER );
	}
	if ( email[0] == '\0' ) {
		return ( EMPTY_STRING );
	}

	// mi calcolo la dimensione del mio buffer.
	buf_size = strlen( email ) + 1;

	// Per sicurezza mi alloco un buffer di dimensione di tutta l'email.
	buf = (char *)malloc( sizeof (char) * buf_size );
	if ( buf == NULL ) {
		return ( MEM_ALLOC_ERROR );
	}

	// Estraggo gli headers e li inserisco nel tokenArray.
	errcode = estraiHeaders( email, buf, buf_size );
	if ( errcode != RESULT_OK ) {
		return ( HEADER_NOT_FOUND );
	}
	
	// Estraggo gli headers fino a quando ce ne sono.
	while( buf[0] != '\0' ) {
		errcode = primoHeader( buf, header_buf, BUFSIZE*2 ); 
		if ( errcode == BAD_HEADER ) {
			return ( BAD_HEADER );
		}else if ( errcode != RESULT_OK ) {
			return ( UNKNOWN_ERROR );
		}
		
		// Estraggo il tipo e valore
		errcode = parseHeader( header_buf, contesto_buf, BUFSIZE, valori_buf, BUFSIZE );
		if ( errcode != RESULT_OK ) {
				return ( BAD_HEADER ); 
		}

		// Inserisco nell'array.
		for ( i = i; valori_buf[0] != '\0'; ++i ) {
			errcode = primaParola( valori_buf, parola_buf, BUFSIZE );
			if ( errcode != RESULT_OK ) {
				return ( UNKNOWN_ERROR );
			}
			
			tokenArray[i].contesto = strdup( contesto_buf );
			tokenArray[i].parola = strdup( parola_buf );
		}
	}
	// Libero la memoria.
	free( buf );

	// Controlla l'esistenza del body.
	if ( email[0] == '\0' ) {
		return ( BODY_NOT_FOUND );
	}
	
	// Tokenizza il body.
	for ( i = i; email[0] != '\0'; ++i ) {
		errcode = primaParola( email, parola_buf, BUFSIZE );
		if ( errcode != RESULT_OK ) {
			return ( UNKNOWN_ERROR );
		}

		if ( i >= dimT ) {
			return ( BUFFER_TOO_SMALL ); 
		} 
			
		tokenArray[i].contesto = NULL;
		tokenArray[i].parola = strdup( parola_buf );
	}

	// Imposta in tokenArray i valori successivi a "i" a 
	// NULL se esistono per bloccare il ciclo 
	if ( i < dimT ) {
		tokenArray[i].contesto = NULL;
		tokenArray[i].parola = NULL;
	}


	return ( RESULT_OK );
}

///////////////
int printTokens( Token tokenArray[], int dimT ) {
	
	int i; // contatore, indice, per contare i token.
	
	// Controllo parametri.	
	if ( tokenArray == NULL ) {
		return NULL_POINTER;
	}

	// Stampa i token, il ciclo si blocca quando trova parola = NULL & contesto = NULL
	for ( i = 0; i < dimT && (tokenArray[i].parola != NULL || tokenArray[i].contesto != NULL); ++i ) {
		
		if ( tokenArray[i].contesto != NULL ) {
			fprintf( stdout, "%s*", tokenArray[i].contesto );
		}

		if ( tokenArray[i].parola != NULL ) {
			fprintf( stdout, "%s\n", tokenArray[i].parola );
		}
	}
	
	return ( RESULT_OK );
}



