#include "testModulo1.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define BUFSIZE 1024
#define MAXTOKENS 100

void err_to_string( int errcode, char *str ) {
	switch ( errcode ) {
		case RESULT_OK:			sprintf( str, "RESULT_OK" ); break;
		case UNKNOWN_ERROR:		sprintf( str, "UNKNOWN_ERROR" ); break;
		case NULL_POINTER:		sprintf( str, "NULL_POINTER" ); break;
		case EMPTY_STRING:		sprintf( str, "EMPTY_STRING" ); break;
		case FILE_MISSING:		sprintf( str, "FILE_MISSING" ); break;
		case FILE_UNREADABLE:	sprintf( str, "FILE_UNREADABLE" ); break;
		case FILE_UNWRITABLE:	sprintf( str, "FILE_UNWRITABLE" ); break;
		case FILE_EMPTY:		sprintf( str, "FILE_EMPTY" ); break;
		case BUFFER_TOO_SMALL:	sprintf( str, "BUFFER_TOO_SMALL" ); break;
		case HEADER_NOT_FOUND:	sprintf( str, "HEADER_NOT_FOUND" ); break;
		case BODY_NOT_FOUND:	sprintf( str, "BODY_NOT_FOUND" ); break;
		case BAD_HEADER:		sprintf( str, "BAD_HEADER" ); break;
		case MEM_ALLOC_ERROR:	sprintf( str, "MEM_ALLOC_ERROR" ); break;
		default:break;
	};
}


int test_leggiFile( int tipo_di_test, int argomento ) {
	
	char test_file[] = "test.t";
	char buf[BUFSIZE];
	int errcode; // valore di ritorno ottenuto
	int code; // valore di ritorno atteso
	int fd;
		
	if ( tipo_di_test < RESULT_OK || tipo_di_test > MEM_ALLOC_ERROR ) {
		return NOT_IMPLEMENTED;
	}
	else if ( tipo_di_test == RESULT_OK ) { // RESULT_OK
		if ( argomento != 0 ) {
			return NOT_IMPLEMENTED;
		}
		code = RESULT_OK; // valore apettato
		errcode = leggiFile( "email1.txt", BUFSIZE, buf );
		sprintf( buf, "(\"email1.txt\", BUFSIZE, buf)" ); 
	}
	else if ( tipo_di_test == NULL_POINTER ) { // NULL_POINTER
		code = NULL_POINTER; // valore aspettato
		switch ( argomento ) {
			case 0: 
				errcode = leggiFile( NULL, BUFSIZE, buf );
				sprintf( buf, "(NULL, BUFSIZE, buf)" );
				break;
			case 1:
				errcode = leggiFile( "email1.txt", BUFSIZE, NULL );
				sprintf( buf, "(\"email1.txt\", BUFSIZE, NULL)" );
				break;
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == EMPTY_STRING ) {
		code = EMPTY_STRING; 
		switch ( argomento ) {
			case 0:
				errcode = leggiFile( "", BUFSIZE, buf );
				sprintf( buf, "(\"\", BUFSIZE, buf)" );
				break;
			case 1:
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == FILE_MISSING ) { // FILE_MISSING
		code = FILE_MISSING; 
		switch ( argomento ) {
			case 0:
				errcode = leggiFile( "@@@__", BUFSIZE, buf );
				sprintf( buf, "(\"@@@__\", BUFSIZE, buf)" );
				break;
			case 1:
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == FILE_UNREADABLE ) { // FILE_UNREADABLE
		code = FILE_UNREADABLE;
		switch ( argomento ) {
			case 0:
				// creo un file di test 
				if ( fd = open( test_file, O_CREAT | O_TRUNC | O_WRONLY, 0000 ) < 0 ) {
					return NOT_IMPLEMENTED;		
				}
				close( fd );
				errcode = leggiFile( test_file, BUFSIZE, buf );
				sprintf( buf, "(test_file, BUFSIZE, buf)" );
				unlink( test_file );
				break;
			case 1:
			case 2: return NOT_IMPLEMENTED;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == FILE_EMPTY ) {
		code = FILE_EMPTY; 
		switch ( argomento ) {
			case 0:
				// creo un file di test 
				if ( fd = open( test_file, O_CREAT | O_TRUNC | O_WRONLY, 0777 ) < 0 ) {
					return NOT_IMPLEMENTED;		
				}
				close( fd );
				errcode = leggiFile( test_file, BUFSIZE, buf );
				sprintf( buf, "(test_file, BUFSIZE, buf)" );
				unlink( test_file );
				break;
			case 1:
			case 2: return NOT_IMPLEMENTED;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == BUFFER_TOO_SMALL ) {
		code = BUFFER_TOO_SMALL;
			switch ( argomento ) {
			case 0:
				// creo un file di test 
				if ( fd = open( test_file, O_CREAT | O_TRUNC | O_WRONLY, 0777 ) < 0 ) {
					return NOT_IMPLEMENTED;		
				}
				write( fd, test_file, strlen( test_file )+1 );
				close( fd );
				errcode = leggiFile( test_file, 3, buf );
				sprintf( buf, "(test_file, 3, buf)" );
				unlink( test_file );
				break;
			case 1:
			case 2: return NOT_IMPLEMENTED;
			default: return ARG_MISSING;
		};
	}
	else return NOT_IMPLEMENTED;

	// se il risultato della funzione e' uguale al risultato
	// atteso, il test viene passato!
	if ( code == errcode ) {
		return TEST_PASSED;
	}
	
	// altrimenti fallisce!
	printf( "Test fallito: "\
			"\tfunzione testata: leggiFile%s\n", buf );
	
	err_to_string( code, buf );
	printf( "\t\trisultato atteso: %s\n", buf );

	err_to_string( errcode, buf );
	printf(	"\t\trisultato ottenuto: %s\n", buf);
	
	return ( TEST_FAILED );
}

////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
int test_estraiHeaders(int tipo_di_test, int argomento ) {
	
	char test_email[] = "From babbo.natale@northpole.org\n"\
						"To: tutti.i.bambini@cattivi.net\n"\
						"Subject: Niente regali\n\n"\
						"Se siete cattivi niente regali!!";
	char buf[BUFSIZE];
	int errcode; // valore di ritorno ottenuto
	int code; // valore di ritorno atteso
	int fd;
		
	if ( tipo_di_test < RESULT_OK || tipo_di_test > MEM_ALLOC_ERROR ) {
		return NOT_IMPLEMENTED;
	}
	else if ( tipo_di_test == RESULT_OK ) { // RESULT_OK
		if ( argomento != 0 ) {
			return NOT_IMPLEMENTED;
		}
		code = RESULT_OK; // valore apettato
		errcode = estraiHeaders( test_email, buf, BUFSIZE );
		sprintf( buf, "(test_email, buf, BUFSIZE)" ); 
	}
	else if ( tipo_di_test == NULL_POINTER ) { // NULL_POINTER
		code = NULL_POINTER; // valore aspettato
		switch ( argomento ) {
			case 0: 
				errcode = estraiHeaders( NULL, buf, BUFSIZE );
				sprintf( buf, "(NULL, BUFSIZE, buf)" );
				break;
			case 1:
				errcode = estraiHeaders( test_email, NULL, BUFSIZE );
				sprintf( buf, "(test_email, NULL, BUFSIZE)" );
				break;
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == EMPTY_STRING ) { // EMPTY_STRING
		code = EMPTY_STRING;
		test_email[0] = '\0';
		switch ( argomento ) {
			case 0: 
				errcode = estraiHeaders( "", buf, BUFSIZE );
				sprintf( buf, "(\"\", buf, BUFSIZE)" );
				break;
			case 1:
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == HEADER_NOT_FOUND ) {
		code = HEADER_NOT_FOUND;
		switch ( argomento ) {
			case 0: 
				errcode = estraiHeaders( "Questo e' solo un body.\n", buf, BUFSIZE );
				sprintf( buf, "(\"Questo e' solo un body.\\n\", buf, BUFSIZE)" );
				break;
			case 1:
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == BUFFER_TOO_SMALL ) {
		code = BUFFER_TOO_SMALL;
		switch ( argomento ) {
			case 2:
				errcode = estraiHeaders( test_email, buf, 2 );
				sprintf( buf, "(test_email, buf, 2)" );
				break;
			case 0:
			case 1: return NOT_VALID;
			default: return ARG_MISSING;
		};	
	}
	else return NOT_IMPLEMENTED;

	
	// se il risultato della funzione e' uguale al risultato
	// atteso, il test viene passato!
	if ( code == errcode ) {
		return TEST_PASSED;
	}
	
	// altrimenti fallisce!
	printf( "Test fallito: "\
			"\tfunzione testata: estraiHeaders%s\n", buf );
	
	err_to_string( code, buf );
	printf( "\t\trisultato atteso: %s\n", buf );

	err_to_string( errcode, buf );
	printf(	"\t\trisultato ottenuto: %s\n", buf);
	
	return ( TEST_FAILED );
}

/////////////////////////////////////////////////////
/////////////////////////////////////////////////////
int test_primoHeader(int tipo_di_test, int argomento ) {
	
	char test_headers[] = "From babbo.natale@northpole.org\n"\
						"To: tutti.i.bambini@cattivi.net\n"\
						"Subject: Niente regali\n";
	char buf[BUFSIZE];
	int errcode; // valore di ritorno ottenuto
	int code; // valore di ritorno atteso
	int fd;
		
	if ( tipo_di_test < RESULT_OK || tipo_di_test > MEM_ALLOC_ERROR ) {
		return NOT_IMPLEMENTED;
	}
	else if ( tipo_di_test == RESULT_OK ) { // RESULT_OK
		if ( argomento != 0 ) {
			return NOT_IMPLEMENTED;
		}
		code = RESULT_OK; // valore apettato
		errcode = primoHeader( test_headers, buf, BUFSIZE );
		sprintf( buf, "(test_headers, buf, BUFSIZE)" ); 
	}
	else if ( tipo_di_test == NULL_POINTER ) { // NULL_POINTER
		code = NULL_POINTER; // valore aspettato
		switch ( argomento ) {
			case 0: 
				errcode = primoHeader( NULL, buf, BUFSIZE );
				sprintf( buf, "(NULL, BUFSIZE, buf)" );
				break;
			case 1:
				errcode = primoHeader( test_headers, NULL, BUFSIZE );
				sprintf( buf, "(test_headers, NULL, BUFSIZE)" );
				break;
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == EMPTY_STRING ) { // EMPTY_STRING
		code = EMPTY_STRING;
		test_headers[0] = '\0';
		switch ( argomento ) {
			case 0: 
				errcode = primoHeader( "", buf, BUFSIZE );
				sprintf( buf, "(\"\", buf, BUFSIZE)" );
				break;
			case 1:
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == BUFFER_TOO_SMALL ) {
		code = BUFFER_TOO_SMALL;
		switch ( argomento ) {
			case 2:
				errcode = primoHeader( test_headers, buf, 2 );
				sprintf( buf, "(test_headers, buf, 2)" );
				break;
			case 0:
			case 1: return NOT_VALID;
			default: return ARG_MISSING;
		};	
	}
	else if ( tipo_di_test == BAD_HEADER ) {
		code = BAD_HEADER;
		switch ( argomento ) {
			case 0:
				sprintf( test_headers, "from: prova@email.org127.0.0.1" );
				errcode = primoHeader( test_headers, buf, BUFSIZE );
				sprintf( buf, "(test_headers, buf, BUFSIZE)" );
				break;
			case 1:
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};	
	}
	else return NOT_IMPLEMENTED;

	
	// se il risultato della funzione e' uguale al risultato
	// atteso, il test viene passato!
	if ( code == errcode ) {
		return TEST_PASSED;
	}
	
	// altrimenti fallisce!
	printf( "Test fallito: "\
			"\tfunzione testata: primoHeader%s\n", buf );
	
	err_to_string( code, buf );
	printf( "\t\trisultato atteso: %s\n", buf );

	err_to_string( errcode, buf );
	printf(	"\t\trisultato ottenuto: %s\n", buf);
	
	return ( TEST_FAILED );
}

//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
int test_parseHeader(int tipo_di_test, int argomento ) {

	char test_header[] = "From babbo.natale@northpole.org\n";
	char buf[BUFSIZE];
	int errcode; // valore di ritorno ottenuto
	int code; // valore di ritorno atteso
		
	if ( tipo_di_test < RESULT_OK || tipo_di_test > MEM_ALLOC_ERROR ) {
		return NOT_IMPLEMENTED;
	}
	else if ( tipo_di_test == RESULT_OK ) { // RESULT_OK
		if ( argomento != 0 ) {
			return NOT_IMPLEMENTED;
		}
		code = RESULT_OK; // valore apettato
		errcode = parseHeader( test_header, buf, BUFSIZE, buf, BUFSIZE );
		sprintf( buf, "(test_header, buf, BUFSIZE, buf, BUFSIZE)" ); 
	}
	else if ( tipo_di_test == NULL_POINTER ) { // NULL_POINTER
		code = NULL_POINTER; // valore aspettato
		switch ( argomento ) {
			case 0: 
				errcode = parseHeader( NULL, buf, BUFSIZE, buf, BUFSIZE );
				sprintf( buf, "(NULL, buf, BUFSIZE, buf, BUFSIZE )" );
				break;
			case 1:
				errcode = parseHeader( test_header, NULL, BUFSIZE, buf, BUFSIZE );
				sprintf( buf, "(test_header, NULL, BUFSIZE, buf, BUFSIZE)" );
				break;
			case 4:
			case 2: return NOT_VALID;
			case 3:
				errcode = parseHeader( test_header, buf, BUFSIZE, NULL, BUFSIZE );
				sprintf( buf, "(test_header, buf, BUFSIZE, NULL, BUFSIZE)" );
				break;
			
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == EMPTY_STRING ) { // EMPTY_STRING
		code = EMPTY_STRING;
		test_header[0] = '\0';
		switch ( argomento ) {
			case 0: 
				errcode = parseHeader( "", buf, BUFSIZE, buf, BUFSIZE );
				sprintf( buf, "(\"\", buf, BUFSIZE, buf, BUFSIZE)" );
				break;
			case 1:
			case 2:
			case 3:
			case 4: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == BUFFER_TOO_SMALL ) {
		code = BUFFER_TOO_SMALL;
		switch ( argomento ) {
			case 2:
				errcode = parseHeader( test_header, buf, 2, buf, BUFSIZE );
				sprintf( buf, "(test_header, buf, 2, buf, BUFSIZE)" );
				break;
			case 4:
				errcode = parseHeader( test_header, buf, BUFSIZE, buf, 2 );
				sprintf( buf, "(test_header, buf, BUFSIZE, buf, 2 )" );
				break;
			case 0:
			case 1:
			case 3: return NOT_VALID;
			default: return ARG_MISSING;
		};	
	}
	else return NOT_IMPLEMENTED;

	
	// se il risultato della funzione e' uguale al risultato
	// atteso, il test viene passato!
	if ( code == errcode ) {
		return TEST_PASSED;
	}
	
	// altrimenti fallisce!
	printf( "Test fallito: "\
			"\tfunzione testata: parseHeader%s\n", buf );
	
	err_to_string( code, buf );
	printf( "\t\trisultato atteso: %s\n", buf );

	err_to_string( errcode, buf );
	printf(	"\t\trisultato ottenuto: %s\n", buf);
	
	return ( TEST_FAILED );
}

/////////////////////////////////////////////////////
/////////////////////////////////////////////////////
int test_primaParola(int tipo_di_test, int argomento ) {
	char test_string[] = "parola parola parola";
	char buf[BUFSIZE];
	int errcode; // valore di ritorno ottenuto
	int code; // valore di ritorno atteso
		
	if ( tipo_di_test < RESULT_OK || tipo_di_test > MEM_ALLOC_ERROR ) {
		return NOT_IMPLEMENTED;
	}
	else if ( tipo_di_test == RESULT_OK ) { // RESULT_OK
		if ( argomento != 0 ) {
			return NOT_IMPLEMENTED;
		}
		code = RESULT_OK; // valore apettato
		errcode = primaParola( test_string, buf, BUFSIZE );
		sprintf( buf, "(test_string, buf, BUFSIZE)" ); 
	}
	else if ( tipo_di_test == NULL_POINTER ) { // NULL_POINTER
		code = NULL_POINTER; // valore aspettato
		switch ( argomento ) {
			case 0: 
				errcode = primaParola( NULL, buf, BUFSIZE );
				sprintf( buf, "(NULL, buf, BUFSIZE)" );
				break;
			case 1:
				errcode = primaParola( test_string, NULL, BUFSIZE );
				sprintf( buf, "(test_string, NULL, BUFSIZE)" );
				break;
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == EMPTY_STRING ) { // EMPTY_STRING
		code = EMPTY_STRING;
		test_string[0] = '\0';
		switch ( argomento ) {
			case 0: 
				errcode = primaParola( "", buf, BUFSIZE );
				sprintf( buf, "(\"\", buf, BUFSIZE)" );
				break;
			case 1:
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == BUFFER_TOO_SMALL ) {
		code = BUFFER_TOO_SMALL;
		switch ( argomento ) {
			case 2:
				errcode = primaParola( test_string, buf, 2 );
				sprintf( buf, "(test_string, buf, 2)" );
				break;
			case 0:
			case 1: return NOT_VALID;
			default: return ARG_MISSING;
		};	
	}
	else return NOT_IMPLEMENTED;

	
	// se il risultato della funzione e' uguale al risultato
	// atteso, il test viene passato!
	if ( code == errcode ) {
		return TEST_PASSED;
	}
	
	// altrimenti fallisce!
	printf( "Test fallito: "\
			"\tfunzione testata: primaParola%s\n", buf );
	
	err_to_string( code, buf );
	printf( "\t\trisultato atteso: %s\n", buf );

	err_to_string( errcode, buf );
	printf(	"\t\trisultato ottenuto: %s\n", buf);
	
	return ( TEST_FAILED );
}

int test_tokenizzaEmail(int tipo_di_test, int argomento ) {
	
	char test_email[] = "From babbo.natale@northpole.org\n"\
						"To: tutti.i.bambini@cattivi.net\n"\
						"Subject: Niente regali\n\n"\
						"Se siete cattivi niente regali!!";
	Token T[MAXTOKENS];
	char buf[BUFSIZE];
	int errcode; // valore di ritorno ottenuto
	int code; // valore di ritorno atteso
		
	if ( tipo_di_test < RESULT_OK || tipo_di_test > MEM_ALLOC_ERROR ) {
		return NOT_IMPLEMENTED;
	}
	else if ( tipo_di_test == RESULT_OK ) { // RESULT_OK
		if ( argomento != 0 ) {
			return NOT_IMPLEMENTED;
		}
		code = RESULT_OK; // valore apettato
		errcode = tokenizzaEmail( test_email, T, MAXTOKENS );
		sprintf( buf, "(test_email, T, MAXTOKENS)" ); 
	}
	else if ( tipo_di_test == NULL_POINTER ) { // NULL_POINTER
		code = NULL_POINTER; // valore aspettato
		switch ( argomento ) {
			case 0: 
				errcode = tokenizzaEmail( NULL, T, MAXTOKENS );
				sprintf( buf, "(NULL, T, MAXTOKENS)" );
				break;
			case 1:
				errcode = tokenizzaEmail( test_email, NULL, MAXTOKENS );
				sprintf( buf, "(test_string, NULL, MAXTOKENS)" );
				break;
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == EMPTY_STRING ) { // EMPTY_STRING
		code = EMPTY_STRING;
		test_email[0] = '\0';
		switch ( argomento ) {
			case 0: 
				errcode = tokenizzaEmail( "", T, MAXTOKENS );
				sprintf( buf, "(\"\", T, MAXTOKENS)" );
				break;
			case 1:
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};
	}
	else if ( tipo_di_test == BUFFER_TOO_SMALL ) {
		code = BUFFER_TOO_SMALL;
		switch ( argomento ) {
			case 2:
				errcode = tokenizzaEmail( test_email, T, 2 );
				sprintf( buf, "(test_email, T, 2)" );
				break;
			case 0:
			case 1: return NOT_VALID;
			default: return ARG_MISSING;
		};	
	}
	else if ( tipo_di_test == HEADER_NOT_FOUND ) { // HEADER_NOT_FOUND
		code = HEADER_NOT_FOUND;
		switch ( argomento ) {
			case 0:
				sprintf( test_email, "@@@@@corpo del messaggio!@@@@" );
				errcode = tokenizzaEmail( test_email, T, MAXTOKENS );
				sprintf( buf, "(test_email, T, MAXTOKENS)" );
				break;
			case 1:
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};	
	}
	else if ( tipo_di_test == BODY_NOT_FOUND ) { // BODY_NOT_FOUND
		code = BODY_NOT_FOUND;
		switch ( argomento ) {
			case 0:
				sprintf( test_email, "From: babbo.natale@northpole.org\n\n" );
				errcode = tokenizzaEmail( test_email, T, MAXTOKENS );
				sprintf( buf, "(test_email, T, MAXTOKENS)" );
				break;
			case 1:
			case 2: return NOT_VALID;
			default: return ARG_MISSING;
		};	
	}
	else return NOT_IMPLEMENTED;

	
	// se il risultato della funzione e' uguale al risultato
	// atteso, il test viene passato!
	if ( code == errcode ) {
		return TEST_PASSED;
	}
	
	// altrimenti fallisce!
	printf( "Test fallito: "\
			"\tfunzione testata: tokenizzaEmail%s\n", buf );
	
	err_to_string( code, buf );
	printf( "\t\trisultato atteso: %s\n", buf );

	err_to_string( errcode, buf );
	printf(	"\t\trisultato ottenuto: %s\n", buf);
	
	return ( TEST_FAILED );
}

int test_printTokens(int tipo_di_test, int argomento ) {
	return ( NOT_IMPLEMENTED );
}



