/* MODULO1.C FUNZIONI DI BASE - by Massimiliano Natale Matricola 697236 *************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "modulo1.h"



int leggiFile(char *nomefile, int dimensione, char *testo) { // Legge dimensione caratteri dal file e li copia su testo
  int i; 
  FILE *fp;

  if(nomefile==NULL || testo==NULL) // Controlla se uno degli argomenti  NULL
    return NULL_POINTER;

  if(strcmp(nomefile, "")==0) // Controlla se la stringa nomefile  vuota
    return EMPTY_STRING;

  // Inizializza tutti gli elementi del buffer testo a '\0'
  for(i=0; i<dimensione; i++) 
    testo[i] = '\0';

  if(access(nomefile, F_OK)==-1) // Se il file non esiste
    return FILE_MISSING;

  if(access(nomefile, R_OK)==-1) // Se il file non pu essere letto
    return FILE_UNREADABLE; 

  if((fp=fopen(nomefile, "r"))==NULL) // Si  verificato un errore inatteso
    return UNKNOWN_ERROR;

  if(dimensione<=0) // Dimensione del buffer non positiva
    return BUFFER_TOO_SMALL;

  if(fread(testo, sizeof(char), dimensione-1, fp)==0) // Se non hai copiato nessun carattere
    return FILE_EMPTY;
  
  return RESULT_OK; // La funzione  andata a termine senza errori
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int estraiHeaders(char *email, char *headers, int dimh) { // Estrae gli headers della mail e li copia nel buffer headers
  int i, letti=0;

  if(email==NULL || headers==NULL) // Uno degli argomenti  il puntatore NULL
    return NULL_POINTER;

  if(strcmp(email, "")==0) // L'argomento  la stringa vuota
    return EMPTY_STRING;

  if(dimh<=0) // Dimensione non positiva
    return BUFFER_TOO_SMALL;

  if(email[0]=='\n') // Individuato nel primo carattere il carattere separatore
    return HEADER_NOT_FOUND;

  if(email[0]==' ' || email[0]=='\t') // L'email inizia con spazio o tab
    return BAD_HEADER;
  

  // Inizializza tutti gli elementi del buffer testo a '\0'
  for(i=0; i<dimh; i++)
    headers[i] = '\0';  

  i=0;
  while(1) {
    if(email[i]=='\n') // Hai individuato la linea vuota di separazione dal testo dell'email
      break;
  
    for( ; email[i]!='\n'; i++, letti++) { // Copia singolarmente gli headers escluso il carattere '\n'
      if(letti>=dimh-1) 
	return BUFFER_TOO_SMALL; // Il buffer headers non  sufficiente a contenere i dati (considera '\0' ultimo char)
      headers[letti] = email[i]; // Copia carattere relativo all'header
    }
    if(letti>=dimh-1)
      return BUFFER_TOO_SMALL;
    headers[letti++] = email[i++]; // Copia '\n' e incrementa gli indici
  }
  
  if(email[i+1]==EOF || email[i+1]=='\0') // Se il carattere successivo al '\n' separatore  EOF
    return BODY_NOT_FOUND;

  // Elimina tutti i caratteri elaborati
  for( ; i>=0; i--) 
    email[i] = ' ';

  return RESULT_OK; // Hai copiato tutti gli headers senza superare la dimensione del buffer
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int primoHeader(char *headers, char *primo_header, int dim) { // Copia il primo header della mail nel buffer primo_header
  int i, letti=0, good_format=0;


  if(headers==NULL || primo_header==NULL) // Un argomento  NULL
    return NULL_POINTER;

  if(strcmp(headers, "")==0) // Il primo argomento  la stringa vuota
    return EMPTY_STRING;

  if(dim<=0) // Dimensione non positiva
    return BUFFER_TOO_SMALL;

  // Inizializza tutti gli elementi del buffer primo_header a '\0'
  for(i=0; i<dim; i++)
    primo_header[i] = '\0';  

  if(headers[0]=='\n')
    return HEADER_NOT_FOUND;

  if(headers[0]=='\t')
    return BAD_HEADER;

  // Escludi caratteri non significativi iniziali nel buffer email
  i=0;
  while(isblank(headers[i]) || headers[i]=='\0' || headers[i]==EOF) {
    if(headers[i]=='\0' || headers[i]==EOF || headers[i]=='\n') // Sei arrivato alla fine degli headers senza trovarne uno
      return HEADER_NOT_FOUND;
    i++;
  }

  // Ti trovi posizionato al primo carattere alfanumerico degli headers
  while(headers[i]!=' ' && headers[i]!='\t') { // Fino a quando non trovi lo spazio (sicuramente presente in un header)
    if(headers[i]==EOF && headers[i]=='\0') // Se arrivi alla fine
      return BAD_HEADER;
    else if(headers[i]=='\n' && (headers[i+1]=='\t' || headers[i+1]==' ')) { // L'header continua su pi righe 
      ; // Lascia che vengano copiati tutti i caratteri bianchi dell'header multiplo
    }
    else if(headers[i]=='\n' && (headers[i+1]!='\t' && headers[i+1]!=' ')) // L'header non ha lo spazio e non continua su pi righe
      return BAD_HEADER;

    if(letti>=dim-1) // Il buffer non  sufficiente a contenere il primo header (considera '\0' ultimo char)
      return BUFFER_TOO_SMALL;

    primo_header[letti++] = headers[i++]; // Copia il carattere dell'header (incluso lo spazio) ed incrementa i due contatori

    // Copia il carattere spazio prima di uscire dal ciclo
    if(headers[i]==' ') { 
      if(letti>=dim-1)
	return BUFFER_TOO_SMALL;
      primo_header[letti++] = headers[i];
    }
  }

  i++; // Salta il carattere spazio gi copiato

  // Fino a quando non arrivi a fine riga e la riga successiva non inizia con un tab o un carattere spazio
  while(headers[i]!='\n' || (headers[i+1]==' ' || headers[i+1]=='\t')) {
    if(letti>=dim-1) // Il buffer non  sufficiente a contenere il primo header (considera '\0' ultimo char)
      return BUFFER_TOO_SMALL;
    primo_header[letti++] = headers[i++]; // Copia il carattere dell'header ed incrementa i due contatori
    good_format++;
  }

  if(good_format<=0) // Non hai copiato nessun carattere come valore dell'header
    return BAD_HEADER;

  for( ; i>=0; i--) // Elimina le informazioni elaborate
    headers[i] = ' ';

  return RESULT_OK; // Hai copiato il primo header senza superare la dimensione del buffer
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int parseHeader(char *header, char *tipo, int dimt, char *valore, int dimv) { // Spezza un header in 2 parti: TIPO - VALORE
  int i, letti=0;

  if(header==NULL || tipo==NULL || valore==NULL) // Uno dei 3 buffer punta a NULL
    return NULL_POINTER;

  if((strcmp(header, "")==0)) // Il primo parametro  la stringa vuota
    return EMPTY_STRING;

  // Inizializza i due array in cui copiare i valori a '\0'
  for(i=0; i<dimt; i++)
    tipo[i] = '\0';
  for(i=0; i<dimv; i++)
    valore[i] = '\0';

  // Escludi eventuali caratteri non significativi
  i=0;
  while(isblank(header[i])) { // Trova il primo carattere
    if(header[i]=='\0') // Sei arrivato alla fine 
      return BAD_HEADER;
    i++;
  }

  // Hai trovato il primo carattere
  while(header[i]!=' ' && header[i]!='\t') { // Copia la prima parte dell'header in tipo trovando il carattere separatore
    if(header[i]=='\0')
      return BAD_HEADER; // L'header non contiene lo spazio
    if(letti>=dimt-1) // Hai superato la capienza del buffer (considera '\0' ultimo char)
      return BUFFER_TOO_SMALL;
    tipo[letti++] = header[i++]; // Copia il carattere ed incrementa i due indici
  }

  // Sei dopo lo spazio dell'header ed hai copiato il tipo

  letti=0; // Azzera l'indice per la copia nel secondo buffer

  i++; // Salta il carattere vuoto su cui ora ti trovi

  if(header[i]=='\0') // L'header non contiene il valore ma solo il tipo
    return BAD_HEADER;

  while(header[i]!='\0') { // Copia fino a quando non trovi l'ultimo carattere
    if(letti>=dimv-1) // Hai superato la capienza del buffer (considera '\0' ultimo char)
      return BUFFER_TOO_SMALL;
    valore[letti++] = header[i++]; // Copia il carattere ed incrementa i due indici
  }

  return RESULT_OK; // Hai spezzato l'header senza superare la capienza dei due buffer
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int primaParola(char *testo, char *parola, int dim) {  // Estrae dal testo la prima parola
  int i, letti=0;

  if(testo==NULL || parola==NULL)
    return NULL_POINTER;

  if(strcmp(testo, "")==0)
    return EMPTY_STRING;

  if(dim<=0)
    return BUFFER_TOO_SMALL;

  // Inizializza il buffer a '\0'
  for(i=0; i<dim; i++)
    parola[i] = '\0';

  // Trova nel testo il primo carattere alfanumerico
  i=0;
  while(!isalnum(testo[i])) {
    if(testo[i]=='\0' || testo[i]==EOF) // Non esiste nessuna parola
      return UNKNOWN_ERROR;
    i++;
  }

  // Copia tutta la sequenza di caratteri alfanumerici
  while(isalnum(testo[i]) || letti<1) { // Devi copiare almeno due caratteri alfanumerici
    if(testo[i]=='\0' || testo[i]==EOF) // Non esiste nessuna parola
      return UNKNOWN_ERROR;

    if(!isalnum(testo[i])) { // Non hai ancora letto 2 caratteri alfanumerici
      letti=0;
      i++;
      continue;
    }
    
    if(letti>=dim-1) // Hai superato la capienza del buffer (considera '\0' come ultimo carattere)
      return BUFFER_TOO_SMALL;
    parola[letti++] = testo[i]; // Copia il carattere ed incrementa l'indice
    testo[i++] = ' '; // Elimina il carattere letto ed incrementa l'indice
  }  

  return RESULT_OK; // Hai copiato la prima parola senza superare la capienza del buffer
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int tokenizzaEmail(char *email, Token tokenArray[], int dimT) { // Copia i token e li inserisce nel vettore tokenArray
  int ritorno, i=0, dim=5000, estratte;
  char headers[dim], primo_header[dim], parola[dim], contx[dim], parl[dim], temp[dim];


  if((ritorno = estraiHeaders(email, headers, dim))!=RESULT_OK)
    return ritorno;

  if(email==NULL || tokenArray==NULL) // Null ad un parametro
    return NULL_POINTER;

  if(strcmp(email, "")==0) // Primo argomento  ""
    return EMPTY_STRING;

  if(dimT <= 0) // Buffer troppo piccolo
    return BUFFER_TOO_SMALL;

  // Alloca memoria per i campi della struttura token
  for(i=0; i<dimT; i++) {  
    if(((tokenArray[i]).parola = (char *)malloc(dim * sizeof(char)))==NULL)
      return MEM_ALLOC_ERROR;
    if(((tokenArray[i]).contesto = (char *)malloc(dim * sizeof(char)))==NULL)
      return MEM_ALLOC_ERROR;
  }



  i=0;
  while((ritorno = primoHeader(headers, primo_header, dim))!=HEADER_NOT_FOUND) { // Estrai singolarmente gli header
    if(ritorno==BAD_HEADER) // Hai trovato un header errato
      return ritorno;

    if((ritorno = parseHeader(primo_header, contx, dim, parl, dim))!=RESULT_OK) // Dividi gli headers
      return ritorno;

    while(primaParola(parl, temp, dim)!=UNKNOWN_ERROR) {
      if(i>=dimT) // Token sufficientemente grandi per l'array
	return BUFFER_TOO_SMALL;     
     

      strcpy((tokenArray[i]).parola, temp); // Copia la singola parola relativa all'header
      strcpy((tokenArray[i]).contesto, contx); // Copia il tipo relativo alla parola
      i++;
    }
  }

  if(i==0) // Non hai trovato nessun header
    return HEADER_NOT_FOUND; 

  // Estrazione delle singole parole che non sono header
  estratte = 0; // Contatore delle parole estratte   
  while((ritorno = primaParola(email, parola, dim))!=UNKNOWN_ERROR) { // Fino a quando ci sono parole
    if(i>=dimT-1) // Numero di token sufficientemente grande per il vettore
      return BUFFER_TOO_SMALL;

    // Alloca memoria per i campi della struttura token
    if(((tokenArray[i]).parola = (char *)malloc(dim * sizeof(char)))==NULL)
      return MEM_ALLOC_ERROR;
    if(((tokenArray[i]).contesto = (char *)malloc(dim * sizeof(char)))==NULL)
      return MEM_ALLOC_ERROR;

    strcpy((tokenArray[i]).contesto, "NULL"); // Il tipo  NULL;
    strcpy((tokenArray[i]).parola, parola); // Copia la parola nel vettore di token
    i++;
    estratte++;
  }

  if(estratte==0) // Non hai estratto nessuna parola
    return UNKNOWN_ERROR;

  // Delimita la fine dei token significativi dell'array
  strcpy((tokenArray[i]).contesto, ""); 
  strcpy((tokenArray[i]).parola, "");

  return RESULT_OK; // Funzione andata a buon fine
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

 
 int printTokens(Token tokenArray[], int dimT) { // Stampa dei token
   int i;

   if(dimT < 0) // Dimensione negativa
     return BUFFER_TOO_SMALL;

   if(tokenArray==NULL)
     return NULL_POINTER;

   for(i=0; i<dimT; i++) { // Scandisci tutti i token dell'array
     if((strcmp((tokenArray[i]).parola, ""))==0) { // Sono terminati i token significativi
       if(i==0) // Non hai stampato nessun token
	 return UNKNOWN_ERROR;
       else // Hai stampato tutti i token significativi
	 return RESULT_OK;
     }

     if((strcmp((tokenArray[i]).contesto, "NULL"))==0) // Il contesto  NULL
       printf("%s\n", (tokenArray[i]).parola);
     else 
       printf("%s*%s\n", (tokenArray[i]).contesto, (tokenArray[i]).parola);
   }

   return RESULT_OK;
 }

/////////////////////***********************************************************************************////////////////////////////////
/////////////////////////////////**************************END FILE*************************///////////////////////////////////////////
/*************************************************************************************************************************************/



/* 6 APRILE h 19.36 */


